/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AddCounterpointDialog
CLASS TYPE:		GUI - dialog box
DESCRIPTION:	A dialog box that is used to get the type of counterpoint that is to be added to an existing
				cantus firmus. A default counterpoint is generated of the type specified.
				The was_updated method can be used to see if the OK button was pressed (as opposed to the
				Cancel button or the close box).
LIMITATIONS:	Only compatible with species counterpoint, not 4-part harmonies.
				Assumes that the time signature is 4/4.
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


class AddCounterpointDialog extends JDialog
{
	/* FIELDS ******************************************************************/


	private JLabel type_label;		// a label for the type_combo combo box
	private JComboBox type_combo;	// a combo box to store the type of composition
	private boolean update;			// is set to true after the OK button is pressed


	/* CONSTRUCTOR *************************************************************/


	public AddCounterpointDialog(JFrame owner)
	{
		// Give the dialog box its owner, its title and make it modal
		super(owner, "Add Counterpoint to Cantus", true);


		// Initialize the dialog box as not having been updated (to distinguish between ok and cancel)
		update = false;


		// Configure a non-editable combo box to allow the user to choose the type of composition
		type_label = new JLabel("Composition type:");
		type_combo = new JComboBox();
		type_combo.setEditable(false);
		type_combo.addItem("First species, cantus lower voice");
		type_combo.addItem("First species, cantus upper voice");
		type_combo.addItem("Second species, cantus lower voice");
		type_combo.addItem("Second species, cantus upper voice");
		type_combo.addItem("Third species, cantus lower voice");
		type_combo.addItem("Third species, cantus upper voice");
		type_combo.addItem("Fourth species, cantus lower voice");
		type_combo.addItem("Fourth species, cantus upper voice");
		type_combo.addItem("Fifth species, cantus lower voice");
		type_combo.addItem("Fifth species, cantus upper voice");


		// Configure an OK button to close the dialog box
		// Sets update to true so that the calling class can check to see if OK was pressed
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				update = true;
				setVisible(false);
			}
		});


		// Configure a cancel dialog box to close the dialog
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				setVisible(false);
			}
		});


		// Display the dialog box elements and set the size of the dialog box
		Container contentPane = getContentPane();
		JPanel outer_panel = new JPanel();
		JPanel combo_panel = new JPanel();
		combo_panel.add(type_label);
		combo_panel.add(type_combo);
		JPanel button_panel = new JPanel();
		button_panel.add(ok);
		button_panel.add(cancel);
		outer_panel.add(combo_panel, BorderLayout.CENTER);
		outer_panel.add(button_panel, BorderLayout.SOUTH);
		contentPane.add(outer_panel);
		setSize(410, 130);
	}


	/* PUBLIC METHODS **********************************************************/


	public boolean was_updated()
	// Returns true if the OK button was pressed the last time the dialog box was visible
	// Returns false otherwise. Resets the update field to false.
	{
		boolean temp = update;
		update = false;
		return temp;
	}


	/***************************************************************************/


	public Composition get_counterpoint(Composition piece)
	// Takes in a composition that contains only a cantus. Returns a composition that is a duplicate
	// of the original composition, but has a default counterpoint added to it. The type of counterpoint
	// depends on the selection the user has made on this dialog box.
	// NOTE: assumes that time signature is 4/4
	{
		// Determine the type of counterpoint from what is selected in the dialog box
		String choice = (String)type_combo.getSelectedItem();
		int counterpoint_type = 1;
		if (choice.equals("First species, cantus lower voice")) counterpoint_type = 1;
		else if (choice.equals("First species, cantus upper voice")) counterpoint_type = 2;
		else if (choice.equals("Second species, cantus lower voice")) counterpoint_type = 3;
		else if (choice.equals("Second species, cantus upper voice")) counterpoint_type = 4;
		else if (choice.equals("Third species, cantus lower voice")) counterpoint_type = 5;
		else if (choice.equals("Third species, cantus upper voice")) counterpoint_type = 6;
		else if (choice.equals("Fourth species, cantus lower voice")) counterpoint_type = 7;
		else if (choice.equals("Fourth species, cantus upper voice")) counterpoint_type = 8;
		else if (choice.equals("Fifth species, cantus lower voice")) counterpoint_type = 9;
		else if (choice.equals("Fifth species, cantus upper voice")) counterpoint_type = 10;


		// Determine the type of composition from the type of counterpoint
		int composition_type = 1;
		if (counterpoint_type == 1 || counterpoint_type == 2) composition_type = 1;
		else if (counterpoint_type == 3 || counterpoint_type == 4) composition_type = 2;
		else if (counterpoint_type == 5 || counterpoint_type == 6) composition_type = 3;
		else if (counterpoint_type == 7 || counterpoint_type == 8) composition_type = 4;
		else if (counterpoint_type == 9 || counterpoint_type == 10) composition_type = 5;


		// Get information about the composition that was passed to this method
		int quality = piece.get_quality();
		int key = piece.get_key_signature();
		int meter = piece.get_time_signature();
		int bars = piece.get_number_of_bars();


		// Create a new Composition that contains the same cantus as the cantus that was passed
		// to this method
		Voice cantus = piece.get_voice(0);
		piece = new Composition(composition_type, quality, key, meter, bars);
		piece.insert_ready_voice(0, cantus);


		// Insert a default counterpoint into the composition
		piece.insert_voice(1, counterpoint_type, 2);
		Voice counterpoint = piece.get_voice(1);
		for (int i = 0; i < piece.get_total_sixteenth_notes(); i++)
		{
			if (composition_type == 1)
			{
				if (i % meter == 0)
					counterpoint.insert_note(new Note (16, 1, 25), i);
			}
			else if (composition_type == 2 || composition_type == 4)
			{
				if (i < piece.get_total_sixteenth_notes() - meter)
				{
					if (i % (meter/2) == 0)
					counterpoint.insert_note(new Note (8, 1, 25), i);
				}
				else if (i == piece.get_total_sixteenth_notes() - meter)
					counterpoint.insert_note(new Note (16, 1, 25), i);
			}
			else
			{
				if (i < piece.get_total_sixteenth_notes() - meter)
				{
					if (i % (meter/4) == 0)
						counterpoint.insert_note(new Note (4, 1, 25), i);
				}
				else if (i == piece.get_total_sixteenth_notes() - meter)
					counterpoint.insert_note(new Note (16, 1, 25), i);
			}
		}


		// Return the new composition with the added counterpoint
		return piece;
	}
}