/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerHorizontalFifth
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the horizontal aspect of a fifth species counterpoint line have been violated.
				There is one method for each rule. AnalyzerHorizontalFifth objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerHorizontalFifth
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerHorizontalFifth(Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_key()
	// Check to see if the following rule is violated:
	// All notes must belong to the given key, with two exceptions.
	// Exception 1: if the composition is in a minor key, then second to last measure may contain
	//    the raised 7th scale degree.
	// Exception 2: if the composition is in a minor key, then third to last measure may contain
	//    the raised 6th scale degree.
	{
		int note_valid_flag; // indicates whether a note is in the key. 1 if it is, 0 if it isn't.
		int valid_note_1, valid_note_2, valid_note_3, valid_note_4, valid_note_5, valid_note_6,
			valid_note_7; // the notes contained in the given key signature
		int notes_remaining; // the notes remaining in the line after a given note


		// Loop through all of the notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Set default for valid note in case there is a rest or no note is struck
			note_valid_flag = 1;


			// Check to see if there is a note struck on the current beat_unit
			if (counterpoint.get_note(i) != null)
			{
				// Check for notes that are illegal in the data structure
				if (counterpoint.get_note(i).get_pitch() < 0 || counterpoint.get_note(i).get_pitch() > 60)
				{
					note_valid_flag = 0;
				}


				// Deal with notes that aren't rests
				else if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Check to see if note is in the key, considering all octaves
					note_valid_flag = 0;
					for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
												 // note that this will undershoot and overshoot the valid
												 // range of pitches, but this doesn't matter
					{
						valid_note_1 = key_signature + (j * 12);
						valid_note_2 = key_signature + 2 + (j * 12);
						valid_note_3 = key_signature + 4 + (j * 12);
						valid_note_4 = key_signature + 5 + (j * 12);
						valid_note_5 = key_signature + 7 + (j * 12);
						valid_note_6 = key_signature + 9 + (j * 12);
						valid_note_7 = key_signature + 11 + (j * 12);
						if (counterpoint.get_note(i).get_pitch() == valid_note_1 ||
							counterpoint.get_note(i).get_pitch() == valid_note_2 ||
							counterpoint.get_note(i).get_pitch() == valid_note_3 ||
							counterpoint.get_note(i).get_pitch() == valid_note_4 ||
							counterpoint.get_note(i).get_pitch() == valid_note_5 ||
							counterpoint.get_note(i).get_pitch() == valid_note_6 ||
							counterpoint.get_note(i).get_pitch() == valid_note_7)
						{
							note_valid_flag = 1;
							j = 5; // exit the loop if the note is valid
						}
					}


					// Mark raised 6th and 7th scale degrees at end of a minor piece as valid
					if (note_valid_flag != 1 && quality == 2 &&
					    i >= total_sixteenth_notes - (2 * time_signature) &&
					    i < total_sixteenth_notes - time_signature)
					{
						for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
						{
							valid_note_1 = key_signature + 8 + (j * 12);
							if (counterpoint.get_note(i).get_pitch() == valid_note_1)
							{
								note_valid_flag = 1;
								j = 5; // exit the loop if the note is valid
							}
						}
						for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
						{
							valid_note_1 = key_signature + 6 + (j * 12);
							if (counterpoint.get_note(i).get_pitch() == valid_note_1)
							{
								note_valid_flag = 1;
								j = 5; // exit the loop if the note is valid
							}
						}
					}
				}
			}


			// Store an error that has been detected
			if (note_valid_flag != 1)
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" contains a note that does not belong to the key of the piece." +
					" Every note in the piece must belong to the given key, with the exception" +
					" of the raised 6th and 7th scale degrees directly preceding the end of a minor piece.");
		}
	}


	/***************************************************************************/


	public void check_note_durations()
	// Check to see if the following rule is violated:
	// A whole note is the only duration permitted in the last measure of a line and this is the only place
	// where a whole note may be used.
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Detect incorrectly placed whole notes
					if (counterpoint.get_note(i).get_duration() == 16)
					{
						if (i < total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an incorrectly placed whole note. Whole notes are only permitted" +
								" in the last measure of the line.");
						}
					}


					// Detect incorrectly placed half notes
					else if ( counterpoint.get_note(i).get_duration() != 16)
					{
						if (i >= total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a note that is not a whole note. Only whole notes may be used" +
								" in the last measure of a line.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_note_placement()
	// Check to see if the following rules are violated:
	// - All parts of the line must either have a note sounding or must be covered by a rest at all times.
	// - Multiple notes and/or rests may not overlap in a single line.
	{
		int last_duration = 0; // the duration of the previously struck note or rest
		int last_location = 0; // the index location of the previously struck note or rest


		// Make sure that a note or rest is present on the first beat of the line
		if (counterpoint.get_note(0) == null)
			errors_found.insert_new_error("ERROR: There is no note or rest on the first beat" +
				" of the line. The first note of the piece may not be empty.");


		// Check for gaps or overlapping notes
		else
		{
			for (int i = 0; i < total_sixteenth_notes; i++)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (i < last_location + last_duration)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note or rest that is struck while another note or rest is still" +
							" sounding. Multiple notes and/or rests may not overlap in a single line.");
					}
					if (i > last_location + last_duration)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note or rest that is struck too late, resulting in a gap in the line." +
							" All parts of the line must either have a note sounding or must be covered by a" +
							" rest at all times.");
					}

					last_location = i;
					last_duration = counterpoint.get_note(i).get_duration();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_first_measure_rhythmic_configuration()
	// Check to see if the following rule is violated:
	// The first measure must contain one of the following rhythmic configurations: two half notes,
	// a half rest followed by a half note, a quarter rest followed by three quarter notes or four quarter notes.
	// If the measure is ended with a half note, it may be tied or untied.
	// NOTE: cannot differentiate between notes and rests
	{
		RhythmicMeasureType measure = new RhythmicMeasureType(); // the configuration of a measure

		for (int j = 0; j < time_signature; j++)
		{
			if (counterpoint.get_note(j) != null)
				measure.add_note(counterpoint.get_note(j).get_duration(), counterpoint.get_note(j).get_tied());
		}

		if ( measure.get_type_of_measure() != 1 && measure.get_type_of_measure() != 2 &&
			 measure.get_type_of_measure() != 12 )
			errors_found.insert_new_error("ERROR: Measure 1 has an invalid rhythmic" +
				" configuration. Only the following rhythmic configurations are permitted in this" +
				" measure: two half notes, a half rest followed by a half note, a quarter rest followed" +
				" by three quarter notes or four quarter notes. If the measure is ended with a half note, it" +
				" may be tied or untied.");
	}


	/***************************************************************************/


	public void check_valid_rhythms()
	// Check to see if the following rule is violated:
	// Only the following rhytmic configurations are permitted in measures other than the first:
	// Measures with four quarter notes, measures with two half notes with the second not tied,
	// measures with a half note followed by two quarter notes, measures with a half note followed
	// by a quarter note a followed by two eighth notes, measures with a quarter note followed by two
	// eighth notes followed by two quarter notes, measures with three quarter notes followed by two
	// eighth notes, measures with two quarter notes followed by a tied half note, measures with a half
	// note followed by a tied half note, measures with a quarter note followed by two eighth notes
	// followed by a tied half note, measures with a quarter note followed by two eighth notes followed
	// by an untied half note and measures with just a half note.
	// NOTE: cannot differentiate between notes and rests
	{
		RhythmicMeasureType measure; // the configuration of a measure

		for (int i = 1; i < number_of_bars; i++)
		{
			measure = new RhythmicMeasureType();
			
			for (int j = 0; j < time_signature; j++)
			{
				if (counterpoint.get_note( i * time_signature + j ) != null)
					measure.add_note(counterpoint.get_note( i * time_signature + j ).get_duration(),
					                 counterpoint.get_note( i * time_signature + j ).get_tied());
			}

			if (measure.get_type_of_measure() == -1)
				errors_found.insert_new_error("ERROR: Measure " + (i + 1) + " has an invalid rhythmic" +
					" configuration. Only the following rhythmic configurations are permitted in a" +
					" measure: Measures that start with a rest, measures with four quarter notes, measures" +
					" with two half notes with the second not tied, measures with a half note followed by" +
					" two quarter notes, measures with a half note followed by a quarter note a followedd by" +
					" two eighth notes, measures with a quarter note followed by two eighth notes followed by" +
					" two quarter notes, measures with three quarter notes followed by two eighth notes," +
					" measures with two quarter notes followed by a tied half note, measures with a half note" +
					" followed by a tied half note, measures with a quarter note followed by two eighth" +
					" notes followed by a tied half note and measures with just a half note.");
		}
	}


	/***************************************************************************/


	public void check_prolonged_rhythmic_values()
	// Check to see if the following rule is violated:
	// A single rhythmic note value (i.e. all quarter notes or all half notes) should not continue without 
	// variation for more than two measures.
	{
		RhythmicMeasureType measure; // the configuration of a measure
		int last_measure = -1; // the rhythmic configuration of the previous measure
		int this_measure = -1; // the rhythmic configuration of the current measure
		int count = 1; // the number of consecutive measures without rhythmic variation

		for (int i = 0; i < number_of_bars; i++)
		{
			measure = new RhythmicMeasureType();

			for (int j = 0; j < time_signature; j++)
			{
				if (counterpoint.get_note( i * time_signature + j ) != null)
					measure.add_note(counterpoint.get_note( i * time_signature + j ).get_duration(),
					                 counterpoint.get_note( i * time_signature + j ).get_tied());
			}

			last_measure = this_measure;
			this_measure = measure.get_type_of_measure();
			
			if ( last_measure == this_measure &&
				 (this_measure == 1 || this_measure == 2 || this_measure == 12) )
				count++;
			else count = 1;

			if (count >= 3)
				errors_found.insert_new_error("WARNING: Measure " + (i + 1) + " contains number " + count +
					" of a series of consecutive measures without any rhythmic variation." +
					" A single rhythmic note value (i.e. all quarter notes or all half notes) should not continue" +
					" without variation for more than two measures.");
		}
	}


	/***************************************************************************/


	public void check_not_enough_rhythmic_variation()
	// Check to see if the following rule is violated:
	// Not more than roughly a third of a line should consist of measures with only a single type of
	// rhythmic note value (i.e. all quarter notes or all half notes).
	{
		RhythmicMeasureType measure; // the configuration of a measure
		int count = 0; // the number of measures holding a single note value

		for (int i = 0; i < number_of_bars; i++)
		{
			measure = new RhythmicMeasureType();

			for (int j = 0; j < time_signature; j++)
			{
				if (counterpoint.get_note( i * time_signature + j ) != null)
					measure.add_note(counterpoint.get_note( i * time_signature + j ).get_duration(),
					                 counterpoint.get_note( i * time_signature + j ).get_tied());
			}

			if ( measure.get_type_of_measure() == 1 || measure.get_type_of_measure() == 2 )
				count++;
		}	

		if (count > number_of_bars / 3)
			errors_found.insert_new_error("WARNING: This line does not have enough rhythmic variation." +
				" Not more than roughly a third of a line should consist of measures with only a single" +
				" type of rhythmic note value (i.e. all quarter notes or all half notes).");
	}


	/***************************************************************************/


	public void check_change_configuration()
	// Check to see if the following rule is violated:
	// Two consecutive measures with the same rhythmic configuration are not permitted unless they are
	// made up of notes with only a single rhythmic value (i.e. all quarter notes or all half notes).
	{
		RhythmicMeasureType measure; // the configuration of a measure
		int last_measure = -1; // the rhythmic configuration of the previous measure
		int this_measure = -1; // the rhythmic configuration of the current measure

		for (int i = 0; i < number_of_bars; i++)
		{
			measure = new RhythmicMeasureType();

			for (int j = 0; j < time_signature; j++)
			{
				if (counterpoint.get_note( i * time_signature + j ) != null)
					measure.add_note(counterpoint.get_note( i * time_signature + j ).get_duration(),
					                 counterpoint.get_note( i * time_signature + j ).get_tied());
			}

			last_measure = this_measure;
			this_measure = measure.get_type_of_measure();
			
			if ( last_measure == this_measure &&
				 (this_measure != 1 && this_measure != 2 && this_measure != 12) )
				errors_found.insert_new_error("ERROR: Measure " + (i + 1) + " is the second" +
					" of two consecutive measures with the same rhythmic configuration." +
					" This is only permitted when the measures are made up of notes with" +
					" only a single rhythmic value (i.e. all quarter notes or all half notes)," +
					" and this is not the case here.");
		}
	}


	/***************************************************************************/


	public void check_eighth_notes()
	// Check to see if the following rules are violated:
	// Eighth notes must occur two at a time and they must move by step. A leap to the first
	// eighth note is sometimes permitted, but a leap to the second eighth note or from the
	// second eighth note is forbidden.
	{
		int next_eighth = -1; // the location of the note after an eighth note
		int prev_eighths = -1; // the location of the note before a detected eighth note
		int after_eighths = -1; // the location of the note after a pair of eighths

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && counterpoint.get_note(i).get_duration() == 2)
				{
					next_eighth = get_next_note(i, counterpoint);
					if (next_eighth != -1)
					{
						// Check that the second eighth note actually is an eighth note
						if (counterpoint.get_note(next_eighth).get_duration() != 2)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an eighth note that is not followed by another eighth note." +
								" All eighth notes must come in pairs");
						}


						// Check for stepwise motion
						else
						{
							prev_eighths = get_previous_note(i, counterpoint);
							after_eighths = get_next_note(next_eighth, counterpoint);
							if (prev_eighths != -1 && after_eighths != -1)
							{
								if (counterpoint.get_note(prev_eighths).get_pitch() != 0 &&
									counterpoint.get_note(i).get_pitch() != 0 &&
									counterpoint.get_note(next_eighth).get_pitch() != 0 &&
									counterpoint.get_note(after_eighths).get_pitch() != 0)
								{
									// Check that the first eighth note is approached by step
									if ( Math.abs( counterpoint.get_note(prev_eighths).get_pitch() -
									               counterpoint.get_note(i).get_pitch() ) > 2 )
									{
										errors_found.insert_new_error("WARNING: Measure " + 
											((i / time_signature) + 1) + " contains an eighth note that is not" +
											" approached by step. Although the first eighth note of a pair" +
											" of eighth notes can occasionally be leapt to if" +
											" the leap is small, the first eighth note of a pair of eighth" +
											" notes should generally be approached stepwise.");
									}
								
								
									// Check that the two eighth notes are separated by a semitone
									if ( Math.abs( counterpoint.get_note(i).get_pitch() -
									               counterpoint.get_note(next_eighth).get_pitch() ) > 2 )
									{
										errors_found.insert_new_error("ERROR: Measure " + 
											((i / time_signature) + 1) + " contains a pair of eighth notes" +
											" with a leap between them. Eighth notes may only be separated" +
											" by a major or minor second.");
									}
						

									// Check that the second eighth notes is left by step
									if ( Math.abs( counterpoint.get_note(next_eighth).get_pitch() -
									               counterpoint.get_note(after_eighths).get_pitch() ) > 2 )
									{
										errors_found.insert_new_error("ERROR: Measure " + 
											((i / time_signature) + 1) + " contains a pair of eighth notes" +
											" that is not left by step. The second eighth note of a pair" +
											" of eighth notes must always be left by step.");
									}
								}
							}
							i = next_eighth; // to avoid double counting
						}
					}
					else
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains an eighth note that is not followed by another eighth note." +
							" All eighth notes must come in pairs");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_tied_notes_properly_formatted()
	// Check to see if the following rules are violated:
	// - two tied notes must have the same pitch
	// - the duration of a note that is tied ahead must be such that it ends when the note it is tied
	//   to starts
	// - rests may not be tied
	// - a note that is tied ahead must have a note ahead of it to receive the tie
	{
		int pitch = 0; // the pitch of a tied note
		int duration = 0; // the duration of a tied note
		int j = 0; // an iteration variable

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() != 1 && counterpoint.get_note(i).get_tied() != 2)
				{
					pitch = counterpoint.get_note(i).get_pitch();
					duration = counterpoint.get_note(i).get_duration();


					// Check to see if a rest is tied
					if (pitch == 0)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a rest that it tied ahead. Rests may not be tied.");
					}


					// Find the next tied note
					for (j = i + 1; j < total_sixteenth_notes; j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							// Check to see if the following note has a different pitch
							if (counterpoint.get_note(j).get_pitch() != pitch)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead to a note of a different pitch." +
									" Notes may only be tied to notes of the same pitch.");
							}


							// Check to see if the note ends when the note it is tied to starts
							if (duration != j - i)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead but does not end when the note" +
									" that it is tied to starts.");
							}


							// Exit the loop and mark j to tell that a receiving note was found
							j = total_sixteenth_notes + 1;
						}
					}


					// Check to see if a tied note is followed by another note
					if (j == total_sixteenth_notes)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note that is tied ahead but does not have a note ahead" +
							" of it to receive the tie.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_tied_notes()
	// Check to see if the following rule is violated:
	// The same pitch may not be tied twice consecutively.
	{
		int count; // the number of consecutive tied notes

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 4)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" contains a note that is tied both ahead and back. A note may not be" +
						" tied twice consecutively.");
			}
		}
	}


	/***************************************************************************/


	public void check_tied_notes_rhythmically_correct()
	// Check to see if the following rules are violated:
	// Only half notes struck on the third beat of a measure may be tied ahead.
	// NOTE: strong beat is assumed to mean the first beat
	{
		int pitch = 0; // the pitch of a tied note
		int duration = 0; // the duration of a tied note
		int j = 0; // an iteration variable

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 3 || counterpoint.get_note(i).get_tied() == 4)
				{
					if (i % time_signature != 8)
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note that is tied ahead but does not occur on the third beat" +
							" of the measure. Only half notes struck on the third beat of a measure may" +
							" be tied ahead.");

					if (counterpoint.get_note(i).get_duration() != 8)
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note that is tied ahead but is not a half note." +
							" Only half notes struck on the third beat of a measure may be tied ahead.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_rests()
	// Check to see if the following rule is violated:
	// The only rest permitted is either a quarter rest or a half rest on the first beat of the line.
	{
		if (counterpoint.get_note(0) != null)
		{
			if (counterpoint.get_note(0).get_pitch() == 0 &&
				counterpoint.get_note(0).get_duration() != 4 && counterpoint.get_note(0).get_duration() != 8)
				errors_found.insert_new_error("ERROR: The first beat of measure 1 contains a rest that is" +
					" neither a quarter rest nor a half rest. A half rest or a quarter rest on the first beat" +
					" of the line are the only rests permitted.");
		}

		for (int i = 1; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() == 0)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" contains a rest. Rests are not permitted, except for a quarter rest or a half rest on" +
						" the first beat of the line.");
			}
		}
	}


	/***************************************************************************/


	public void check_range()
	// Check to see if the following rule is violated:
	// The range of the line should be between a 5th and a 10th.
	{
		int high_note = -1; // the highest note in the line
		int low_note = 1000; // the lowest note in the line


		// Determine the highest and lowest notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() > high_note && counterpoint.get_note(i).get_pitch() != 0)
					high_note = counterpoint.get_note(i).get_pitch();
				if (counterpoint.get_note(i).get_pitch() < low_note && counterpoint.get_note(i).get_pitch() != 0)
					low_note = counterpoint.get_note(i).get_pitch();
			}
		}


		// Detect errors
		if (high_note == -1 || low_note == 1000)
			errors_found.insert_new_error("ERROR: There are no notes in the counterpoint.");
		else if ((high_note - low_note) < 7)
			errors_found.insert_new_error("WARNING: The total range of the line is less than a 5th." +
				" The total range should be between a 5th and a 10th.");
		else if ((high_note - low_note) > 16)
			errors_found.insert_new_error("WARNING: The total range of the line is greater than a 10th." +
				" The total range should be between a 5th and a 10th.");
	}


	/***************************************************************************/


	public void check_repeated_note()
	// Check to see if the following rule is violated:
	// The same note may not be struck twice in a row unless it is tied.
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() == last_pitch &&
					         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3))
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a repeated note." +
							" The same note may not be struck twice in a row unless it is tied.");
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_horizontal_intervals()
	// Check to see if the following rule is violated:
	// The only permitted horizontal intervals are minor seconds, major seconds, minor thirds, major
	// thirds, perfect forths, perfect fifths, minor sixths, major sixths and octaves. Unisons are
	// also permitted, but only if they are tied (THIS LAST ERROR MESSAGE IS DISABLED. SEE BELOW).
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if ( Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 1 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 2 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 3 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 4 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 5 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 7 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 8 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 9 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 12)
					{
						// Deal with horizontal unisons
						if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) == 0)
						{
							for (int j = i - 1; j >= 0; j--)
							{
								if (counterpoint.get_note(j) != null)
								{
									if (counterpoint.get_note(j).get_tied() != 3 &&
									    counterpoint.get_note(j).get_tied() != 4)
									{
										/* THIS MESSAGE IS DISABLED BECAUSE THE check_repeated_note() METHOD
										   DEALS WITH THIS PROBLEM
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains an untied repeated note." +
											" Notes may only be repeated if they are tied.");
										*/
									}
									j = -1;
								}
							}
						}


						// Deal with illegal horizontal intervals
						else
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains the second note of an illegal horizontal interval." +
								" The only permitted horizontal intervals are unisons, minor seconds, major" +
								" seconds, minor thirds, major thirds, perfect forths, perfect fifths," +
								" minor sixths, major sixths and octaves.");
							last_pitch = counterpoint.get_note(i).get_pitch();
						}
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leading_tone_leap()
	// Check to see if the following rule is violated:
	// You may not leap from the leading tone.
	{
		int leading_tone_flag = 0; // is set to 1 if the previous note was the leading tone, 0 otherwise
		int leading_tone = 0; // stores values of the leading tone in different octaves
		int last_pitch = 0; // stores pitch of previous note

		if (quality == 1)
		{
			for (int i = 0; i < total_sixteenth_notes; i++)
			{
				if (counterpoint.get_note(i) != null)
				{
					if ( counterpoint.get_note(i).get_pitch() != 0  &&
				         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3)  )
					{
						if (leading_tone_flag == 0)
						{
							// Check to see if the current note is the leading tone
							for (int j = 0; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								leading_tone = key_signature - 1 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == leading_tone)
								{
									leading_tone_flag = 1;
									last_pitch = counterpoint.get_note(i).get_pitch();
									j = 5; // exit the loop if the note is valid
								}
							}
						}
						else if (last_pitch - counterpoint.get_note(i).get_pitch() != -1 &&
								last_pitch - counterpoint.get_note(i).get_pitch() != 2)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a note which directly follows the leading tone, but is" +
								" not approached stepwise. You may not leap from the leading tone.");
							leading_tone_flag = 0;
							last_pitch = 0;
						}
						else
						{
							leading_tone_flag = 0;
							last_pitch = 0;
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_a_large_leap()
	// Check to see if the following rule is violated:
	// There should be at least one leap larger than a third in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps larger than a third detected so far
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps == 0)
			errors_found.insert_new_error("WARNING: There are no leaps larger than a third in the line." +
				" There should generally be at least one leap larger than a third.");
	}


	/***************************************************************************/


	public void check_total_number_of_large_leaps()
	// Check to see if the following rule is violated:
	// There should be no more than two leaps larger than a fourth in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps that have been destected so far in the piece
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 5)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps > 2)
			errors_found.insert_new_error("WARNING: There are " + number_of_large_leaps +
				" leaps larger than a fourth in your line." +
				" There should generally be no more than two such leaps in a line.");
	}


	/***************************************************************************/


	public void check_number_consecutive_leaps()
	// Check to see if the following rule is violated:
	// There should not be more than two consecutive leaps in a line.
	{
		int consecutive_leaps = 0; // the number of consecutive leaps
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						consecutive_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						consecutive_leaps = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
			if (consecutive_leaps > 2 && counterpoint.get_note(i) != null)
				errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
					" is involved in an overly long sequence of leaps." +
					" There should not be more than two consecutive leaps.");
		}
	}


	/***************************************************************************/


	public void check_consecutive_leaps_in_same_direction()
	// Check to see if the following rule is violated:
	// Two consecutive leaps in the same direction are not permitted.
	{
		int direction_of_last_leap = 0; // positive for a leap up and negative for a leap down
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Detect two leaps in the same direction
			if (counterpoint.get_note(i) != null)
			{
				if ( counterpoint.get_note(i).get_pitch() != 0 &&
				     (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						if (counterpoint.get_note(i).get_pitch() - last_pitch > 0)
						{
							if (direction_of_last_leap > 0) direction_of_last_leap = 2;
							else direction_of_last_leap = 1;
						}
						else if (counterpoint.get_note(i).get_pitch() - last_pitch < 0)
						{
							if (direction_of_last_leap < 0) direction_of_last_leap = -2;
							else direction_of_last_leap = -1;
						}
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						direction_of_last_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
				
				if (direction_of_last_leap == -2 || direction_of_last_leap == 2)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" is involved in two consecutive leaps in the same direction." +
						" Two consecutive leaps in the same direction are not permitted.");
			}
		}
	}


	/***************************************************************************/


	public void check_leaps_accross_measure_lines()
	// Check to see if the following rule is violated:
	// Leaps across bar lines are only permitted if the leap is a fourth or smaller and
	// is in the direction of motion opposite to the direction of motion preceding  it.
	// NOTE: assumes that a note is struck on the first beat of every measure but
	//       the first
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (i % time_signature == 0 && i != 0)
					{
						int count = 0; // the number of notes encountered
						int j = i - 1; // ends up being the index of the previous notes
						int first_previous = 0; // the pitch of the first note of the leap
						int second_previous = 0; // the pitch of the note preceeding the first note of the leap
						while (count != 2 && j >= 0) // get the index of the first note of the leap
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0 && first_previous == 0)
									first_previous = counterpoint.get_note(j).get_pitch();
								else if (counterpoint.get_note(j).get_pitch() != 0)
									second_previous = counterpoint.get_note(j).get_pitch();
								count++;
							}
							j--;
							if (j < 0) count = 3; // to avoid going out of bounds
						}
						if (count == 2)
						{
							// Check for a leap greater than a fourth
							if (Math.abs(counterpoint.get_note(i).get_pitch() - first_previous) > 5)
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains the second note of a leap greater than a fourth accross a bar line." +
									" Leaps across bar lines are only permitted if the leap is a fourth or smaller.");


							// Check for a leap of a third that does not change direction
							else if (Math.abs(counterpoint.get_note(i).get_pitch() - first_previous) > 2)
							{
								int direction_of_motion = 0; // positive for upwards and negative for downwards
								if (counterpoint.get_note(i).get_pitch() > first_previous)
									direction_of_motion = 1;
								if (counterpoint.get_note(i).get_pitch() < first_previous)
									direction_of_motion = -1;

								if ( (direction_of_motion == 1 && first_previous > second_previous) ||
									 (direction_of_motion == -1 && first_previous < second_previous) )
									errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains the second note of a leap accross a bar line" +
										" that does not change direction. Leaps across bar lines are" +
										" only permitted if the leap is in the direction opposite to" +
										" the direction of motion preceding the leap.");
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leap_filled_in()
	// Check to see if the following rule is violated:
	// Leaps larger than a third should be filled in by the next note.
	{
		int last_pitch = 0; // stores pitch of previous note
		int first_note_of_leap = 0; // the first note of a series of leaps.

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (first_note_of_leap != 0)
					{
						if ( !(counterpoint.get_note(i).get_pitch() > first_note_of_leap &&
								counterpoint.get_note(i).get_pitch() < last_pitch)           &&
							 !(counterpoint.get_note(i).get_pitch() < first_note_of_leap &&
								counterpoint.get_note(i).get_pitch() > last_pitch ))
						{
							errors_found.insert_new_error("WARNING: A note in measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not filled in." +
								" Leaps larger than a third should be filled in by the next note.");
						}
						first_note_of_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						first_note_of_leap = last_pitch;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						first_note_of_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_large_leap_change_direction()
	// Check to see if the following rule is violated:
	// Leaps larger than a fourth should move in the direction opposite to the direction of the
	// motion preceding them
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 5)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch() && direction_of_motion == 1)
						{
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not in the direction opposite to" +
								" the previous motion. Leaps larger than a forth should move in the" +
								" direction opposite to the direction of the motion preceding them.");
							direction_of_motion = 1;
						}
						else if (last_pitch > counterpoint.get_note(i).get_pitch() && direction_of_motion == -1)
						{
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not in the direction opposite to" +
								" the previous motion. Leaps larger than a forth should move in the" +
								" direction opposite to the direction of the motion preceding them.");
							direction_of_motion = -1;
						}
						else if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						direction_of_motion = 1;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						direction_of_motion = -1;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_is_tonic()
	// Check to see if the following rule is violated:
	// The last note of the line must be the first degree of the given scale and it must be on the first
	// beat of the last bar.
	{
		int note_valid_flag = 0; // indicates whether a note is in the key. 1 if it is, 0 if it isn't

		if (counterpoint.get_note((number_of_bars - 1) * time_signature) != null)
		{
			// Check that the note on the first beat of the last bar is the first degree of the scale
			if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch() != 0 && number_of_bars > 0)
			{
				for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
					 // note that this will undershoot and overshoot the valid
					 // range of pitches, but this doesn't matter
				{
					if ( (quality == 1 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature + (j * 12))
					    ||
					     (quality == 2 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature - 3 + (j * 12)))
					{
						note_valid_flag = 1;
						j = 5; // exit the loop if the note is valid
					}
				}
			}

			// Check that there are no notes after this
			if (note_valid_flag == 1)
				if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_duration() != time_signature)
					note_valid_flag = 0;
		}
		if (note_valid_flag != 1)
			errors_found.insert_new_error("ERROR: The last note of the line is not the first degree of the" +
			    " scale and/or it is not on the first beat of the last measure. The last note of the line must be the first" +
			    " degree of the given scale and it must be on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void compare_first_and_last_notes()
	// Check to see if the following rule is violated:
	// If there are no notes lower than the first note of the line, then
	// the last note of the line must have the same pitch as the first note.
	{
		int first_pitch = 0; // the pitch of the first note in the line
		boolean is_lowest = true; // false if there is a note lower than the first note


		// Find out if the first note is the lowest note in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() < first_pitch)
						is_lowest = false;
				}
			}
		}


		// Check the last note in the line
		if (is_lowest == true)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (counterpoint.get_note(i).get_pitch() != first_pitch)
							errors_found.insert_new_error("ERROR: The last note of the line is has a" +
								" different pitch than the first note of the line. If there are no" +
								" notes lower than the first note of the line, then the last note of" +
								" the line must have the same pitch as the first note.");
						i = -1; // exit the loop
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_second_to_last_note()
	// Check to see if the following rule is violated:
	// The second to last note of the line may not be approached by a leap larger than a third.
	{
		int pitch = 0; // the pitch of the second to last note in the line
		int count = 0; // the number of notes found so far

		for (int i = total_sixteenth_notes - 1; i >= 0; i--)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (count == 1)
						pitch = counterpoint.get_note(i).get_pitch();
					else if (count == 2 && Math.abs(counterpoint.get_note(i).get_pitch() - pitch) > 4)
					{
						errors_found.insert_new_error("ERROR: The second to last note of the line is approached" +
							" by a leap larger than a third. This note must be approached by an interval of" +
							" a third or smaller.");
						i = -1;
					}

					count ++;
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_approached_stepwise()
	// Check to see if the following rule is violated:
	// The last note of the line must be approached stepwise.
	{
		int pitch_of_last_note = 0; // the pitch of the last note in the line

		for (int i = total_sixteenth_notes - 1; i >= 0; i--)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (pitch_of_last_note == 0)
						pitch_of_last_note = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - pitch_of_last_note) > 2)
					{
						errors_found.insert_new_error("ERROR: The last note of the line is approached" +
							" by a leap. The last note of the line must be approached stepwise.");
						i = -1; // exit the loop
					}
					else
						i = -1; // exit the loop
				}
			}
		}
	}


	/***************************************************************************/


	public void check_raised_sixth_and_seventh_in_minor()
	// Check to see if the following rules are violated:
	// If the line is in a minor key, and the second to last note is the seventh scale degree, then this
	// note should be raised a semitone.
	// If the above condition is true and the third to last note is the sixth scale degree, then this note
	// should also be raised a semitone.
	{
		int number_of_notes_found = 0; // the number of notes that have been counted so far
		int seventh_was_present = 0; // is set to one if the seventh scale degree is the second to last note. 0 otherwise

		if (quality == 2)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (number_of_notes_found == 0)
							number_of_notes_found = 1;
						else if (number_of_notes_found == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 7 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The second to last note should be raised" +
										" a semitone. If a line is minor and the second to last note is the" +
										" seventh scale degree, then this note should be raised a semitone.");
									seventh_was_present = 1;
								}
								else if (counterpoint.get_note(i).get_pitch() == key_signature + 8 + (j * 12))
									seventh_was_present = 1;
							}
							number_of_notes_found = 2;
						}
						else if (number_of_notes_found == 2 && seventh_was_present == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 5 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The third to last note should be raised" +
										" a semitone. If a line is minor, the second to last note is the" +
										" seventh scale degree and the thrid to last note is the sixth scale" +
										" degree, then the third to last note should be raised a semitone.");
								}
							}
							number_of_notes_found = 3;
						}
						else
							number_of_notes_found++;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_climax()
	// Check to see if the following rules are violated:
	// - Every line must contain a climax. It is usually the highest note of the line, but may be the lowest
	//   point of the line if the first and last notes of the line are the highest points of the line.
	// - The climax note may not be exceeded or repeated anywhere in the line (note that a climax is not
	//   considered to be repeated if it is tied or if it decorated with a neighbour note (NOTE: this means
	//   only that it is repeated after 8 16th notes).
	// - The climax must be on a metrically strong beat (first or third beat)
	// - The climax may not be on the first two or last two notes of the line.
	// - The climax may not be on the leading tone in a major key or the raised seventh scale degree in a
	//   minor key.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note


		// Determine whether the piece is long enough to have a climax
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		if (number_of_notes < 5)
		{
			errors_found.insert_new_error("ERROR: The line is too short to have a climax. It only has " +
				number_of_notes + " notes in it. The climax may not be on the first two or last two notes" +
				" of a line.");
			return;
		}


		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         counterpoint.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		if (high_or_low == 0)
		{
			errors_found.insert_new_error("ERROR: The line does not have a clear climax." +
				" Every line must contain a climax. It is usually the highest note of the line," +
				" but may be the lowest. The climax may not be on the first two or last two notes of" +
				" the line and may not be repeated.");
			return;
		}
		notes_so_far = 0;
		climax_note = 0;


		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && counterpoint.get_note(i).get_pitch() < climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}


		// Determine if the climax is on one of the first two or one of the last two notes
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far < 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" beginning of the line. The climax should not be one of the first two notes of the line.");
						notes_so_far++;
					}
					else if (notes_so_far >= number_of_notes - 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" end of the line. The climax may not be one of the last two notes of the line.");
						notes_so_far++;
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;


 		// Determine if the climax is on a metrically strong beat (first or third beats)
		if (climax_note_position % time_signature != 0 && climax_note_position % time_signature != time_signature / 2)
			errors_found.insert_new_error("ERROR: The climax of the line is on a metrically weak beat." +
				" The climax must fall on the first or third beats of a measure.");


		// Determine if the climax is repeated (although decoration with a neighbour note is permitted)
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (counterpoint.get_note(i).get_pitch() == climax_note && i != climax_note_position
						&& (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3))
					{
						if (i == climax_note_position + 8 )
						{
							if (counterpoint.get_note(climax_note_position).get_duration() != 4)
								errors_found.insert_new_error("ERROR: The climax of the line is repeated." +
									" The climax note may not be exceeded or repeated anywhere in the line" +
									" (although decoration with a neighbour note is permitted).");
						}
						else
							errors_found.insert_new_error("ERROR: The climax of the line is repeated." +
								" The climax note may not be exceeded or repeated anywhere in the line" +
								" (although decoration with a neighbour note is permitted).");
					}
				}
			}
		}


		// Determine if the climax is on the leading tone or raised minor 7th
		for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
		{
			if ( (quality == 1 && climax_note == key_signature - 1 + (j * 12)) ||
			     (quality == 2 && climax_note == key_signature - 4 + (j * 12)) )
				errors_found.insert_new_error("ERROR: The climax of the line is a semi-tone below" +
					" the first scale degree. The climax may not be on the leading tone in a major" +
					" key or the raised seventh scale degree in a minor key.");
		}
	}


	/***************************************************************************/


	public void check_interval_of_series_of_notes()
	// Check to see if the following rule is violated:
	// The interval between the first and last notes of a series of notes moving in the same direction may not be
	// dissonant(i.e. tritone, minor 7th or major 7th).
	{
		int direction_of_movement = 0; // -1 means a series of notes is moving down, 1 means it is moving up,
		                               // 0 means that there is currently no series of notes currently recognized
		int beginning_pitch = 0; // pitch of the first note of a series of notes moving in the same direction
		int index_of_beginning_pitch = 0; // where the above note occurs
		int last_pitch = 0; // pitch of previous note
		int index_of_last_pitch = 0; // the index of the previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
					{
						beginning_pitch = counterpoint.get_note(i).get_pitch();
						index_of_beginning_pitch = i;
					}
					else if (direction_of_movement == 0)
					{
						if (counterpoint.get_note(i).get_pitch() > last_pitch)
							direction_of_movement = 1;
						else if (counterpoint.get_note(i).get_pitch() < last_pitch)
							direction_of_movement = -1;
						else if (counterpoint.get_note(i).get_pitch() == last_pitch)
						{
							direction_of_movement = 0;
							beginning_pitch = counterpoint.get_note(i).get_pitch();
							index_of_beginning_pitch = i;
						}
					}
					else if ( ( direction_of_movement == 1 && counterpoint.get_note(i).get_pitch() < last_pitch ) ||
					          ( direction_of_movement == -1 && counterpoint.get_note(i).get_pitch() > last_pitch ) )
					{
						if (Math.abs(last_pitch - beginning_pitch) == 6 ||
						    Math.abs(last_pitch - beginning_pitch) == 10 ||
						    Math.abs(last_pitch - beginning_pitch) == 11 ||
						    Math.abs(last_pitch - beginning_pitch) == 18)
						{
							errors_found.insert_new_error("WARNING: Measure " +
								((index_of_beginning_pitch / time_signature) + 1) +
								" holds the first note of a series of notes that outline a dissonant interval." +
								" The interval between the first and last notes of a series of notes moving in the same" +
								" direction should not be dissonant (i.e. tritone, minor 7th or major 7th).");
						}
						direction_of_movement = -1 * direction_of_movement;
						beginning_pitch = last_pitch;
						index_of_beginning_pitch = index_of_last_pitch;
					}

					last_pitch = counterpoint.get_note(i).get_pitch();
					index_of_last_pitch = i;
				}
			}
		}
	}


	/***************************************************************************/


	public void check_too_many_in_same_direction()
	// Check to see if the following rule is violated:
	// There should not be more than six consecutive notes moving in the same direction
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of consecutive notes moving in the same direction

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
						count = 2;
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
						count = 0;
					}
					if (count > 6)
					errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
						" holds note number " + count + " of a series of notes moving in the same direction." +
						" There should not be more than six consecutive notes without a change of direction.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_sufficient_changes_of_direction()
	// Check to see if the following rule is violated:
	// The line should change direction at least twice
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of changes of direction counted

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
					}
				}
			}
		}

		if (count < 2)
			errors_found.insert_new_error("WARNING: The line only changes direction " + count +
				" times. The line should change direction at least twice.");
	}


	/***************************************************************************/


	public void check_ornamentation()
	// Check to see if the following rule is violated:
	// Lines may not have ornamentation where the two pitches are repeated immediately (e.g. ABAB or BABA).
	{
		int first_pitch = 0; // the first pitch of the pair
		int second_pitch = 0; // the second pitch of the pair
		int third_pitch = 0; // the pitch being compared to the first pitch
		int forth_pitch = 0; // the pitch being compared to the second pitch

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (second_pitch == 0)
						second_pitch = counterpoint.get_note(i).get_pitch();
					else if (third_pitch == 0)
						third_pitch = counterpoint.get_note(i).get_pitch();
					else
					{
						if (forth_pitch == 0)
							forth_pitch = counterpoint.get_note(i).get_pitch();
						else
						{
							first_pitch = second_pitch;
							second_pitch = third_pitch;
							third_pitch = forth_pitch;
							forth_pitch = counterpoint.get_note(i).get_pitch();
						}

						if (third_pitch == first_pitch && forth_pitch == second_pitch)
						{
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" contains the last note of an inappropriate ornamentation." +
								" Two pitches that are repeated immediately (e.g. ABAB or BABA) should be" +
								" avoided because they make the line seem static.");
						}
					}
				}
			}
		}
	}


	/* PRIVATE METHODS *********************************************************/


	private int get_next_note(int given_index, Voice given_voice)
	// Get the index of the first non-rest note after given_index in given_voice
	// Does not count notes that are tied back
	// Returns -1 if this note cannot be found
	{
		int k = given_index + 1; // the index of the next note
		int count = 0; // the number of notes detected

		while (count != 1 && k < total_sixteenth_notes)
		{
			if (given_voice.get_note(k) != null)
			{
				if (given_voice.get_note(k).get_pitch() != 0 &&
					( given_voice.get_note(k).get_tied() == 1 || given_voice.get_note(k).get_tied() == 3 ))
					count++;
				else k++;
			}
			else k++;
			if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
		}

		if (count == 1)
			return k;
		else
			return -1;
	}


	/***************************************************************************/


	private int get_previous_note(int given_index, Voice given_voice)
	// Get the index of the first non-rest note before given_index in given_voice
	// Does not count notes that are tied back
	// Returns -1 if this note cannot be found
	{
		int k = given_index - 1; // the index of the previous note
		int count = 0; // the number of notes detected

		while (count != 1 && k >= 0)
		{
			if (given_voice.get_note(k) != null)
			{
				if (given_voice.get_note(k).get_pitch() != 0 &&
					( given_voice.get_note(k).get_tied() == 1 || given_voice.get_note(k).get_tied() == 3 ))
					count++;
				else k--;
			}
			else k--;
			if (k < 0) count = 2; // to avoid going out of bounds
		}

		if (count == 1)
			return k;
		else
			return -1;
	}


	/* INTERNAL CLASSES ********************************************************/


	class RhythmicMeasureType
	// Contains the type of rhythm that is held in a measure
	{
		private int[] dur; // the duration of notes in the measure, starting with the first
		private boolean[] tied; // whether or not a note in the measure is tied ahead
		private int number_of_notes; // the number of notes in the measure
		

		public RhythmicMeasureType()
		// Constructor
		// Sets measure to empty
		{
			dur = new int[8];
			tied = new boolean[8];
			for (int i = 0; i < 7; i++)
			{
				dur[i] = 0;
				tied[i] = false;
			}
			number_of_notes = 0;
		}


		public void add_note(int note_duration, int tied_status)
		// Add a duration to the array of durations for the measure. tied_status and note_duration
		// correspond to the codes used by Note objects for ties and durations
		{
			dur[number_of_notes] = note_duration;
			if (tied_status == 3 || tied_status == 4)
				tied[number_of_notes] = true;
			number_of_notes++;
		}


		public int get_type_of_measure()
		// Returns the type of measure, based on the following key:
		// -1 = invalid type
		// 1 = four quarter notes
		// 2 = two half notes with the second NOT tied
		// 3 = half note followed by two quarter notes
		// 4 = half note followed by quarter note folled by two eighth notes
		// 5 = quarter note followed by two eighth notes followed by two quarter notes
		// 6 = three quarter notes followed by two eighth notes
		// 11 = two quarter notes followed by a tied half note
		// 12 = half note followed by a tied half note
		// 13 = quarter note followed by two eighth notes followed by a tied half note
		// 14 = quarter note followed by two eighth notes followed by a tied half note
		// 20 = just a whole note
		// NOTE: cannot differentiate between notes and rests
		{
			if (dur[0] == 4 && dur[1] == 4 && dur[2] == 4 && dur[3] == 4)
				return 1;
			else if (dur[0] == 8 && dur[1] == 8 && tied[1] == false)
				return 2;
			else if (dur[0] == 8 && dur[1] == 4 && dur[2] == 4)
				return 3;
			else if (dur[0] == 8 && dur[1] == 4 && dur[2] == 2 && dur[3] == 2)
				return 4;
			else if (dur[0] == 4 && dur[1] == 2 && dur[2] == 2 && dur[3] == 4 && dur[4] == 4)
				return 5;
			else if (dur[0] == 4 && dur[1] == 4 && dur[2] == 4 && dur[3] == 2 && dur[4] == 2)
				return 6;
			else if (dur[0] == 4 && dur[1] == 4 && dur[2] == 8 && tied[2] == true)
				return 11;
			else if (dur[0] == 8 && dur[1] == 8 && tied[1] == true)
				return 12;
			else if (dur[0] == 4 && dur[1] == 2 && dur[2] == 2 && dur[3] == 8 && tied[3] == true)
				return 13;
			else if (dur[0] == 4 && dur[1] == 2 && dur[2] == 2 && dur[3] == 8 && tied[3] == false)
				return 14;
			else if (dur[0] == 16)
				return 20;
			else
				return -1;
		}
	}
}