/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerHorizontalFirst
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the horizontal aspect of a first species counterpoint line have been violated.
				There is one method for each rule. AnalyzerHorizontalFirst objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerHorizontalFirst
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece


	/* CONSTRUCTOR *************************************************************/


	AnalyzerHorizontalFirst(Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_key()
	// Check to see if the following rule is violated:
	// All notes must belong to the given key, with two exceptions.
	// Exception 1: if the composition is in a minor key, then second to last measure may contain
	//    the raised 7th scale degree.
	// Exception 2: if the composition is in a minor key, then third to last measure may contain
	//    the raised 6th scale degree.
	// NOTE: these two exceptions assume that the line has one whole note per measure
	{
		int note_valid_flag; // indicates whether a note is in the key. 1 if it is, 0 if it isn't.
		int valid_note_1, valid_note_2, valid_note_3, valid_note_4, valid_note_5, valid_note_6,
			valid_note_7; // the notes contained in the given key signature


		// Loop through all of the notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Set default fo valid note in case there is a rest or no note is struck
			note_valid_flag = 1;


			// Check to see if there is a note struck on the current beat_unit
			if (counterpoint.get_note(i) != null)
			{
				// Check for notes that are illegal in the data structure
				if (counterpoint.get_note(i).get_pitch() < 0 || counterpoint.get_note(i).get_pitch() > 60)
				{
					note_valid_flag = 0;
				}


				// Deal with notes that aren't rests
				else if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Check to see if note is in the key, considering all octaves
					note_valid_flag = 0;
					for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
												 // note that this will undershoot and overshoot the valid
												 // range of pitches, but this doesn't matter
					{
						valid_note_1 = key_signature + (j * 12);
						valid_note_2 = key_signature + 2 + (j * 12);
						valid_note_3 = key_signature + 4 + (j * 12);
						valid_note_4 = key_signature + 5 + (j * 12);
						valid_note_5 = key_signature + 7 + (j * 12);
						valid_note_6 = key_signature + 9 + (j * 12);
						valid_note_7 = key_signature + 11 + (j * 12);
						if (counterpoint.get_note(i).get_pitch() == valid_note_1 ||
							counterpoint.get_note(i).get_pitch() == valid_note_2 ||
							counterpoint.get_note(i).get_pitch() == valid_note_3 ||
							counterpoint.get_note(i).get_pitch() == valid_note_4 ||
							counterpoint.get_note(i).get_pitch() == valid_note_5 ||
							counterpoint.get_note(i).get_pitch() == valid_note_6 ||
							counterpoint.get_note(i).get_pitch() == valid_note_7)
						{
							note_valid_flag = 1;
							j = 5; // exit the loop if the note is valid
						}
					}


					// Mark raised 6th and 7th scale degrees at end of a minor piece as valid
					// NOTE: this assumes that the line has one whole note per measure
					if (note_valid_flag != 1 && quality == 2)
					{
						if ( i == total_sixteenth_notes - (2 * time_signature))
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								valid_note_1 = key_signature + 8 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == valid_note_1)
								{
									note_valid_flag = 1;
									j = 5; // exit the loop if the note is valid
								}
							}
						}
						else if ( i == total_sixteenth_notes - (3 * time_signature))
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								valid_note_1 = key_signature + 6 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == valid_note_1)
								{
									note_valid_flag = 1;
									j = 5; // exit the loop if the note is valid
								}
							}
						}
					}
				}
			}


			// Store an error that has been detected
			if (note_valid_flag != 1)
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" contains a note that does not belong to the key of the piece." +
					" Every note in the piece must belong to the given key, with the exception" +
					" of the raised 6th and 7th scale degrees directly preceding the end of a minor piece.");
		}
	}


	/***************************************************************************/


	public void check_note_durations()
	// Check to see if the following rule is violated:
	// Only whole notes should be used.
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_duration() != 16 && counterpoint.get_note(i).get_pitch() != 0)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" contains a note which is not a whole note." +
					" Only whole notes should be used.");
			}
		}
	}


	/***************************************************************************/


	public void check_note_placement()
	// Check to see if the following rule is violated:
	// A note must be struck on the first beat of each measure, and notes may only be struck then.
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if ((i + time_signature) % time_signature == 0)
			{
				if (counterpoint.get_note(i) == null)
				{
					errors_found.insert_new_error("ERROR: No note is struck on the first beat of" +
						" measure " + ((i / time_signature) + 1) + "." +
						" Notes must be struck on the first beat of every measure.");
				}
			}
			else if (counterpoint.get_note(i) != null)
			{
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" contains a note which is not struck on the first beat of the measure." +
					" Notes may only be struck on the first beat of each measure.");
			}
		}
	}


	/***************************************************************************/


	public void check_tied_notes_properly_formatted()
	// Check to see if the following rules are violated:
	// - two tied notes must have the same pitch
	// - the duration of a note that is tied ahead must be such that it ends when the note it is tied
	//   to starts
	// - rests may not be tied
	// - a note that is tied ahead must have a note ahead of it to receive the tie
	{
		int pitch = 0; // the pitch of a tied note
		int duration = 0; // the duration of a tied note
		int j = 0; // an iteration variable

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() != 1 && counterpoint.get_note(i).get_tied() != 2)
				{
					pitch = counterpoint.get_note(i).get_pitch();
					duration = counterpoint.get_note(i).get_duration();


					// Check to see if a rest is tied
					if (pitch == 0)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a rest that it tied ahead. Rests may not be tied.");
					}


					// Find the next tied note
					for (j = i + 1; j < total_sixteenth_notes; j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							// Check to see if the following note has a different pitch
							if (counterpoint.get_note(j).get_pitch() != pitch)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead to a note of a different pitch." +
									" Notes may only be tied to notes of the same pitch.");
							}


							// Check to see if the note ends when the note it is tied to starts
							if (duration != j - i)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead but does not end when the note" +
									" that it is tied to starts.");
							}


							// Exit the loop and mark j to tell that a receiving note was found
							j = total_sixteenth_notes + 1;
						}
					}


					// Check to see if a tied note is followed by another note
					if (j == total_sixteenth_notes)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note that is tied ahead but does not have a note ahead" +
							" of it to receive the tie.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_number_consecutive_tied_notes()
	// Check to see if the following rule is violated:
	// Consecutive ties are not permitted.
	{
		int count; // the number of consecutive tied notes

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() != 1)
				{
					count = 1;

					// Count the number of consecutive ties
					for (int j = i + 1; j < total_sixteenth_notes; j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_tied() == 4)
								count++;
							else
							{
								if (count > 1)
								{
									errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains the first of " + count + " consecutive ties." +
										" Consecutive ties are not permitted.");
								}
								j = total_sixteenth_notes;
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_number_tied_notes()
	// Check to see if the following rule is violated:
	// A line should not contain more than two ties in total.
	{
		int count = 0; // the number of tied notes in the line

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 3 || counterpoint.get_note(i).get_tied() == 4)
					count++;
			}
		}

		if (count > 2)
			errors_found.insert_new_error("WARNING: There are " + count + " notes tied ahead in your line." +
				" There should not be more than two ties in a line.");
	}


	/***************************************************************************/


	public void check_rests()
	// Check to see if the following rule is violated:
	// Rests are not permitted.
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() == 0)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" contains a rest. Rests are not permitted.");
			}
		}
	}


	/***************************************************************************/


	public void check_range()
	// Check to see if the following rule is violated:
	// The range of the line should be between a 5th and a 10th.
	{
		int high_note = -1; // the highest note in the line
		int low_note = 1000; // the lowest note in the line


		// Determine the highest and lowest notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() > high_note && counterpoint.get_note(i).get_pitch() != 0)
					high_note = counterpoint.get_note(i).get_pitch();
				if (counterpoint.get_note(i).get_pitch() < low_note && counterpoint.get_note(i).get_pitch() != 0)
					low_note = counterpoint.get_note(i).get_pitch();
			}
		}


		// Detect errors
		if (high_note == -1 || low_note == 1000)
			errors_found.insert_new_error("ERROR: There are no notes in the counterpoint.");
		else if ((high_note - low_note) < 7)
			errors_found.insert_new_error("WARNING: The total range of the line is less than a 5th." +
				" The total range should be between a 5th and a 10th.");
		else if ((high_note - low_note) > 16)
			errors_found.insert_new_error("WARNING: The total range of the line is greater than a 10th." +
				" The total range should be between a 5th and a 10th.");
	}


	/***************************************************************************/


	public void check_repeated_note()
	// Check to see if the following rule is violated:
	// The same note may not be struck twice in a row unless it is tied.
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() == last_pitch &&
					         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3))
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a repeated note." +
							" The same note may not be struck twice in a row unless it is tied.");
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_horizontal_intervals()
	// Check to see if the following rule is violated:
	// The only permitted horizontal intervals are minor seconds, major seconds, minor thirds, major
	// thirds, perfect forths, perfect fifths, minor sixths, major sixths and octaves. Unisons are
	// also permitted, but only if they are tied (THIS LAST ERROR MESSAGE IS DISABLED. SEE BELOW).
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if ( Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 1 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 2 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 3 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 4 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 5 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 7 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 8 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 9 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 12)
					{
						// Deal with horizontal unisons
						if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) == 0)
						{
							for (int j = i - 1; j >= 0; j--)
							{
								if (counterpoint.get_note(j) != null)
								{
									if (counterpoint.get_note(j).get_tied() != 3 &&
									    counterpoint.get_note(j).get_tied() != 4)
									{
										/* THIS MESSAGE IS DISABLED BECAUSE THE check_repeated_note() METHOD
										   DEALS WITH THIS PROBLEM
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains an untied repeated note." +
											" Notes may only be repeated if they are tied.");
										*/
									}
									j = -1;
								}
							}
						}


						// Deal with illegal horizontal intervals
						else
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains the second note of an illegal horizontal interval." +
								" The only permitted horizontal intervals are unisons, minor seconds, major" +
								" seconds, minor thirds, major thirds, perfect forths, perfect fifths," +
								" minor sixths, major sixths and octaves.");
							last_pitch = counterpoint.get_note(i).get_pitch();
						}
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leading_tone_leap()
	// Check to see if the following rule is violated:
	// You may not leap from the leading tone.
	{
		int leading_tone_flag = 0; // is set to 1 if the previous note was the leading tone, 0 otherwise
		int leading_tone = 0; // stores values of the leading tone in different octaves
		int last_pitch = 0; // stores pitch of previous note

		if (quality == 1)
		{
			for (int i = 0; i < total_sixteenth_notes; i++)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (  counterpoint.get_note(i).get_pitch() != 0  &&
				         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
					{
						if (leading_tone_flag == 0)
						{
							// Check to see if the current note is the leading tone
							for (int j = 0; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								leading_tone = key_signature - 1 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == leading_tone)
								{
									leading_tone_flag = 1;
									last_pitch = counterpoint.get_note(i).get_pitch();
									j = 5; // exit the loop if the note is valid
								}
							}
						}
						else if (last_pitch - counterpoint.get_note(i).get_pitch() != -1 &&
								last_pitch - counterpoint.get_note(i).get_pitch() != 2)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a note which directly follows the leading tone, but is" +
								" not approached stepwise. You may not leap from the leading tone.");
							leading_tone_flag = 0;
							last_pitch = 0;
						}
						else
						{
							leading_tone_flag = 0;
							last_pitch = 0;
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_total_number_of_leaps()
	// Check to see if the following rule is violated:
	// There should be between two to four leaps in the line.
	{
		int number_of_leaps = 0; // the number of leaps that have been destected so far in the piece
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						number_of_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_leaps < 2 || number_of_leaps > 4)
			errors_found.insert_new_error("WARNING: There are " + number_of_leaps +
				" leaps in the line." +
				" There should generally be between two and four leaps in each line.");
	}


	/***************************************************************************/


	public void check_a_large_leap()
	// Check to see if the following rule is violated:
	// There should be at least one leap larger than a third in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps larger than a third detected so far
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps == 0)
			errors_found.insert_new_error("WARNING: There are no leaps larger than a third in the line." +
				" There should generally be at least one leap larger than a third.");
	}


	/***************************************************************************/


	public void check_total_number_of_large_leaps()
	// Check to see if the following rule is violated:
	// There should be no more than two leaps larger than a fourth in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps that have been destected so far in the piece
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 5)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps > 2)
			errors_found.insert_new_error("WARNING: There are " + number_of_large_leaps +
				" leaps larger than a fourth in your line." +
				" There should generally be no more than two such leaps in a line.");
	}


	/***************************************************************************/


	public void check_number_consecutive_leaps()
	// Check to see if the following rule is violated:
	// There should never be more than two consecutive leaps in a line.
	{
		int consecutive_leaps = 0; // the number of consecutive leaps
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						consecutive_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						consecutive_leaps = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
			if (consecutive_leaps > 2 && counterpoint.get_note(i) != null)
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" is involved in an overly long sequence of leaps." +
					" There should never be more than two consecutive leaps in a line.");
		}
	}


	/***************************************************************************/


	public void check_consecutive_leaps_in_same_direction()
	// Check to see if the following rule is violated:
	// Two consecutive leaps in the same direction are not permitted.
	{
		int direction_of_last_leap = 0; // positive for a leap up and negative for a leap down
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						if (counterpoint.get_note(i).get_pitch() - last_pitch > 0)
						{
							if (direction_of_last_leap > 0) direction_of_last_leap = 2;
							else direction_of_last_leap = 1;
						}
						else if (counterpoint.get_note(i).get_pitch() - last_pitch < 0)
						{
							if (direction_of_last_leap < 0) direction_of_last_leap = -2;
							else direction_of_last_leap = -1;
						}
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						direction_of_last_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
			if ((direction_of_last_leap < -1 || direction_of_last_leap > 1) && counterpoint.get_note(i) != null)
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" is involved in multiple consecutive leaps in the same direction." +
					" There should never be two consecutive leaps in the same direction.");
		}
	}


	/***************************************************************************/


	public void check_leap_filled_in()
	// Check to see if the following rule is violated:
	// All leaps larger than a third must be filled in by the next note.
	{
		int last_pitch = 0; // stores pitch of previous note
		int first_note_of_leap = 0; // the first note of a series of leaps.

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (first_note_of_leap != 0)
					{
						if ( !(counterpoint.get_note(i).get_pitch() > first_note_of_leap &&
								counterpoint.get_note(i).get_pitch() < last_pitch)           &&
							 !(counterpoint.get_note(i).get_pitch() < first_note_of_leap &&
								counterpoint.get_note(i).get_pitch() > last_pitch ))
						{
							errors_found.insert_new_error("ERROR: A note in measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not filled in." +
								" All leaps larger than a third must be filled in by the next note.");
						}
						first_note_of_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						first_note_of_leap = last_pitch;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						first_note_of_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_large_leap_change_direction()
	// Check to see if the following rule is violated:
	// Leaps larger than a forth should move in the direction opposite to the direction of the
	// motion preceding them
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 5)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch() && direction_of_motion == 1)
						{
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not in the direction opposite to" +
								" the previous motion. Leaps larger than a forth should move in the" +
								" direction opposite to the direction of the motion preceding them.");
							direction_of_motion = 1;
						}
						else if (last_pitch > counterpoint.get_note(i).get_pitch() && direction_of_motion == -1)
						{
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not in the direction opposite to" +
								" the previous motion. Leaps larger than a forth should move in the" +
								" direction opposite to the direction of the motion preceding them.");
							direction_of_motion = -1;
						}
						else if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						direction_of_motion = 1;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						direction_of_motion = -1;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_change_direction_after_leap()
	// Check to see if the following rule is violated:
	// All leaps larger than a third must be followed by a change of direction.
	// NOTE: The check_leap_filled_in rule will automatically be violated if this rule is violated.
	//       This rule is therefore somewhat redundant, but it is nonetheless included because some
	//       texts cite this rule and not the check_leap_filled_in rule.
	{
		int direction_of_last_leap = 0; // positive for a leap up and negative for a leap down
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						if (counterpoint.get_note(i).get_pitch() - last_pitch > 0)
							direction_of_last_leap++;
						else if (counterpoint.get_note(i).get_pitch() - last_pitch < 0)
							direction_of_last_leap--;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						if (counterpoint.get_note(i).get_pitch() - last_pitch > 0)
							direction_of_last_leap++;
						else if (counterpoint.get_note(i).get_pitch() - last_pitch < 0)
							direction_of_last_leap--;

						if (direction_of_last_leap < -1 || direction_of_last_leap > 1)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" is involved in a large leap that is not followed by a change of direction." +
								" All leaps larger than a third must be followed by a change of direction.");
						}

						direction_of_last_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_is_tonic()
	// Check to see if the following rule is violated:
	// The last note of the line must be the first degree of the given scale and it must be on the first
	// beat of the last bar.
	{
		int note_valid_flag = 0; // indicates whether a note is in the key. 1 if it is, 0 if it isn't

		if (counterpoint.get_note((number_of_bars - 1) * time_signature) != null)
		{
			// Check that the note on the first beat of the last bar is the first degree of the scale
			if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch() != 0 && number_of_bars > 0)
			{
				for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
					 // note that this will undershoot and overshoot the valid
					 // range of pitches, but this doesn't matter
				{
					if ( (quality == 1 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature + (j * 12))
					    ||
					     (quality == 2 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature - 3 + (j * 12)))
					{
						note_valid_flag = 1;
						j = 5; // exit the loop if the note is valid
					}
				}
			}

			// Check that there are no notes after this
			if (note_valid_flag == 1)
				if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_duration() != time_signature)
					note_valid_flag = 0;
		}
		if (note_valid_flag != 1)
			errors_found.insert_new_error("ERROR: The last note of the line is not the first degree of the" +
			    " scale on the first beat of the last measure. The last note of the line must be the first" +
			    " degree of the given scale and it must be on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void compare_first_and_last_notes()
	// Check to see if the following rule is violated:
	// If there are no notes lower than the first note of the line, then
	// the last note of the line must have the same pitch as the first note.
	{
		int first_pitch = 0; // the pitch of the first note in the line
		boolean is_lowest = true; // false if there is a note lower than the first note


		// Find out if the first note is the lowest note in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() < first_pitch)
						is_lowest = false;
				}
			}
		}


		// Check the last note in the line
		if (is_lowest == true)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (counterpoint.get_note(i).get_pitch() != first_pitch)
							errors_found.insert_new_error("ERROR: The last note of the line is has a" +
								" different pitch than the first note of the line. If there are no" +
								" notes lower than the first note of the line, then the last note of" +
								" the line must have the same pitch as the first note.");
						i = -1; // exit the loop
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_second_to_last_note()
	// Check to see if the following rule is violated:
	// The second to last note of the line may not be approached by a leap larger than a third.
	{
		int pitch = 0; // the pitch of the second to last note in the line
		int count = 0; // the number of notes found so far

		for (int i = total_sixteenth_notes - 1; i >= 0; i--)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (count == 1)
						pitch = counterpoint.get_note(i).get_pitch();
					else if (count == 2 && Math.abs(counterpoint.get_note(i).get_pitch() - pitch) > 4)
					{
						errors_found.insert_new_error("ERROR: The second to last note of the line is approached" +
							" by a leap larger than a third. This note must be approached by an interval of" +
							" a third or smaller.");
						i = -1;
					}

					count ++;
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_approached_stepwise()
	// Check to see if the following rule is violated:
	// The last note of the line must be approached stepwise.
	{
		int pitch_of_last_note = 0; // the pitch of the last note in the line

		for (int i = total_sixteenth_notes - 1; i >= 0; i--)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (pitch_of_last_note == 0)
						pitch_of_last_note = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - pitch_of_last_note) > 2)
					{
						errors_found.insert_new_error("ERROR: The last note of the line is approached" +
							" by a leap. The last note of the line must be approached stepwise.");
						i = -1; // exit the loop
					}
					else
						i = -1; // exit the loop
				}
			}
		}
	}


	/***************************************************************************/


	public void check_raised_sixth_and_seventh_in_minor()
	// Check to see if the following rules are violated:
	// If the line is in a minor key, and the second to last note is the seventh scale degree, then this
	// note should be raised a semitone.
	// If the above condition is true and the third to last note is the sixth scale degree, then this note
	// should also be raised a semitone.
	{
		int number_of_notes_found = 0; // the number of notes that have been counted so far
		int seventh_was_present = 0; // is set to one if the seventh scale degree is the second to last note. 0 otherwise

		if (quality == 2)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (number_of_notes_found == 0)
							number_of_notes_found = 1;
						else if (number_of_notes_found == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 7 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The second to last note should be raised" +
										" a semitone. If a line is minor and the second to last note is the" +
										" seventh scale degree, then this note should be raised a semitone.");
									seventh_was_present = 1;
								}
								else if (counterpoint.get_note(i).get_pitch() == key_signature + 8 + (j * 12))
									seventh_was_present = 1;
							}
							number_of_notes_found = 2;
						}
						else if (number_of_notes_found == 2 && seventh_was_present == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 5 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The third to last note should be raised" +
										" a semitone. If a line is minor, the second to last note is the" +
										" seventh scale degree and the thrid to last note is the sixth scale" +
										" degree, then the third to last note should be raised a semitone.");
								}
							}
							number_of_notes_found = 3;
						}
						else
							number_of_notes_found++;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_climax()
	// Check to see if the following rules are violated:
	// - Every line must contain a climax. It is usually the highest note of the line, but may be the lowest
	//   point of the line if the first and last notes of the line are the highest points of the line.
	// - The climax note may not be exceeded or repeated anywhere in the line.
	// - The climax may not be on the first two or last two notes of the line.
	// - The climax may not be on the leading tone in a major key or the raised seventh scale degree in a
	//   minor key.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note


		// Determine whether the piece is long enough to have a climax
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		if (number_of_notes < 5)
		{
			errors_found.insert_new_error("ERROR: The line is too short to have a climax. It only has " +
				number_of_notes + " notes in it. The climax may not be on the first two or last two notes" +
				" of a line.");
			return;
		}


		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         counterpoint.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		if (high_or_low == 0)
		{
			errors_found.insert_new_error("ERROR: The line does not have a clear climax." +
				" Every line must contain a climax. It is usually the highest note of the line," +
				" but may be the lowest. The climax may not be on the first two or last two notes of" +
				" the line and may not be repeated.");
			return;
		}
		notes_so_far = 0;
		climax_note = 0;


		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && counterpoint.get_note(i).get_pitch() < climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}


		// Determine if the climax is on one of the first two or one of the last two notes
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far < 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" beginning of the line. The climax should not be one of the first two notes of the line.");
						notes_so_far++;
					}
					else if (notes_so_far >= number_of_notes - 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" end of the line. The climax may not be one of the last two notes of the line.");
						notes_so_far++;
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;


		// Determine if the climax is repeated
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (counterpoint.get_note(i).get_pitch() == climax_note && i != climax_note_position
						&& (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
						errors_found.insert_new_error("ERROR: The climax of the line is repeated." +
							" The climax note may not be exceeded or repeated anywhere in the line.");
				}
			}
		}


		// Determine if the climax is on the leading tone or raised minor 7th
		for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
		{
			if ( (quality == 1 && climax_note == key_signature - 1 + (j * 12)) ||
			     (quality == 2 && climax_note == key_signature - 4 + (j * 12)) )
				errors_found.insert_new_error("ERROR: The climax of the line is a semi-tone below" +
					" the first scale degree. The climax may not be on the leading tone in a major" +
					" key or the raised seventh scale degree in a minor key.");
		}
	}


	/***************************************************************************/


	public void check_note_occurs_too_often()
	// No note should appear more than four times in the same register.
	{
		int[] pitch_frequencies = new int[61]; // the number of times each pitch occurs in the piece


		// Initialize pitch_frequencies
		for(int i = 0; i < 61; i++)
			pitch_frequencies[i] = 0;


		// Count the number of times each pitch occurs in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					pitch_frequencies[counterpoint.get_note(i).get_pitch()]++;
				}
			}
		}


		// Report errors
		MusicTranslator translator = new MusicTranslator();
		for(int i = 1; i < 61; i++)
		{
			if (pitch_frequencies[i] > 4)
				errors_found.insert_new_error("WARNING: The note " + translator.interperet_pitch(i, key_signature, quality) +
					" is given too much emphasis." +
					" No note should appear more than four times in the same register.");
		}
	}


	/***************************************************************************/


	public void check_note_occurs_too_often_strict()
	// Check to see if the following rules are violated:
	// - No note should appear more than four times in the same register.
	// - A note should not appear more than three times in the same register if it is ever used to change
	//   direction or is involved in a skip.
	// - A note should not appear more than twice in the same register if it is used to change direction or is
	//   involved in a skip more than once.
	// NOTE: This method automatically enforces the check_note_occurs_too_often method. The
	//       check_note_occurs_too_often method should be disabled if this one is enabled.
	{
		int[] pitch_frequencies = new int[61]; // the number of times each pitch occurs in the piece
		int last_pitch = 0; // pitch of the previous note
		int direction_of_line = 0; // -1 means a series of notes is moving down, 1 means it is moving up,
		                           // 0 means that there is currently no series of notes currently recognized
		int prevent_double_count_flag = 0; // stops a note from being double counted for both a skip and a
		                                   // change of direction


		// Initialize pitch_frequencies
		for(int i = 0; i < 61; i++)
			pitch_frequencies[i] = 0;


		// Count the number of times each pitch occurs in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Increment note occurence
					pitch_frequencies[counterpoint.get_note(i).get_pitch()]++;


					// Detect skips
					if (last_pitch == 0)
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (Math.abs(last_pitch - counterpoint.get_note(i).get_pitch()) > 2)
					{
						pitch_frequencies[counterpoint.get_note(i).get_pitch()]++;
						pitch_frequencies[last_pitch]++;
						prevent_double_count_flag = 1;
					}


					// Determine initial direction of movement
					if (last_pitch != 0 && direction_of_line == 0)
					{
						if (counterpoint.get_note(i).get_pitch() > last_pitch)
							direction_of_line = 1;
						else if (counterpoint.get_note(i).get_pitch() < last_pitch)
							direction_of_line = -1;
						else if (counterpoint.get_note(i).get_pitch() == last_pitch)
							direction_of_line = 0;
					}


					// Detect changes of direction
					if (last_pitch != 0)
					{
						if ( ( direction_of_line == 1 && counterpoint.get_note(i).get_pitch() < last_pitch ) ||
					          ( direction_of_line == -1 && counterpoint.get_note(i).get_pitch() > last_pitch ) )
						{
							direction_of_line = -1 * direction_of_line;
							if (prevent_double_count_flag != 1)
								pitch_frequencies[last_pitch]++;
						}
					}


					// Update last_pitch and prevent_double_count_flag
					last_pitch = counterpoint.get_note(i).get_pitch();
					prevent_double_count_flag = 0;
				}
			}
		}


		// Report errors
		MusicTranslator translator = new MusicTranslator();
		for(int i = 1; i < 61; i++)
		{
			if (pitch_frequencies[i] > 4)
				errors_found.insert_new_error("WARNING: The note " + translator.interperet_pitch(i, key_signature, quality) +
					" is given too much emphasis." +
					" No note should appear more than four times in the same register. A note should not appear more" +
					" than three times in the same register if it is used to change direction or is involved in" +
					" a skip. A note should not appear more than twice in the same register if it is used to change" +
					" direction or is involved in a skip more than once.");
		}
	}


	/***************************************************************************/


	public void check_interval_of_series_of_notes()
	// Check to see if the following rule is violated:
	// The interval between the first and last notes of a series of notes moving in the same direction may not be
	// dissonant(i.e. tritone, minor 7th or major 7th).
	{
		int direction_of_movement = 0; // -1 means a series of notes is moving down, 1 means it is moving up,
		                               // 0 means that there is currently no series of notes currently recognized
		int beginning_pitch = 0; // pitch of the first note of a series of notes moving in the same direction
		int index_of_beginning_pitch = 0; // where the above note occurs
		int last_pitch = 0; // pitch of previous note
		int index_of_last_pitch = 0; // the index of the previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
					{
						beginning_pitch = counterpoint.get_note(i).get_pitch();
						index_of_beginning_pitch = i;
					}
					else if (direction_of_movement == 0)
					{
						if (counterpoint.get_note(i).get_pitch() > last_pitch)
							direction_of_movement = 1;
						else if (counterpoint.get_note(i).get_pitch() < last_pitch)
							direction_of_movement = -1;
						else if (counterpoint.get_note(i).get_pitch() == last_pitch)
						{
							direction_of_movement = 0;
							beginning_pitch = counterpoint.get_note(i).get_pitch();
							index_of_beginning_pitch = i;
						}
					}
					else if ( ( direction_of_movement == 1 && counterpoint.get_note(i).get_pitch() < last_pitch ) ||
					          ( direction_of_movement == -1 && counterpoint.get_note(i).get_pitch() > last_pitch ) )
					{
						if (Math.abs(last_pitch - beginning_pitch) == 6 ||
						    Math.abs(last_pitch - beginning_pitch) == 10 ||
						    Math.abs(last_pitch - beginning_pitch) == 11 ||
						    Math.abs(last_pitch - beginning_pitch) == 18)
						{
							errors_found.insert_new_error("WARNING: Measure " +
								((index_of_beginning_pitch / time_signature) + 1) +
								" holds the first note of a series of notes that outline a dissonant interval." +
								" The interval between the first and last notes of a series of notes moving in the same" +
								" direction should not be dissonant (i.e. tritone, minor 7th or major 7th).");
						}
						direction_of_movement = -1 * direction_of_movement;
						beginning_pitch = last_pitch;
						index_of_beginning_pitch = index_of_last_pitch;
					}

					last_pitch = counterpoint.get_note(i).get_pitch();
					index_of_last_pitch = i;
				}
			}
		}
	}


	/***************************************************************************/


	public void check_too_may_stepwise()
	// Check to see if the following rule is violated:
	// There should not be more than five consecutive notes moving stepwise in the same direction.
	{
		int direction_of_line = 0; // -1 means a series of notes is moving down, 1 means it is moving up,
		                           // 0 means that there is currently no series of notes currently recognized
		int last_pitch = 0; // pitch of previous note
		int consecutive_notes = 0; // the number of consecutive stepwise notes

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					// Case for the first note
					if (last_pitch == 0)
						consecutive_notes = 1;


					// Reset if there is a leap
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						consecutive_notes = 1;
						if (counterpoint.get_note(i).get_pitch() > last_pitch)
							direction_of_line = 1;
						else if (counterpoint.get_note(i).get_pitch() < last_pitch)
							direction_of_line = -1;
					}


					// Determine direction of line if there are two stepwise notes
					else if (direction_of_line == 0)
					{
						if (counterpoint.get_note(i).get_pitch() > last_pitch)
						{
							direction_of_line = 1;
							consecutive_notes = 2;
						}
						else if (counterpoint.get_note(i).get_pitch() < last_pitch)
						{
							direction_of_line = -1;
							consecutive_notes = 2;
						}
						else if (counterpoint.get_note(i).get_pitch() == last_pitch)
						{
							consecutive_notes = 1;
							direction_of_line = 0;
						}
					}


					// Reset if there is a change of direction
					else if ( ( direction_of_line == 1 && counterpoint.get_note(i).get_pitch() < last_pitch ) ||
					          ( direction_of_line == -1 && counterpoint.get_note(i).get_pitch() > last_pitch ) )
					{
							consecutive_notes = 2;
							direction_of_line = -1 * direction_of_line;
					}


					// Increment consecutive_notes and find error if present
					else
					{
						consecutive_notes++;
						if (consecutive_notes > 5)
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" contains note number " + consecutive_notes + " of a series of" +
								" consecutive stepwise notes moving in the same direction." +
								" There should not be more than five consecutive notes moving" +
								" stepwise in the same direction.");
					}


					// Update the most recent pitch
					last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_too_many_in_same_direction()
	// Check to see if the following rule is violated:
	// There should not be more than five consecutive notes moving in the same direction
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of consecutive notes moving in the same direction

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
						count = 2;
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
						count = 0;
					}
					if (count > 5)
					errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
						" holds note number " + count + " of a series of notes moving in the same direction." +
						" There should not be more than five consecutive notes without a change of direction.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_sufficient_changes_of_direction()
	// Check to see if the following rule is violated:
	// The line should change direction at least twice
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of changes of direction counted

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
					}
				}
			}
		}

		if (count < 2)
			errors_found.insert_new_error("WARNING: The line only changes direction " + count +
				" times. The line should change direction at least twice.");
	}


	/***************************************************************************/


	public void check_repetition_of_groups_of_tones()
	// Check to see if the following rule is violated:
	// Lines should, in general, not contain more than one occurence of of the same three pitches occuring
	// consecutively in the same order.
	{
		int first_pitch = 0; // the first pitch in the group of three pitches
		int second_pitch = 0; // the second pitch in the group of three pitches
		int third_pitch = 0; // the third pitch in the group of three pitches
		int first_pitch_prime = 0; // the first pitch in the group of three pitches in the second loop
		int second_pitch_prime = 0; // the second pitch in the group of three pitches in the second loop
		int third_pitch_prime = 0; // the third pitch in the group of three pitches in the second loop

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (second_pitch == 0)
						second_pitch = counterpoint.get_note(i).get_pitch();
					else
					{
						if (third_pitch == 0)
							third_pitch = counterpoint.get_note(i).get_pitch();
						else
						{
							first_pitch = second_pitch;
							second_pitch = third_pitch;
							third_pitch = counterpoint.get_note(i).get_pitch();
						}

						for (int j = i + 1; j < total_sixteenth_notes; j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									if (first_pitch_prime == 0)
										first_pitch_prime = counterpoint.get_note(j).get_pitch();
									else if (second_pitch_prime == 0)
										second_pitch_prime = counterpoint.get_note(j).get_pitch();
									else
									{
										if (third_pitch_prime == 0)
											third_pitch_prime = counterpoint.get_note(j).get_pitch();
										else
										{
											first_pitch_prime = second_pitch_prime;
											second_pitch_prime = third_pitch_prime;
											third_pitch_prime = counterpoint.get_note(j).get_pitch();
										}

										if (first_pitch_prime == first_pitch && second_pitch_prime == second_pitch &&
										    	third_pitch_prime == third_pitch)
											errors_found.insert_new_error("WARNING: Measure " + ((j / time_signature) + 1) +
												" is involved in the repetition of a group of three or more pitches." +
												" Lines should not, in general, contain the" +
												" same three pitches occuring consecutively in the same order in more than" +
												" one place in the line.");
									}
								}
							}
						}

						first_pitch_prime = 0;
						second_pitch_prime = 0;
						third_pitch_prime = 0;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_ornamentation()
	// Check to see if the following rule is violated:
	// Lines may not have ornamentation where the two pitches are repeated immediately (e.g. ABAB or BABA).
	{
		int first_pitch = 0; // the first pitch of the pair
		int second_pitch = 0; // the second pitch of the pair
		int third_pitch = 0; // the pitch being compared to the first pitch
		int forth_pitch = 0; // the pitch being compared to the second pitch

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (second_pitch == 0)
						second_pitch = counterpoint.get_note(i).get_pitch();
					else if (third_pitch == 0)
						third_pitch = counterpoint.get_note(i).get_pitch();
					else
					{
						if (forth_pitch == 0)
							forth_pitch = counterpoint.get_note(i).get_pitch();
						else
						{
							first_pitch = second_pitch;
							second_pitch = third_pitch;
							third_pitch = forth_pitch;
							forth_pitch = counterpoint.get_note(i).get_pitch();
						}

						if (third_pitch == first_pitch && forth_pitch == second_pitch)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains the last note of an ornamentation." +
								" Two pitches that are repeated immediately (e.g. ABAB or BABA) are not permitted.");
						}
					}
				}
			}
		}
	}
}