/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerHorizontalForth
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the horizontal aspect of a forth species counterpoint line have been violated.
				There is one method for each rule. AnalyzerHorizontalForth objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerHorizontalForth
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerHorizontalForth(Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_key()
	// Check to see if the following rule is violated:
	// All notes must belong to the given key, with two exceptions.
	// Exception 1: if the composition is in a minor key, then second to last measure may contain
	//    the raised 7th scale degree.
	// Exception 2: if the composition is in a minor key, then third to last measure may contain
	//    the raised 6th scale degree.
	{
		int note_valid_flag; // indicates whether a note is in the key. 1 if it is, 0 if it isn't.
		int valid_note_1, valid_note_2, valid_note_3, valid_note_4, valid_note_5, valid_note_6,
			valid_note_7; // the notes contained in the given key signature
		int notes_remaining; // the notes remaining in the line after a given note


		// Loop through all of the notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Set default for valid note in case there is a rest or no note is struck
			note_valid_flag = 1;


			// Check to see if there is a note struck on the current beat_unit
			if (counterpoint.get_note(i) != null)
			{
				// Check for notes that are illegal in the data structure
				if (counterpoint.get_note(i).get_pitch() < 0 || counterpoint.get_note(i).get_pitch() > 60)
				{
					note_valid_flag = 0;
				}


				// Deal with notes that aren't rests
				else if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Check to see if note is in the key, considering all octaves
					note_valid_flag = 0;
					for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
												 // note that this will undershoot and overshoot the valid
												 // range of pitches, but this doesn't matter
					{
						valid_note_1 = key_signature + (j * 12);
						valid_note_2 = key_signature + 2 + (j * 12);
						valid_note_3 = key_signature + 4 + (j * 12);
						valid_note_4 = key_signature + 5 + (j * 12);
						valid_note_5 = key_signature + 7 + (j * 12);
						valid_note_6 = key_signature + 9 + (j * 12);
						valid_note_7 = key_signature + 11 + (j * 12);
						if (counterpoint.get_note(i).get_pitch() == valid_note_1 ||
							counterpoint.get_note(i).get_pitch() == valid_note_2 ||
							counterpoint.get_note(i).get_pitch() == valid_note_3 ||
							counterpoint.get_note(i).get_pitch() == valid_note_4 ||
							counterpoint.get_note(i).get_pitch() == valid_note_5 ||
							counterpoint.get_note(i).get_pitch() == valid_note_6 ||
							counterpoint.get_note(i).get_pitch() == valid_note_7)
						{
							note_valid_flag = 1;
							j = 5; // exit the loop if the note is valid
						}
					}


					// Mark raised 6th and 7th scale degrees at end of a minor piece as valid
					if (note_valid_flag != 1 && quality == 2)
					{
						// Check to see if the note is the second to las or third to last note
						// by determining how many notes are remaining in the line.
						notes_remaining = 0;
						for (int j = i + 1; j < total_sixteenth_notes; j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
									notes_remaining++;
							}
						}
						if (notes_remaining == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								valid_note_1 = key_signature + 8 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == valid_note_1)
								{
									note_valid_flag = 1;
									j = 5; // exit the loop if the note is valid
								}
							}
						}
						else if (notes_remaining == 2)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								valid_note_1 = key_signature + 6 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == valid_note_1)
								{
									note_valid_flag = 1;
									j = 5; // exit the loop if the note is valid
								}
							}
						}
					}
				}
			}


			// Store an error that has been detected
			if (note_valid_flag != 1)
				errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
					" contains a note that does not belong to the key of the piece." +
					" Every note in the piece must belong to the given key, with the exception" +
					" of the raised 6th and 7th scale degrees directly preceding the end of a minor piece.");
		}
	}


	/***************************************************************************/


	public void check_note_durations()
	// Check to see if the following rule is violated:
	// All notes must be half notes, except for the last note, which must be a whole note and
	// must be struck on the first beat of the last measure.
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					// Detect incorrectly placed whole notes
					if (counterpoint.get_note(i).get_duration() == 16)
					{
						if (i != total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an incorrectly placed whole note. Whole notes are only permitted" +
								" in the last measure of the line.");
						}
					}


					// Detect incorrectly placed half notes
					else if (counterpoint.get_note(i).get_duration() == 8)
					{
						if (i >= total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an incorrectly placed half note. Half notes are not permitted" +
								" in the last measure of the line.");
						}
					}

					
					// Detect notes that are neither whole notes nor half notes
					if (counterpoint.get_note(i).get_duration() != 16 && counterpoint.get_note(i).get_duration() != 8)						
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note which is neither a whole note nor a half note." +
							" Only whole notes and half notes may be used.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_note_placement()
	// Check to see if the following rules are violated:
	// - All parts of the line must either have a note sounding or must be covered by a rest at all times.
	// - Multiple notes and/or rests may not overlap in a single line.
	{
		int last_duration = 0; // the duration of the previously struck note or rest
		int last_location = 0; // the index location of the previously struck note or rest
		

		// Make sure that a note or rest is present on the first beat of the line
		if (counterpoint.get_note(0) == null)
			errors_found.insert_new_error("ERROR: There is no note or rest on the first beat" +
				" of the line. The first note of the piece may not be empty.");


		// Check for gaps or overlapping notes
		else
		{
			for (int i = 0; i < total_sixteenth_notes; i++)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (i < last_location + last_duration)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note or rest that is struck while another note or rest is still" +
							" sounding. Multiple notes and/or rests may not overlap in a single line.");
					}
					if (i > last_location + last_duration)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note or rest that is struck too late, resulting in a gap in the line." +
							" All parts of the line must either have a note sounding or must be covered by a" +
							" rest at all times.");
					}

					last_location = i;
					last_duration = counterpoint.get_note(i).get_duration();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_tied_notes_properly_formatted()
	// Check to see if the following rules are violated:
	// - two tied notes must have the same pitch
	// - the duration of a note that is tied ahead must be such that it ends when the note it is tied
	//   to starts
	// - rests may not be tied
	// - a note that is tied ahead must have a note ahead of it to receive the tie
	{
		int pitch = 0; // the pitch of a tied note
		int duration = 0; // the duration of a tied note
		int j = 0; // an iteration variable

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() != 1 && counterpoint.get_note(i).get_tied() != 2)
				{
					pitch = counterpoint.get_note(i).get_pitch();
					duration = counterpoint.get_note(i).get_duration();


					// Check to see if a rest is tied
					if (pitch == 0)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a rest that it tied ahead. Rests may not be tied.");
					}


					// Find the next tied note
					for (j = i + 1; j < total_sixteenth_notes; j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							// Check to see if the following note has a different pitch
							if (counterpoint.get_note(j).get_pitch() != pitch)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead to a note of a different pitch." +
									" Notes may only be tied to notes of the same pitch.");
							}


							// Check to see if the note ends when the note it is tied to starts
							if (duration != j - i)
							{
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a note that is tied ahead but does not end when the note" +
									" that it is tied to starts.");
							}


							// Exit the loop and mark j to tell that a receiving note was found
							j = total_sixteenth_notes + 1;
						}
					}


					// Check to see if a tied note is followed by another note
					if (j == total_sixteenth_notes)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note that is tied ahead but does not have a note ahead" +
							" of it to receive the tie.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_tied_notes()
	// Check to see if the following rule is violated:
	// The same pitch may not be tied twice consecutively.
	{
		int count; // the number of consecutive tied notes

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 4)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" contains a note that is tied both ahead and back. A note may not be" +
						" tied twice consecutively.");
			}
		}
	}


	/***************************************************************************/


	public void check_tied_notes_properly_placed()
	// Check to see if the following rules are violated:
	// Notes on strong beats may not be tied ahead to notes on weak beats.
	// NOTE: strong beat is assumed to mean the first beat
	{
		int pitch = 0; // the pitch of a tied note
		int duration = 0; // the duration of a tied note
		int j = 0; // an iteration variable

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 3 || counterpoint.get_note(i).get_tied() == 4)
				{
					if (i % time_signature == 0)
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a note on a strong beat that is tied ahead." +
							" Notes on strong beats may not be tied ahead to notes on weak beats.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_times_species_broken()
	// Check to see if the following rules are violated:
	// There should not be more than 8 notes that are not involved in a tie.
	// This is because the species should be predominantly fourth species.
	{
		int count = 0; // the number of notes not invlolved in a tie

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_tied() == 1)
					count++;
			}
		}

		if (count > 8)
			errors_found.insert_new_error("WARNING: Species is broken too often." +
				" A line should generally not contain more than eight notes that" +
				" are not involved in a suspension.");
	}


	/***************************************************************************/


	public void check_consecutive_untied_notes()
	// Check to see if the following rules are violated:
	// There may not be more than 2 consecutive notes that are not involved is suspensions.
	{
		int count = 0; // the number of notes not invlolved in a tie

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && counterpoint.get_note(i).get_tied() == 1)
					count++;
				else count = 0;

				if (count > 2)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +  
						" is involved in a set of " + count + " consecutive" +
						" notes that are not involved in suspensions. Species may not be broken for more" +
						" than two consecutive notes.");
			}
		}
	}


	/***************************************************************************/


	public void check_rests()
	// Check to see if the following rule is violated:
	// There must be a half rest on the first beat of the line and this is the only place where a rest
	// may be present.
	{
		if (counterpoint.get_note(0) != null)
		{
			if (counterpoint.get_note(0).get_pitch() != 0 || counterpoint.get_note(0).get_duration() != 8)
				errors_found.insert_new_error("ERROR: There is not a half rest on the first beat of the" +
					" line. There must be a half rest on the first beat of the line.");
		}
		
		for (int i = 1; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() == 0)
					errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
						" contains a rest. Rests are not permitted, except for a half rest on" +
						" the first beat of the line.");
			}
		}
	}


	/***************************************************************************/


	public void check_range()
	// Check to see if the following rule is violated:
	// The range of the line should be between a 5th and a 10th.
	{
		int high_note = -1; // the highest note in the line
		int low_note = 1000; // the lowest note in the line


		// Determine the highest and lowest notes in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() > high_note && counterpoint.get_note(i).get_pitch() != 0)
					high_note = counterpoint.get_note(i).get_pitch();
				if (counterpoint.get_note(i).get_pitch() < low_note && counterpoint.get_note(i).get_pitch() != 0)
					low_note = counterpoint.get_note(i).get_pitch();
			}
		}


		// Detect errors
		if (high_note == -1 || low_note == 1000)
			errors_found.insert_new_error("ERROR: There are no notes in the counterpoint.");
		else if ((high_note - low_note) < 7)
			errors_found.insert_new_error("WARNING: The total range of the line is less than a 5th." +
				" The total range should be between a 5th and a 10th.");
		else if ((high_note - low_note) > 16)
			errors_found.insert_new_error("WARNING: The total range of the line is greater than a 10th." +
				" The total range should be between a 5th and a 10th.");
	}


	/***************************************************************************/


	public void check_repeated_note()
	// Check to see if the following rule is violated:
	// The same note may not be struck twice in a row unless it is tied.
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() == last_pitch &&
					         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3))
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains a repeated note." +
							" The same note may not be struck twice in a row unless it is tied.");
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_horizontal_intervals()
	// Check to see if the following rule is violated:
	// The only permitted horizontal intervals are minor seconds, major seconds, minor thirds, major
	// thirds, perfect forths, perfect fifths, minor sixths, major sixths and octaves. Unisons are
	// also permitted, but only if they are tied (THIS LAST ERROR MESSAGE IS DISABLED. SEE BELOW).
	{
		int last_pitch = 0; // the pitch of the last note which was struck

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if ( Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 1 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 2 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 3 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 4 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 5 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 7 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 8 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 9 &&
							Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) != 12)
					{
						// Deal with horizontal unisons
						if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) == 0)
						{
							for (int j = i - 1; j >= 0; j--)
							{
								if (counterpoint.get_note(j) != null)
								{
									if (counterpoint.get_note(j).get_tied() != 3 &&
									    counterpoint.get_note(j).get_tied() != 4)
									{
										/* THIS MESSAGE IS DISABLED BECAUSE THE check_repeated_note() METHOD
										   DEALS WITH THIS PROBLEM
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains an untied repeated note." +
											" Notes may only be repeated if they are tied.");
										*/
									}
									j = -1;
								}
							}
						}


						// Deal with illegal horizontal intervals
						else
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains the second note of an illegal horizontal interval." +
								" The only permitted horizontal intervals are unisons, minor seconds, major" +
								" seconds, minor thirds, major thirds, perfect forths, perfect fifths," +
								" minor sixths, major sixths and octaves.");
							last_pitch = counterpoint.get_note(i).get_pitch();
						}
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leading_tone_leap()
	// Check to see if the following rule is violated:
	// You may not leap from the leading tone.
	{
		int leading_tone_flag = 0; // is set to 1 if the previous note was the leading tone, 0 otherwise
		int leading_tone = 0; // stores values of the leading tone in different octaves
		int last_pitch = 0; // stores pitch of previous note

		if (quality == 1)
		{
			for (int i = 0; i < total_sixteenth_notes; i++)
			{
				if (counterpoint.get_note(i) != null)
				{
					if ( counterpoint.get_note(i).get_pitch() != 0  &&
				         (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3)  )
					{
						if (leading_tone_flag == 0)
						{
							// Check to see if the current note is the leading tone
							for (int j = 0; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								leading_tone = key_signature - 1 + (j * 12);
								if (counterpoint.get_note(i).get_pitch() == leading_tone)
								{
									leading_tone_flag = 1;
									last_pitch = counterpoint.get_note(i).get_pitch();
									j = 5; // exit the loop if the note is valid
								}
							}
						}
						else if (last_pitch - counterpoint.get_note(i).get_pitch() != -1 &&
								last_pitch - counterpoint.get_note(i).get_pitch() != 2)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a note which directly follows the leading tone, but is" +
								" not approached stepwise. You may not leap from the leading tone.");
							leading_tone_flag = 0;
							last_pitch = 0;
						}
						else
						{
							leading_tone_flag = 0;
							last_pitch = 0;
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_a_large_leap()
	// Check to see if the following rule is violated:
	// There should be at least one leap larger than a third in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps larger than a third detected so far
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 4)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps == 0)
			errors_found.insert_new_error("WARNING: There are no leaps larger than a third in the line." +
				" There should generally be at least one leap larger than a third.");
	}


	/***************************************************************************/


	public void check_total_number_of_large_leaps()
	// Check to see if the following rule is violated:
	// There should be no more than two leaps larger than a fourth in the line.
	{
		int number_of_large_leaps = 0; // the number of leaps that have been destected so far in the piece
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 5)
					{
						number_of_large_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
						last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
		if (number_of_large_leaps > 2)
			errors_found.insert_new_error("WARNING: There are " + number_of_large_leaps +
				" leaps larger than a fourth in your line." +
				" There should generally be no more than two such leaps in a line.");
	}


	/***************************************************************************/


	public void check_number_consecutive_leaps()
	// Check to see if the following rule is violated:
	// There should not be more than two consecutive leaps in a line.
	{
		int consecutive_leaps = 0; // the number of consecutive leaps
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						consecutive_leaps++;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						consecutive_leaps = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
				}
			}
			if (consecutive_leaps > 2 && counterpoint.get_note(i) != null)
				errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
					" is involved in an overly long sequence of leaps." +
					" There should not be more than two consecutive leaps in a line.");
		}
	}


	/***************************************************************************/


	public void check_consecutive_leaps_in_same_direction()
	// Check to see if the following rule is violated:
	// More than two consecutive leaps in the same direction should be avoided
	{
		int direction_of_last_leap = 0; // positive for a leap up and negative for a leap down
		int last_pitch = 0; // stores pitch of previous note

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Detect two leaps in the same direction
			if (counterpoint.get_note(i) != null)
			{
				if ( counterpoint.get_note(i).get_pitch() != 0  &&
				     (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3)  )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2)
					{
						if (counterpoint.get_note(i).get_pitch() - last_pitch > 0)
						{
							if (direction_of_last_leap > 0) direction_of_last_leap++;
							else direction_of_last_leap = 1;
						}
						else if (counterpoint.get_note(i).get_pitch() - last_pitch < 0)
						{
							if (direction_of_last_leap < 0) direction_of_last_leap--;
							else direction_of_last_leap = -1;
						}
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else
					{
						direction_of_last_leap = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}

					if (direction_of_last_leap < -2 || direction_of_last_leap > 2)
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains a note involved in more than two consecutive leaps in the same" +
							" direction. More than two consecutive leaps in the same direction should be avoided.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leaps_accross_measure_lines()
	// Check to see if the following rule is violated:
	// Leaps across measure lines are only permitted if the leap is a fourth or smaller and the
	// direction of motion is changed immediately after the leap.
	// NOTE: assumes that a note is struck on the first beat of every measure but
	//       the first
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
					counterpoint.get_note(i).get_tied() != 2 && counterpoint.get_note(i).get_tied() != 4)
				{
					if (i % time_signature == 0 && i != 0)
					{
						int count = 0; // the number of notes encountered
						int j = i - 1; // ends up being the index of the previous note
						while (count != 1 && j >= 0) // get the index of the first note of the leap
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
									count++;
							}
							if (count != 1) j--;
							if (j < 0) count = 2; // to avoid going out of bounds
						}
						if (count == 1)
						{
							// Check for a leap greater than a third
							if (Math.abs(counterpoint.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch()) > 5)
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains the second note of a leap greater than a fourth accross a bar line." +
									" Leaps across bar lines are only permitted if the leap is a fourth or smaller and the" +
									" direction of motion is changed immediately after the leap.");
							
							
							// Check for a leap of a third that does not change direction
							else if (Math.abs(counterpoint.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch()) > 2)
							{
								int direction_of_motion = 0; // positive for upwards and negative for downwards
								if (counterpoint.get_note(i).get_pitch() > counterpoint.get_note(j).get_pitch())
									direction_of_motion = 1;
								if (counterpoint.get_note(i).get_pitch() < counterpoint.get_note(j).get_pitch())
									direction_of_motion = -1;
								count = 0;
								int k = i + 1; // ends up being the index of the next note
								while (count != 1 && k < total_sixteenth_notes) // get the index of the first note of the leap
								{
									if (counterpoint.get_note(k) != null)
									{
										if (counterpoint.get_note(k).get_pitch() != 0)
											count++;
									}
									if (count != 1) k++;
									if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
								}
								if (count == 1)
								{
									if ( (counterpoint.get_note(k).get_pitch() > counterpoint.get_note(i).get_pitch()
									        && direction_of_motion == 1) ||
									     (counterpoint.get_note(k).get_pitch() < counterpoint.get_note(i).get_pitch()
									        && direction_of_motion == -1))
									{
										errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains the second note of a leap accross a bar line" +
											" that does not change direction. Leaps across bar lines are" +
											" only permitted if the leap is a fourth or smaller and the" +
											" direction of motion is changed immediately after the leap.");
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_is_tonic()
	// Check to see if the following rule is violated:
	// The last note of the line must be the first degree of the given scale and it must be on the first
	// beat of the last bar.
	{
		int note_valid_flag = 0; // indicates whether a note is in the key. 1 if it is, 0 if it isn't

		if (counterpoint.get_note((number_of_bars - 1) * time_signature) != null)
		{
			// Check that the note on the first beat of the last bar is the first degree of the scale
			if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch() != 0 && number_of_bars > 0)
			{
				for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
					 // note that this will undershoot and overshoot the valid
					 // range of pitches, but this doesn't matter
				{
					if ( (quality == 1 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature + (j * 12))
					    ||
					     (quality == 2 &&
					      counterpoint.get_note((number_of_bars - 1) * time_signature).get_pitch()
						  == key_signature - 3 + (j * 12)))
					{
						note_valid_flag = 1;
						j = 5; // exit the loop if the note is valid
					}
				}
			}

			// Check that there are no notes after this
			if (note_valid_flag == 1)
				if (counterpoint.get_note((number_of_bars - 1) * time_signature).get_duration() != time_signature)
					note_valid_flag = 0;
		}
		if (note_valid_flag != 1)
			errors_found.insert_new_error("ERROR: The last note of the line is not the first degree of the" +
			    " scale and/or it is not on the first beat of the last measure. The last note of the line must be the first" +
			    " degree of the given scale and it must be on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void compare_first_and_last_notes()
	// Check to see if the following rule is violated:
	// If there are no notes lower than the first note of the line, then
	// the last note of the line must have the same pitch as the first note.
	{
		int first_pitch = 0; // the pitch of the first note in the line
		boolean is_lowest = true; // false if there is a note lower than the first note


		// Find out if the first note is the lowest note in the line
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (counterpoint.get_note(i).get_pitch() < first_pitch)
						is_lowest = false;
				}
			}
		}


		// Check the last note in the line
		if (is_lowest == true)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (counterpoint.get_note(i).get_pitch() != first_pitch)
							errors_found.insert_new_error("ERROR: The last note of the line is has a" +
								" different pitch than the first note of the line. If there are no" +
								" notes lower than the first note of the line, then the last note of" +
								" the line must have the same pitch as the first note.");
						i = -1; // exit the loop
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_note_approached_properly()
	// Check to see if the following rule is violated:
	// The last note of the line may not be approached by leap and may not be involved in a suspension.
	{
		int pitch_of_last_note = 0; // the pitch of the last note in the line

		for (int i = total_sixteenth_notes - 1; i >= 0; i--)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (pitch_of_last_note == 0)
						pitch_of_last_note = counterpoint.get_note(i).get_pitch();
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - pitch_of_last_note) > 2)
					{
						errors_found.insert_new_error("ERROR: The last note of the line is approached" +
							" by a leap. The last note of the line must be approached stepwise.");
						i = -1; // exit the loop
					}
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - pitch_of_last_note) == 0)
					{
						errors_found.insert_new_error("ERROR: The last note of the line is tied" +
							" back. The last note of the line may not be involved in a suspension.");
						i = -1; // exit the loop
					}
					else
						i = -1; // exit the loop
				}
			}
		}
	}


	/***************************************************************************/


	public void check_raised_sixth_and_seventh_in_minor()
	// Check to see if the following rules are violated:
	// If the line is in a minor key, and the second to last note is the seventh scale degree, then this
	// note should be raised a semitone.
	// If the above condition is true and the third to last note is the sixth scale degree, then this note
	// should also be raised a semitone.
	{
		int number_of_notes_found = 0; // the number of notes that have been counted so far
		int seventh_was_present = 0; // is set to one if the seventh scale degree is the second to last note. 0 otherwise

		if (quality == 2)
		{
			for (int i = total_sixteenth_notes - 1; i >= 0; i--)
			{
				if (counterpoint.get_note(i) != null)
				{
					if (counterpoint.get_note(i).get_pitch() != 0)
					{
						if (number_of_notes_found == 0)
							number_of_notes_found = 1;
						else if (number_of_notes_found == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 7 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The second to last note should be raised" +
										" a semitone. If a line is minor and the second to last note is the" +
										" seventh scale degree, then this note should be raised a semitone.");
									seventh_was_present = 1;
								}
								else if (counterpoint.get_note(i).get_pitch() == key_signature + 8 + (j * 12))
									seventh_was_present = 1;
							}
							number_of_notes_found = 2;
						}
						else if (number_of_notes_found == 2 && seventh_was_present == 1)
						{
							for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
							{
								if (counterpoint.get_note(i).get_pitch() == key_signature + 5 + (j * 12))
								{
									errors_found.insert_new_error("ERROR: The third to last note should be raised" +
										" a semitone. If a line is minor, the second to last note is the" +
										" seventh scale degree and the thrid to last note is the sixth scale" +
										" degree, then the third to last note should be raised a semitone.");
								}
							}
							number_of_notes_found = 3;
						}
						else
							number_of_notes_found++;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_climax()
	// Check to see if the following rules are violated:
	// - Every line must contain a climax. It is usually the highest note of the line, but may be the lowest
	//   point of the line if the first and last notes of the line are the highest points of the line.
	// - The climax note may not be exceeded or repeated anywhere in the line (note that a climax is not
	//   considered to be repeated if it is tied).
	// - The climax may not be on the first two or last two notes of the line.
	// - The climax may not be on the leading tone in a major key or the raised seventh scale degree in a
	//   minor key.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note


		// Determine whether the piece is long enough to have a climax
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		if (number_of_notes < 5)
		{
			errors_found.insert_new_error("ERROR: The line is too short to have a climax. It only has " +
				number_of_notes + " notes in it. The climax may not be on the first two or last two notes" +
				" of a line.");
			return;
		}


		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         counterpoint.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		if (high_or_low == 0)
		{
			errors_found.insert_new_error("ERROR: The line does not have a clear climax." +
				" Every line must contain a climax. It is usually the highest note of the line," +
				" but may be the lowest. The climax may not be on the first two or last two notes of" +
				" the line and may not be repeated.");
			return;
		}
		notes_so_far = 0;
		climax_note = 0;


		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == 1 && counterpoint.get_note(i).get_pitch() > climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && counterpoint.get_note(i).get_pitch() < climax_note)
					{
						climax_note = counterpoint.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}


		// Determine if the climax is on one of the first two or one of the last two notes
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far < 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" beginning of the line. The climax should not be one of the first two notes of the line.");
						notes_so_far++;
					}
					else if (notes_so_far >= number_of_notes - 2 && i == climax_note_position)
					{
						errors_found.insert_new_error("WARNING: The climax of the line occurs too close to the" +
							" end of the line. The climax may not be one of the last two notes of the line.");
						notes_so_far++;
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;


		// Determine if the climax is repeated
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0)
				{
					if (counterpoint.get_note(i).get_pitch() == climax_note && i != climax_note_position
						&& (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
						errors_found.insert_new_error("ERROR: The climax of the line is repeated." +
							" The climax note may not be exceeded or repeated anywhere in the line.");
				}
			}
		}


		// Determine if the climax is on the leading tone or raised minor 7th
		for (int j = -1; j < 5; j++) // 5 because there are 5 octaves allowed in Note objects
		{
			if ( (quality == 1 && climax_note == key_signature - 1 + (j * 12)) ||
			     (quality == 2 && climax_note == key_signature - 4 + (j * 12)) )
				errors_found.insert_new_error("ERROR: The climax of the line is a semi-tone below" +
					" the first scale degree. The climax may not be on the leading tone in a major" +
					" key or the raised seventh scale degree in a minor key.");
		}
	}


	/***************************************************************************/


	public void check_too_may_stepwise()
	// Check to see if the following rule is violated:
	// There should not be more than six consecutive notes moving stepwise.
	{
		int last_pitch = 0; // pitch of previous note
		int consecutive_notes = 0; // the number of consecutive stepwise notes

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					// Case for the first note
					if (last_pitch == 0)
						consecutive_notes = 1;


					// Reset if there is a leap or a repeated note
					else if (Math.abs(counterpoint.get_note(i).get_pitch() - last_pitch) > 2 ||
					         counterpoint.get_note(i).get_pitch() == last_pitch)
						consecutive_notes = 1;


					// Increment consecutive_notes and find error if present
					else
					{
						consecutive_notes++;
						if (consecutive_notes > 6)
							errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
								" contains note number " + consecutive_notes + " of a series of" +
								" consecutive stepwise notes." +
								" There should not be more than six consecutive notes without a" +
								" leap.");
					}


					// Update the most recent pitch
					last_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_too_many_in_same_direction()
	// Check to see if the following rule is violated:
	// There should not be more than six consecutive notes moving in the same direction
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of consecutive notes moving in the same direction

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
						count = 2;
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						else
							count = 2;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
						count = 0;
					}
					if (count > 6)
					errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
						" holds note number " + count + " of a series of notes moving in the same direction." +
						" There should not be more than six consecutive notes without a change of direction.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_sufficient_changes_of_direction()
	// Check to see if the following rule is violated:
	// The line should change direction at least twice
	{
		int direction_of_motion = 0; // positive for upwards and negative for downwards
		int last_pitch = 0; // stores pitch of previous note
		int count = 0; // the number of changes of direction counted

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (last_pitch == 0)
						last_pitch = counterpoint.get_note(i).get_pitch();
					else if (direction_of_motion == 0)
					{
						if (last_pitch < counterpoint.get_note(i).get_pitch())
							direction_of_motion = 1;
						else if (last_pitch > counterpoint.get_note(i).get_pitch())
							direction_of_motion = -1;
						else
							direction_of_motion = 0;
						last_pitch = counterpoint.get_note(i).get_pitch();
					}
					else if (last_pitch < counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == -1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 1;
					}
					else if (last_pitch > counterpoint.get_note(i).get_pitch())
					{
						if (direction_of_motion == 1)
							count++;
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = -1;
					}
					else
					{
						last_pitch = counterpoint.get_note(i).get_pitch();
						direction_of_motion = 0;
					}
				}
			}
		}

		if (count < 2)
			errors_found.insert_new_error("WARNING: The line only changes direction " + count +
				" times. The line should change direction at least twice.");
	}


	/***************************************************************************/


	public void check_ornamentation()
	// Check to see if the following rule is violated:
	// Lines may not have ornamentation where the two pitches are repeated immediately (e.g. ABAB or BABA).
	{
		int first_pitch = 0; // the first pitch of the pair
		int second_pitch = 0; // the second pitch of the pair
		int third_pitch = 0; // the pitch being compared to the first pitch
		int forth_pitch = 0; // the pitch being compared to the second pitch

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 &&
				    (counterpoint.get_note(i).get_tied() == 1 || counterpoint.get_note(i).get_tied() == 3) )
				{
					if (first_pitch == 0)
						first_pitch = counterpoint.get_note(i).get_pitch();
					else if (second_pitch == 0)
						second_pitch = counterpoint.get_note(i).get_pitch();
					else if (third_pitch == 0)
						third_pitch = counterpoint.get_note(i).get_pitch();
					else
					{
						if (forth_pitch == 0)
							forth_pitch = counterpoint.get_note(i).get_pitch();
						else
						{
							first_pitch = second_pitch;
							second_pitch = third_pitch;
							third_pitch = forth_pitch;
							forth_pitch = counterpoint.get_note(i).get_pitch();
						}

						if (third_pitch == first_pitch && forth_pitch == second_pitch)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains the last note of an ornamentation." +
								" Two pitches that are repeated immediately (e.g. ABAB or BABA) are not permitted.");
						}
					}
				}
			}
		}
	}
}