/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerVerticalFirst
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the vertical aspect of a first species counterpoint line have been violated.
				There is one method for each rule. AnalyzerVerticalFirst objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerVerticalFirst
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private Voice cantus;				// the voice corresponding to the counterpoint to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerVerticalFirst(Voice cant, Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		cantus = cant;
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_vertical_intervals()
	// Check to see if the following rule is violated:
	// May not have vertical intervals other than P1, m3, M3, P5, m6, M6, P8, m10, M10, P12
	// NOTE: assumes that all notes in both lines are struck together
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						difference != 9 && difference != 12 && difference != 15 && difference != 16 &&
						difference != 19)
					{
						errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
							" contains an illegal interval between the cantus and the counterpoint." + 
							" The only permitted vertical intervals are unisons, minor and major thirds," +
							" perfect fifths, minor and major sixths, octaves, minor and major tenths" +
							" and perfect twelfths.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_unisons()
	// Check to see if the following rule is violated:
	// Vertical unisons may only be present in the first and last measures of the piece
	// NOTE: assumes that all notes in both lines are struck together
	// NOTE: assumes that only whole notes are used
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == 0)
					{
						if (i != 0 && i != total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an illegal vertical unison." + 
								" Vertical unisons may only be present in the first and last measures" +
								" of the piece.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_perfect_twelvthes()
	// Check to see if the following rule is violated:
	// Vertical perfect 12ths are only permitted in the climax of the counterpoint, and they may not
	// be leapt to or from a perfect 5th.
	// NOTE: assumes that all notes in both lines are struck together
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == 19)
					{
						// Check if the perfect 12th occurs on the counterpoint's climax
						if (i != get_climax_position(counterpoint))
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a vertical perfect 12th. This measure does not contain the" +
								" climax of the counterpoint. Vertical perfect 12ths are only permitted" +
								" on the climax of the counterpoint.");
						}


						// Check if the perfect 12th is leapt to from a perfect 5th
						for (int j = i - 1; j >= 0; j--)
						{
							if (counterpoint.get_note(j) != null && cantus.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0 && cantus.get_note(j).get_pitch() != 0)
								{
									if (Math.abs(counterpoint.get_note(j).get_pitch() -
										cantus.get_note(j).get_pitch()) == 7)
									{
										errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains a vertical perfect 12th that is leapt to from a vertical" +
											" perfect 5th. Perfect 12ths may not be approached from perfect 5ths.");
									}
									j = -1;
								}
							}
						}


						// Check if the perfect 12th is followed by a perfect fifth
						for (int j = i + 1; j < total_sixteenth_notes; j++)
						{
							if (counterpoint.get_note(j) != null && cantus.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0 && cantus.get_note(j).get_pitch() != 0)
								{
									if (Math.abs(counterpoint.get_note(j).get_pitch() -
										cantus.get_note(j).get_pitch()) == 7)
									{
										errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains a vertical perfect 12th that is followed by a vertical" +
											" perfect 5th. Perfect 12ths may not be followed by perfect 5ths.");
									}
									j = total_sixteenth_notes;
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_perfect_approach()
	// Check to see if the following rule is violated:
	// Perfect intervals must be approached with contrary or oblique motion.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								prev_counterpoint = 0;
								prev_cantus = 0;

								
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									// Find the previous counterpoint note
									for (int k = j - 1; k >= 0; k--)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
											{
												prev_counterpoint = counterpoint.get_note(k).get_pitch();
												k = -1;
											}
										}
									}
								
								
									// Find the previous cantus note
									for (int k = j - 1; k >= 0; k--)
									{
										if (cantus.get_note(k) != null)
										{
											if (cantus.get_note(k).get_pitch() != 0)
											{
												prev_cantus = cantus.get_note(k).get_pitch();
												k = -1;
											}
										}
									}


									// Generate an error if the motion isn't oblique or consonant
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										if (get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'c' &&
											get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'o')
										{
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains a perfect interval that is not approached with the proper type of" +
												" motion. Perfect intervals must be approached with oblique or contrary motion.");
										}
									}
								}
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_successive_unison_octave()
	// Check to see if the following rule is violated:
	// Cannot move from a vertical unison to an octave or from an octave to a unison.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Detect unisons or octaves intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 12)
								{
									// Find the previous counterpoint note
									for (int k = j - 1; k >= 0; k--)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
											{
												prev_counterpoint = counterpoint.get_note(k).get_pitch();
												k = -1;
											}
										}
									}
								
								
									// Find the previous cantus note
									for (int k = j - 1; k >= 0; k--)
									{
										if (cantus.get_note(k) != null)
										{
											if (cantus.get_note(k).get_pitch() != 0)
											{
												prev_cantus = cantus.get_note(k).get_pitch();
												k = -1;
											}
										}
									}


									// Generate an error if appropriate
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										if (difference == 0 && Math.abs(prev_cantus - prev_counterpoint) == 12)
										{
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains a vertical unison that is preceded by an octave." +
												" It is not permitted to move from a vertical octave to" +
												" a unison.");
										}
										if (difference == 12 &&	Math.abs(prev_cantus - prev_counterpoint) == 0)
										{
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains a vertical octave that is preceded by a unison." +
												" It is not permitted to move from a vertical unison to" +
												" an octave.");
										}
									}
								}
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_vertical_intervals()
	// Check to see if the following rule is violated:
	// Cannot have more than three consecutive identical vertical intervals (e.g. four thirds in a row).
	// NOTE: assumes that all notes in both lines are struck together
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int last_interval = 0; // the difference of the last pair of notes which were compared
		int count = 0; // the number of times that an interval has occured consecutively

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == last_interval) count++;
					else count = 0;
					last_interval = difference;
					
					if (count > 2)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains number " + (count + 1) + " of a series of consecutive identical" +
							" vertical intervals. Only up to three consecutive identical vertical" +
							" intervals are permitted.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_parallel()
	// Check to see if the following rule is violated:
	// May not have more than three pairs of notes in a row that move using parallel motion
	// NOTE: assumes that all notes in both lines are struck together
	// NOTE: the definition of parallel motion used here includes only exactly parallel motion.
	//       Movement from a minor third to a major third, for example, is considered similar.
	//       This diverges from some definitions of parallel motion.
	{
		int count = 0; // the number of parallel motions so far
		boolean first_pair_found = false; // whether or not the first pair of notes have been found
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					if (first_pair_found == false)
					{
						prev_cantus = cantus.get_note(i).get_pitch();
						prev_counterpoint = counterpoint.get_note(i).get_pitch();
						first_pair_found = true;
					}
					else
					{
						if (get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
								counterpoint.get_note(i).get_pitch()) == 'p')
							count++;
						else
							count = 0;

						prev_cantus = cantus.get_note(i).get_pitch();
						prev_counterpoint = counterpoint.get_note(i).get_pitch();
					}

					if (count > 2)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" containts number " + (count + 1) + " of a series of parallel motions. There should not be" +
							" more than three pairs of notes in a row that move using parallel motion.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leaps_from_perfect()
	// Check to see if the following rule is violated:
	// Do not leap to or from notes in the counterpoint that are involved in perfect intervals.
	// NOTE: assumes that all notes in both lines are struck together
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int pitch; // the pitch of the counterpoint where there is a perfect interval with the cantus

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					pitch = counterpoint.get_note(i).get_pitch();
					if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
					{
						// Check if the perfect interval was leapt to
						for (int j = i - 1; j >= 0; j--)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (Math.abs(counterpoint.get_note(j).get_pitch() - pitch) > 2)
								{
									errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
										" contains a vertical perfect interval and it is approached by a" +
										" leap in the counterpoint. If there is a perfect interval between" +
										" the cantus and the counterpoint, then the note in the counterpoint" +
										" should be approached and left by step.");
								}
								j = -1;
							}
						}


						// Check if the perfect interval was leapt from
						for (int j = i + 1; j < total_sixteenth_notes; j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (Math.abs(counterpoint.get_note(j).get_pitch() - pitch) > 2)
								{
									errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
										" contains a vertical perfect interval and it is left by a" +
										" leap in the counterpoint. If there is a perfect interval between" +
										" the cantus and the counterpoint, then the note in the counterpoint" +
										" should be approached and left by step.");
								}
								j = total_sixteenth_notes;
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_horizontal_leap()
	// Check to see if the following rule is violated:
	// If there is a horizontal leap in one line, then the other line should move stepwise if
	// i)  the leap is greater than a forth OR
	// ii) the lines are moving using similar motion
	// NOTE: Assumes that notes are struck on first beat of both cantus and counterpoint
	{
		int last_cantus_pitch = -1; // the last pitch of the cantus
		int last_counterpoint_pitch = -1; // the last pitch in the counterpoint
		int initialized = 0; // will be set to 2 when last_cantus_pitch and last_counterpoint_pitch
		                     // have both been initialized

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			// Initialize last_cantus_pitch and last_counterpoint_pitch
			if (cantus.get_note(i) != null)
			{
				if (last_cantus_pitch == -1 && cantus.get_note(i).get_pitch() != 0)
				{
					last_cantus_pitch = cantus.get_note(i).get_pitch();
					initialized++;
				}
			}
			if (counterpoint.get_note(i) != null)
			{
				if (last_counterpoint_pitch == -1 && counterpoint.get_note(i).get_pitch() != 0)
				{
					last_counterpoint_pitch = counterpoint.get_note(i).get_pitch();
					initialized++;
				}
			}


			// Check if both lines leap
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null && initialized == 2)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					if (Math.abs(counterpoint.get_note(i).get_pitch() - last_counterpoint_pitch) > 2 &&
					    Math.abs(cantus.get_note(i).get_pitch() - last_cantus_pitch) > 2)
					{
						if (Math.abs(counterpoint.get_note(i).get_pitch() - last_counterpoint_pitch) > 5 ||
					        Math.abs(cantus.get_note(i).get_pitch() - last_cantus_pitch) > 5)
						{
							errors_found.insert_new_error("WARNING: There is an improper leap just before" +
								" measure " + ((i / time_signature) + 1) + ". If one line leaps more than" +
								" a forth then the other line should move stepwise.");
						}

						if (get_type_of_motion(last_cantus_pitch, cantus.get_note(i).get_pitch(),
						                       last_counterpoint_pitch, counterpoint.get_note(i).get_pitch())
						    == 's' ||
							get_type_of_motion(last_cantus_pitch, cantus.get_note(i).get_pitch(),
						                       last_counterpoint_pitch, counterpoint.get_note(i).get_pitch())
						    == 'p')
						{
							errors_found.insert_new_error("WARNING: There is an improper leap just before" +
								" measure " + ((i / time_signature) + 1) + ". If the lines move using" +
								" similar motion, then at least one line should move stepwise.");
						}
					}
					
					last_cantus_pitch = cantus.get_note(i).get_pitch();
					last_counterpoint_pitch = counterpoint.get_note(i).get_pitch();
				}
			}
		}
	}


	/***************************************************************************/


	public void check_simultaneous_climaxes()
	// Check to see if the following rule is violated:
	// The climax may not occur at the same time in both the cantus and the counterpoint
	{
		if (get_climax_position(cantus) == get_climax_position(counterpoint))
			errors_found.insert_new_error("WARNING: Both the cantus firmus and the counterpoint have" +
				" their climaxes occuring in the same measure. The lines" +
				" should have their climaxes occuring in different measures.");
	}


	/***************************************************************************/


	public void check_voice_crossing()
	// Check to see if the following rule is violated:
	// A note in the bottom line may never be above the corresponding note of the top line and vice versa
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Deal with case where cantus is the upper voice
								if ( counterpoint.get_type_of_line() % 2 == 0 )
								{
									if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() < 0 )
										errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
											((i / time_signature) + 1) + ". A note in the bottom line may never be" +
											" above the corresponding note of the top line and vice versa.");
								}
								
								
								// Deal with case where cantus is lower voice
								else if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() > 0 )
									errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
										((i / time_signature) + 1) + ". A note in the bottom line may never be" +
										" above the corresponding note of the top line and vice versa.");
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_voice_overlap()
	// Check to see if the following rule is violated:
	// A note in the bottom line cannot move up past where the last note of the top line was just
	// previously and vice versa
	{
		int prev_cantus = -1; // the previous note of the cantus
		int prev_counterpoint = -1; // the previous note of the counerpoint
		boolean found_one = false; // set to true when the first counterpoint note in a measure is found
		boolean error_found = false; // set to true if voice overlap is found

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			error_found = false;
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					if (prev_cantus == -1)
					{
						prev_cantus = cantus.get_note(i).get_pitch();
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}		
					}
					else
					{
						// Check cantus for voice overlap in case where cantus is the upper voice
						if ( counterpoint.get_type_of_line() % 2 == 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint < 0 )
								error_found = true;
						}


						// Check cantus for voice overlap in case where cantus is the lower voice
						else if ( counterpoint.get_type_of_line() % 2 != 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint > 0 )
								error_found = true;
						}


						// Check counterpoint for voice overlap
						found_one = false;
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									if (found_one == false)
									{
										// Check counterpoint for voice overlap in case where cantus is the upper voice
										if ( counterpoint.get_type_of_line() % 2 == 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus > 0 )
												error_found = true;
										}


										// Check counterpoint for voice overlap in case where cantus is the lower voice
										else if ( counterpoint.get_type_of_line() % 2 != 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus < 0 )
												error_found = true;
										}
									}
									found_one = true;
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}
						prev_cantus = cantus.get_note(i).get_pitch();
					}
				}
			}
			
			if (error_found == true)
				errors_found.insert_new_error("ERROR: There is voice overlap in measure " + ((i / time_signature) + 1) + 
					".  A note in the bottom line cannot move up past where the last note of the top line was just previously" +
					" and vice versa.");
		}
	}


	/***************************************************************************/

	
	public void check_beginning_interval()
	// Check to see if the following rule is violated:
	// The first vertical interval must be a P1, P5 or P8. A P5 is only allowed when the counterpoint is
	// above the cantus.
	{
		int cantus_pitch = -1; // the first pitch in the cantus
		int counterpoint_pitch = -1; // the first pitch in the counterpoint
		
		
		// Find the first vertical interval
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					cantus_pitch = cantus.get_note(i).get_pitch();

					for (int j = 0; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								counterpoint_pitch = counterpoint.get_note(j).get_pitch();
								j = i + cantus.get_note(i).get_duration();
							}
						}
					}

					i = total_sixteenth_notes;
				}
			}
		}
		int interval = counterpoint_pitch - cantus_pitch; // the interval between the first two notes


		// Detect a missign first inteval
		if (cantus_pitch == -1 || counterpoint_pitch == -1)
			errors_found.insert_new_error("ERROR: The line has an improper first interval.");


		// Detect errors when the cantus is the upper voice
		else if ( counterpoint.get_type_of_line() % 2 == 0 )
		{
			if (interval != 0 && interval != -12)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the upper voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or an octave below it.");
			}
		}


		// Detect errors when the cantus is the lower voice
		else if ( counterpoint.get_type_of_line() % 2 != 0 )
		{
			if (interval != 0 && interval != 12 && interval != 7)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the lower voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or a fifth or an octave above it.");
			}
		}
	}


	/***************************************************************************/


	public void check_last_interval()
	// Check to see if the following rule is violated:
	// The last vertical interval in the piece must be a P1 or a P8.
	// NOTE: assumes that all notes in both lines are struck together
	{
		int difference; // the interval between the two voices

		for (int i = total_sixteenth_notes - 1; i >= 0 ; i--)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference != 0 && difference != 12)
						errors_found.insert_new_error("ERROR: The last vertical interval is incorrect." +
							" It must be either a unison or an octave.");
					i = -1;
				}
			}
		}
	}


	/***************************************************************************/


	public void check_last_approach()
	// Check to see if the following rule is violated:
	// The last interval must be approached with contrary motion.
	// NOTE: assumes that all notes in both lines are struck together
	{
		int cantus_last = -1; // the last note in the cantus
		int counterpoint_last = -1; // the last note in the counterpoint
		int count = 0; // the number of note pairs that have been read

		for (int i = total_sixteenth_notes - 1; i >= 0 ; i--)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					if (cantus_last == -1 && counterpoint_last == -1)
					{
						cantus_last = cantus.get_note(i).get_pitch();
						counterpoint_last = counterpoint.get_note(i).get_pitch();
					}
					else if (get_type_of_motion(cantus.get_note(i).get_pitch(), cantus_last,
								counterpoint.get_note(i).get_pitch(), counterpoint_last) != 'c')
						errors_found.insert_new_error("ERROR: The last vertical interval is approached" +
							" incorrectly. It must be approached with contrary motion.");
					count++;
					if (count == 2)
						i = -1;
				}
			}
		}
	}


	/* PRIVATE METHODS *********************************************************/


	private int get_number_notes(Voice voice_to_check)
	// Return the number of non-rest notes struck in the given Voice
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		return number_of_notes;
	}


	/***************************************************************************/


	private int get_climax_position(Voice voice_to_check)
	// Return the position of the climax in the voice_to_check voice. If the climax is repeated, the position
	// of its first occurence is returned. The climax is found to be the highest note of the line or, if the
	// first and last notes of the line are the hightes, the lowest note of the line.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note
		

		// Determine whether the number of notes in the line
		number_of_notes = get_number_notes(voice_to_check);
		
		
		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && voice_to_check.get_note(i).get_pitch() >= climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         voice_to_check.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;
		climax_note = 0;
		
		
		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;					
					}
					else if (high_or_low == 1 && voice_to_check.get_note(i).get_pitch() > climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && voice_to_check.get_note(i).get_pitch() < climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}
		
	
		// Return the position of the climax
		return climax_note_position;
	}


	/***************************************************************************/


	private char get_type_of_motion(int v1_first_pitch, int v1_second_pitch,
	                                int v2_first_pitch, int v2_second_pitch)
	// Return whether the motion from pitch v1_first_pitch to v1_second_pitch in one voice
	// and from v2_first_pitch to v2_second_pitch in the other voice is similar, parallel,
	// contrary or oblique. 's' means similar, 'p' means parallel, 'c' means contrary and
	// 'o' means oblique.
	// NOTE: the definition of parallel motion used here includes only exactly parallel motion.
	//       Movement from a minor third to a major third, for example, is considered similar.
	//       This diverges from some definitions of parallel motion.
	{
		if (v1_first_pitch == v1_second_pitch || v2_first_pitch == v2_second_pitch)
			return 'o';
		else if ( (v1_first_pitch - v1_second_pitch > 0 && v2_first_pitch - v2_second_pitch < 0) ||
		          (v1_first_pitch - v1_second_pitch < 0 && v2_first_pitch - v2_second_pitch > 0) )
			return 'c';
		else if (v1_first_pitch - v2_first_pitch == v1_second_pitch - v2_second_pitch)
			return 'p';
		else
			return 's';
	}
}