/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerVerticalForth
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the vertical aspect of a forth species counterpoint line have been violated.
				There is one method for each rule. AnalyzerVerticalForth objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerVerticalForth
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private Voice cantus;				// the voice corresponding to the counterpoint to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerVerticalForth(Voice cant, Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		cantus = cant;
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_vertical_intervals()
	// Check to see if the following rule is violated:
	// Vertical intervals other than P1, m3, M3, P5, m6, M6, P8 and their multiples are only
	// permitted on:
	// - weak beats, when they are not part of suspensions and are approached and left with stepwise
	//   motion
	// - strong beats, when they are part of a suspension
	// NOTE: assumes that "weak beats" means the third beat
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for illegal vertical intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() - 
					                      cantus.get_note(cantus_location).get_pitch());
					difference = difference % 12;


					// Check if the interval is dissonant
					if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						difference != 9)
					{
						// If a note is on a strong beat
						if (i % time_signature == 0)
						{
							// If is not part of a suspension
							if (counterpoint.get_note(counterpoint_location).get_tied() != 2 && 
								counterpoint.get_note(counterpoint_location).get_tied() != 4)
							{
								errors_found.insert_new_error("ERROR: The first beat of measure " + ((i / time_signature) + 1) +
									" contains an illegal interval between the cantus and the counterpoint." + 
									" Vertical intervals other than unisons, minor and major thirds, perfect" +
									" fifths, minor and major sixths, octaves and the multiples of these" +
									" intervals are only permitted on strong beats when they are part of" +
									" a suspension.");
							}
						}

						
						// If a note is on a weak beat
						else 
						{
							int count = 0; // the number of notes detected
							int k = counterpoint_location - 1;
							while (count != 1 && k >= 0)
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k--;
								}
								else k--;
								if (k < 0) count = 2; // to avoid going out of bounds
							}
							if (count == 1)
							{
								// Check if note is part of a suspension
								if (counterpoint.get_note(k).get_tied() != 1 ||	counterpoint.get_note(counterpoint_location).get_tied() != 1)
								{
									errors_found.insert_new_error("ERROR: The weak beat of measure " + ((i / time_signature) + 1) +
										" contains an illegal interval between the cantus and the counterpoint." + 
										" Vertical intervals other than unisons, minor and major thirds, perfect" +
										" fifths, minor and major sixths, octaves and the multiples of these" +
										" intervals are only permitted on weak beats when they are not invovled" +
										" in any part of a suspension.");
								}

								
								// Check if a note is not approached and left stepwise if it is not part of a suspension
								else
								{
									count = 0; // the number of notes detected
									k = counterpoint_location + 1;
									while (count != 1 && k < total_sixteenth_notes)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
												count++;
											else k++;
										}
										else k++;
										if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
									}
									if (count == 1)
									{
										if (Math.abs( counterpoint.get_note(k).get_pitch() - 
													  counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
										errors_found.insert_new_error("ERROR: The weak beat of measure " + ((i / time_signature) + 1) +
											" contains an illegal interval between the cantus and the counterpoint." + 
											" Vertical intervals other than unisons, minor and major thirds, perfect" +
											" fifths, minor and major sixths, octaves and the multiples of these" +
											" intervals are only permitted on weak beats when they are not involved" +
											" in a any part of a suspension and are approached and left left stepwise.");
									}
									count = 0; // the number of notes detected
									k = counterpoint_location - 1;
									while (count != 1 && k >= 0)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
												count++;
											else k--;
										}
										else k--;
										if (k < 0) count = 2; // to avoid going out of bounds
									}
									if (count == 1)
									{
										if (Math.abs( counterpoint.get_note(k).get_pitch() - 
													  counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
											errors_found.insert_new_error("ERROR: The weak beat of measure " + ((i / time_signature) + 1) +
												" contains an illegal interval between the cantus and the counterpoint." + 
												" Vertical intervals other than unisons, minor and major thirds, perfect" +
												" fifths, minor and major sixths, octaves and the multiples of these" +
												" intervals are only permitted on weak beats when they are not involved" +
												" in any part of a suspension and are approached and left left stepwise.");
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_distance_between_lines()
	// Check to see if the following rule is violated:
	// Intervals greater than perfect twelfths should generally be avoided
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for illegal vertical intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() - 
					                      cantus.get_note(cantus_location).get_pitch());


					// Check if a vertical interval is greater than a perfect 12th
					if (difference > 19)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains a large interval between the cantus and the counterpoint." + 
							" Intervals greater than perfect twelfths are generally to be avoided.");
					}
				}
			}
		}
	}


	/***************************************************************************/

	
	public void check_perfect_approach()
	// Check to see if the following rule is violated:
	// If species is broken, perfect intervals must be approached with contrary or oblique motion.
	// This rules out the possibility of parallel fifths or octaves, which are absolutely forbidden.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint
		int prev_counterpoint_location = 0; // the location of the previous note of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0 && counterpoint.get_note(j).get_tied() == 1)
							{
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									prev_counterpoint = 0;
									prev_cantus = 0;
									prev_counterpoint_location = 0;


									// Find the previous counterpoint note
									for (int k = j - 1; k >= 0; k--)
									{
										if (counterpoint.get_note(k) != null)
										{
											prev_counterpoint = counterpoint.get_note(k).get_pitch();
											prev_counterpoint_location = k;
											k = -1;
										}
									}
									

									// Find the cantus note corresponding to this note
									if (cantus.get_note(prev_counterpoint_location) != null)
										prev_cantus = cantus.get_note(prev_counterpoint_location).get_pitch();
									else 
									{
										for (int k = j - 1; k >= 0; k--)
										{
											if (cantus.get_note(k) != null)
											{
												prev_cantus = cantus.get_note(k).get_pitch();
												k = -1;
											}
										}
									}


									// Generate an error if the motion isn't oblique or consonant
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										if (get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'c' &&
											get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'o')
										{
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains a perfect interval that is not approached with the proper type of" +
												" motion. Perfect intervals must be approached with oblique or contrary motion." +
												" This rules out the possibility of parallel fifths or octaves, which are absolutely forbidden.");
										}
									}
								}
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_voice_crossing()
	// Check to see if the following rule is violated:
	// A note in the bottom line may never be above the corresponding note of the top line and vice versa
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Deal with case where cantus is the upper voice
								if ( counterpoint.get_type_of_line() % 2 == 0 )
								{
									if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() < 0 )
										errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
											((i / time_signature) + 1) + ". A note in the bottom line may never be" +
											" above the corresponding note of the top line and vice versa.");
								}
								
								
								// Deal with case where cantus is lower voice
								else if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() > 0 )
									errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
										((i / time_signature) + 1) + ". A note in the bottom line may never be" +
										" above the corresponding note of the top line and vice versa.");
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_voice_overlap()
	// Check to see if the following rule is violated:
	// A note in the bottom line cannot move up past where the last note of the top line was just
	// previously and vice versa
	{
		int prev_cantus = -1; // the previous note of the cantus
		int prev_counterpoint = -1; // the previous note of the counerpoint
		boolean found_one = false; // set to true when the first counterpoint note in a measure is found
		boolean error_found = false; // set to true if voice overlap is found

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			error_found = false;
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					if (prev_cantus == -1)
					{
						prev_cantus = cantus.get_note(i).get_pitch();
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}		
					}
					else
					{
						// Check cantus for voice overlap in case where cantus is the upper voice
						if ( counterpoint.get_type_of_line() % 2 == 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint < 0 )
								error_found = true;
						}


						// Check cantus for voice overlap in case where cantus is the lower voice
						else if ( counterpoint.get_type_of_line() % 2 != 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint > 0 )
								error_found = true;
						}


						// Check counterpoint for voice overlap
						found_one = false;
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									if (found_one == false)
									{
										// Check counterpoint for voice overlap in case where cantus is the upper voice
										if ( counterpoint.get_type_of_line() % 2 == 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus > 0 )
												error_found = true;
										}


										// Check counterpoint for voice overlap in case where cantus is the lower voice
										else if ( counterpoint.get_type_of_line() % 2 != 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus < 0 )
												error_found = true;
										}
									}
									found_one = true;
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}
						prev_cantus = cantus.get_note(i).get_pitch();
					}
				}
			}
			
			if (error_found == true)
				errors_found.insert_new_error("ERROR: There is voice overlap in measure " + ((i / time_signature) + 1) + 
					".  A note in the bottom line cannot move up past where the last note of the top line was just previously" +
					" and vice versa.");
		}
	}


	/***************************************************************************/

	
	public void check_beginning_interval()
	// Check to see if the following rule is violated:
	// The first vertical interval must be a P1, P5 or P8. A P5 is only allowed when the counterpoint is
	// above the cantus.
	{
		int cantus_pitch = -1; // the first pitch in the cantus
		int counterpoint_pitch = -1; // the first pitch in the counterpoint
		
		
		// Find the first vertical interval
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					cantus_pitch = cantus.get_note(i).get_pitch();

					for (int j = 0; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								counterpoint_pitch = counterpoint.get_note(j).get_pitch();
								j = i + cantus.get_note(i).get_duration();
							}
						}
					}

					i = total_sixteenth_notes;
				}
			}
		}
		int interval = counterpoint_pitch - cantus_pitch; // the interval between the first two notes


		// Detect a missign first inteval
		if (cantus_pitch == -1 || counterpoint_pitch == -1)
			errors_found.insert_new_error("ERROR: The line has an improper first interval.");


		// Detect errors when the cantus is the upper voice
		else if ( counterpoint.get_type_of_line() % 2 == 0 )
		{
			if (interval != 0 && interval != -12)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the upper voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or an octave below it.");
			}
		}


		// Detect errors when the cantus is the lower voice
		else if ( counterpoint.get_type_of_line() % 2 != 0 )
		{
			if (interval != 0 && interval != 12 && interval != 7)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the lower voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or a fifth or an octave above it.");
			}
		}
	}


	/***************************************************************************/


	public void check_last_interval()
	// Check to see if the following rule is violated:
	// The interval of the notes struck on the first beat of the last measure of the piece must be a P1 or a P8.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int difference; // the interval between the two voices

		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			difference = Math.abs(counterpoint.get_note(index).get_pitch() - cantus.get_note(index).get_pitch());
			if (difference != 0 && difference != 12)
				errors_found.insert_new_error("ERROR: The vertical interval on the first beat of the last measure" +
					" is incorrect. It must be either a unison or an octave.");
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void check_last_approach()
	// Check to see if the following rule is violated:
	// The notes on the first beat of the last measure must be approached with contrary motion.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int last_cantus; // the pitch of the last note of the cantus
		int last_counterpoint; // the pitch of the last note of the counterpoint
		int approach_cantus = 0; // the pitch of the second to last note of the cantus
		int approach_counterpoint = 0; // the pitch of the second to last note of the counterpoint
		
		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			last_cantus = cantus.get_note(index).get_pitch();
			last_counterpoint = counterpoint.get_note(index).get_pitch();
			

			// Find the second to last note of the counterpoint
			int count = 0; // the number of notes detected
			int k = index - 1;
			while (count != 1 && k >= 0)
			{
				if (counterpoint.get_note(k) != null)
				{
					if (counterpoint.get_note(k).get_pitch() != 0)
					{
						approach_counterpoint = counterpoint.get_note(k).get_pitch();
						count++;
					}
					else k--;
				}
				else k--;
				if (k < index - time_signature) count = 2; // to avoid going out of bounds
			}
			if (count == 1) // find the second to last note of the cantus
			{
				count = 0;
				k = index - 1;
				while (count != 1 && k < total_sixteenth_notes)
				{
					if (cantus.get_note(k) != null)
					{
						if (cantus.get_note(k).get_pitch() != 0)
						{
							approach_cantus = cantus.get_note(k).get_pitch();
							count++;
						}
						else k--;
					}
					else k--;
					if (k < index - time_signature) count = 2; // to avoid going out of bounds
				}
				if (count == 1)
				{
					if ( get_type_of_motion(approach_cantus, last_cantus, approach_counterpoint,
					                        last_counterpoint) != 'c')
						errors_found.insert_new_error("ERROR: The notes on the first beat of the last" +
							" measure are approached incorrectly. They must be approached with contrary motion.");
				}
			}
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void check_downward_resolution()
	// Check to see if the following rule is violated:
	// Dissonant suspensions must resolve downwards by step
	// NOTE: assumes that "strong beat" means the first beat
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for illegal vertical intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() - 
					                      cantus.get_note(cantus_location).get_pitch());
					difference = difference % 12;


					// Check if the interval is dissonant, on a strong beat and part of a suspension
					if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 && difference != 9 && 
						i % time_signature == 0 && counterpoint.get_note(counterpoint_location).get_tied() == 2)
					{
						int count = 0; // the number of notes detected
						int k = counterpoint_location + 1;
						while (count != 1 && k < total_sixteenth_notes)
						{
							if (counterpoint.get_note(k) != null)
							{
								if (counterpoint.get_note(k).get_pitch() != 0)
									count++;
								else k++;
							}
							else k++;
							if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
						}
						if (count == 1)
						{
							if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch()
							              - counterpoint.get_note(k).get_pitch() ) > 2)
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a dissonant suspension that does not resolve by step." +
									" Dissonant suspensions must resolve downwards by step.");
							if ( counterpoint.get_note(counterpoint_location).get_pitch()
							     < counterpoint.get_note(k).get_pitch() )
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
									" contains a dissonant suspension that does not resolve downwards." +
									" Dissonant suspensions must resolve downwards by step.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_suspensions()
	// Check to see if the following rule is violated:
	// It is not permitted to have more than three suspensions of the same type occurring consecutively
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int count = 1; // the number of consecutive suspensions of the same type	
		Suspension susp;
		int first_interval = -2;
		int second_interval = -2;

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (first_interval == susp.first && second_interval == susp.second)
						count++;
					else
					{
						first_interval = susp.first;
						second_interval = susp.second;
						count = 1;
					}

					if (count > 3)
						errors_found.insert_new_error("WARNING: Measure " + (bar + 1) +
							" contains number " + count + " of a series of consecutive suspensions" +
							" of the same type. More than three suspensions" +
							" of the same type occurring consecutively should be avoided.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_9_8_sequences()
	// Check to see if the following rule is violated:
	// Consecutive 9-8 suspensions are not permitted
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int location = -2; // the measure of the previous suspension
		
		Suspension susp;
		int first_interval = -2;
		int second_interval = -2;

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (susp.first == 9 && susp.second == 8)
					{
						if (location == bar - 1)
							errors_found.insert_new_error("ERROR: Measure " + (bar + 1) +
								" contains number a repeated 9-8 suspension." +
								" Consecutive 9-8 suspensions are not permitted.");
						location = bar;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_2_1_sequences()
	// Check to see if the following rule is violated:
	// Consecutive 2-1 suspensions are not permitted
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int location = -2; // the measure of the previous suspension
		Suspension susp;
		int first_interval = -2;
		int second_interval = -2;

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (susp.first == 2 && susp.second == 1)
					{
						if (location == bar - 1)
							errors_found.insert_new_error("ERROR: Measure " + (bar + 1) +
								" contains number a repeated 2-1 suspension." +
								" Consecutive 2-1 suspensions are not permitted.");
						location = bar;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_4_5_sequences()
	// Check to see if the following rule is violated:
	// Consecutive 4-5 suspensions are not permitted
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int location = -2; // the measure of the previous suspension	
		Suspension susp; // the intervals involved in a suspension
		int first_interval = -2; // the first interval of a suspension
		int second_interval = -2; // the second interval of a suspension

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (susp.first == 4 && susp.second == 5)
					{
						if (location == bar - 1)
							errors_found.insert_new_error("ERROR: Measure " + (bar + 1) +
								" contains number a repeated 4-5 suspension." +
								" Consecutive 4-5 suspensions are not permitted.");
						location = bar;
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_7_8_present()
	// Check to see if the following rule is violated:
	// 7-8 suspensions are forbidden
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int location = -2; // the measure of the previous suspension
		Suspension susp; // the intervals involved in a suspension
		int first_interval = -2; // the first interval of a suspension
		int second_interval = -2; // the second interval of a suspension

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (susp.first == 7 && susp.second == 8)
						errors_found.insert_new_error("ERROR: Measure " + (bar + 1) +
							" contains number a 7-8 suspension. 7-8 suspensions are forbidden.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_2_1_present()
	// Check to see if the following rule is violated:
	// 2-1 suspensions should be avoided if possible
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int location = -2; // the measure of the previous suspension
		Suspension susp; // the intervals involved in a suspension
		int first_interval = -2; // the first interval of a suspension
		int second_interval = -2; // the second interval of a suspension

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					susp = get_type_of_suspension( cantus.get_note(strong).get_pitch(), 
					                               counterpoint.get_note(strong).get_pitch(),
					                               counterpoint.get_note(weak).get_pitch() );
				
					if (susp.first == 2 && susp.second == 1)
						errors_found.insert_new_error("WARNING: Measure " + (bar + 1) +
							" contains number a 2-1 suspension. 2-1 suspensions should be avoided if possible.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_enough_consonant_suspensions()
	// Check to see if the following rule is violated:
	// There should be at least one consonant suspension in the piece
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		boolean consonance = false; // is true if a suspension is consonant

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					if ( is_consonant( cantus.get_note(strong).get_pitch(), 
					                   counterpoint.get_note(strong).get_pitch(),
					                   counterpoint.get_note(weak).get_pitch() )
						 == true)
					{
						consonance = true;
					}
				}
			}
		}

		if (consonance == false)
			errors_found.insert_new_error("WARNING: There are no consonant suspensions" +
				" in the piece. There should be at least one consonant suspension in the piece.");
	}


	/***************************************************************************/


	public void check_enough_dissonant_suspensions()
	// Check to see if the following rule is violated:
	// There should be more dissonant suspensions than consonant suspensions
	// NOTE: Assumes that the strong beat is the first and the weak beat is the third
	{
		int strong; // the strong beat
		int weak; // the weak_beat
		int consonance_count = 0; // the number of consonant suspensions in the piece
		int dissonance_count = 0; // the number of dissonant suspensions in the piece

		for (int bar = 0; bar < number_of_bars; bar++)
		{
			strong = bar * time_signature;
			weak = bar * time_signature + time_signature / 2;

			if (counterpoint.get_note(strong) != null && 
				cantus.get_note(strong) != null &&
				counterpoint.get_note(weak) != null)
			{
				// Check that there are no rests and that the first note is tied back
				if (counterpoint.get_note(strong).get_pitch() != 0 && 
					cantus.get_note(strong).get_pitch() != 0 &&
					counterpoint.get_note(weak).get_pitch() != 0 &&
					counterpoint.get_note(strong).get_tied() == 2)
				{
					if ( is_consonant( cantus.get_note(strong).get_pitch(), 
					                   counterpoint.get_note(strong).get_pitch(),
					                   counterpoint.get_note(weak).get_pitch() )
						 == true)
					{
						consonance_count++;
					}
					else dissonance_count++;
				}
			}
		}

		if (consonance_count >= dissonance_count)
			errors_found.insert_new_error("WARNING: There are " + dissonance_count + " dissonant" +
				" suspensions and " + consonance_count + " consonant suspensions." +
				" There should be more dissonant suspensions than consonant suspensions in the piece.");
	}


	/* PRIVATE METHODS *********************************************************/


	private char get_type_of_motion(int v1_first_pitch, int v1_second_pitch,
	                                int v2_first_pitch, int v2_second_pitch)
	// Return whether the motion from pitch v1_first_pitch to v1_second_pitch in one voice
	// and from v2_first_pitch to v2_second_pitch in the other voice is similar, parallel,
	// contrary or oblique. 's' means similar, 'p' means parallel, 'c' means contrary and
	// 'o' means oblique.
	// NOTE: the definition of parallel motion used here includes only exactly parallel motion.
	//       Movement from a minor third to a major third, for example, is considered similar.
	//       This diverges from some definitions of parallel motion.
	{
		if (v1_first_pitch == v1_second_pitch || v2_first_pitch == v2_second_pitch)
			return 'o';
		else if ( (v1_first_pitch - v1_second_pitch > 0 && v2_first_pitch - v2_second_pitch < 0) ||
		          (v1_first_pitch - v1_second_pitch < 0 && v2_first_pitch - v2_second_pitch > 0) )
			return 'c';
		else if (v1_first_pitch - v2_first_pitch == v1_second_pitch - v2_second_pitch)
			return 'p';
		else
			return 's';
	}
	
	
	/***************************************************************************/


	private boolean is_consonant(int cantus_note, int first_note, int second_note)
	// Returns true if the suspension involving the given notes is consonant, returns false if it is
	// dissonant.
	{
		Suspension to_check = get_type_of_suspension(cantus_note, first_note, second_note);
		if (to_check.first == 2) return false;
		else if (to_check.first == 4) return false;
		else if (to_check.first == 7) return false;
		else if (to_check.first == 9) return false;
		else if (to_check.first == 11) return false;
		else return true;
	}

	
	/***************************************************************************/


	private Suspension get_type_of_suspension(int cantus_note, int first_note, int second_note)
	// Returns the type of suspension as defined by the intervals between the pitches
	// of the cantus_note and the first_note and the cantus_note and the second_note.
	// Intervals in the return object will be set to -1 to indicate intervals greater
	// than perfect twlefths or if one of the given notes is a rest.
	{
		int top_note_1;
		int top_note_2;
		int bottom_note_1;
		int bottom_note_2;
		int first_interval;
		int second_interval;

		if (cantus_note == 0 || first_note == 0 || second_note == 0)
			return new Suspension(-1, -1);
		else if (counterpoint.get_type_of_line() % 2 == 0) // lower counterpoint
		{
			bottom_note_1 = first_note;
			bottom_note_2 = second_note;
			top_note_1 = cantus_note;
			top_note_2 = cantus_note;
		}
		else // upper counterpoint
		{
			bottom_note_1 = cantus_note;
			bottom_note_2 = cantus_note;
			top_note_1 = first_note;
			top_note_2 = second_note;
		}

		first_interval = top_note_1 - bottom_note_1;
		first_interval = convert_to_interval(first_interval);
		second_interval = top_note_2 - bottom_note_2;
		second_interval = convert_to_interval(second_interval);

		return new Suspension(first_interval, second_interval);
	}

					
	/***************************************************************************/


	private int convert_to_interval(int raw_interval)
	// Takes in a pitch difference and gives it an interval number
	// e.g. 4 (major 3rd) -> 3
	// Returns -1 if an interval over a perfect twelfth is input.
	// NOTE: Considers a diminished 5th to be an augmented 4th
	{
		if (raw_interval == 0) return 1;
		else if (raw_interval == 1 || raw_interval == 2) return 2;
		else if (raw_interval == 3 || raw_interval == 4) return 3;
		else if (raw_interval == 5 || raw_interval == 6) return 4;
		else if (raw_interval == 7) return 5;
		else if (raw_interval == 8 || raw_interval == 9) return 6;
		else if (raw_interval == 10 || raw_interval == 11) return 7;
		else if (raw_interval == 12) return 8;
		else if (raw_interval == 13 || raw_interval == 14) return 9;
		else if (raw_interval == 15 || raw_interval == 16) return 10;
		else if (raw_interval == 17 || raw_interval == 18) return 11;
		else if (raw_interval == 19) return 12;
		else return -1;
	}


	/* INTERNAL CLASSES ********************************************************/


	class Suspension
	// Used to communicate information about the intervals involved in a suspension
	{
		public int first;	// the first interval of a suspension
		public int second;	// the second interval of a suspension

		public Suspension (int fir, int sec)
		{
			first = fir;
			second = sec;
		}
	}
}