/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerVerticalSecond
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the vertical aspect of a second species counterpoint line have been violated.
				There is one method for each rule. AnalyzerVerticalSecond objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerVerticalSecond
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private Voice cantus;				// the voice corresponding to the counterpoint to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerVerticalSecond(Voice cant, Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		cantus = cant;
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_vertical_intervals()
	// Check to see if the following rule is violated:
	// Vertical intervals other than P1, m3, M3, P5, m6, M6, P8, m10, M10, P12 and their multiples are only
	// permitted on weak beats, and must be approached and left with stepwise motion. Intervals greater than P12
	// should be avoided.
	// NOTE: assumes that "weak beats" means the third beat
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for illegal vertical intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() - 
					                      cantus.get_note(cantus_location).get_pitch());


					// Check if a vertical interval is greater than a perfect 12th
					if (difference > 19)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains a large interval between the cantus and the counterpoint." + 
							" Intervals greater than perfect twelfths are generally to be avoided.");
					}

					
					// Check if the interval is illegal
					difference = difference % 12;
					if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						difference != 9 && difference != 12 && difference != 15 && difference != 16 &&
						difference != 19)
					{
						// Check if a note is not on a weak beat
						if (i % time_signature != time_signature / 2)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an illegal interval between the cantus and the counterpoint." + 
								" Vertical intervals other than unisons, minor and major thirds, perfect fifths," +
								" minor and major sixths, octaves, minor and major tenths and perfect" +
								" twelfths are only permitted on weak beats.");
						}

						
						// Check if a note is not approached and left stepwise
						else 
						{
							int count = 0; // the number of notes detected
							int k = counterpoint_location + 1;
							while (count != 1 && k < total_sixteenth_notes)
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k++;
								}
								else k++;
								if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
							}
							if (count == 1)
							{
								if (Math.abs( counterpoint.get_note(k).get_pitch() - 
								              counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
								errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains an illegal interval between the cantus and the counterpoint." + 
										" Vertical intervals other than unisons, minor and major thirds, perfect fifths," +
										" minor and major sixths, octaves, minor and major tenths and perfect" +
										" twelfths must be left stepwise.");
							}
							count = 0; // the number of notes detected
							k = counterpoint_location - 1;
							while (count != 1 && k >= 0)
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k--;
								}
								else k--;
								if (k < 0) count = 2; // to avoid going out of bounds
							}
							if (count == 1)
							{
								if (Math.abs( counterpoint.get_note(k).get_pitch() - 
								              counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
									errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains an illegal interval between the cantus and the counterpoint." + 
										" Vertical intervals other than unisons, minor and major thirds, perfect fifths," +
										" minor and major sixths, octaves, minor and major tenths and perfect" +
										" twelfths must be approached stepwise.");
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_unisons()
	// Check to see if the following rule is violated:
	// Vertical unisons are only permitted on weak beaks, except for in the first beats of the first
	// and last measures, where they may be present. If they occur on weak beats, they must be left by
	// stepwise motion in the opposite direction to that by which they were approached.
	// NOTE: assumes that "weak beats" means the third beat
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for unisons
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					if (counterpoint.get_note(counterpoint_location).get_pitch() ==
						cantus.get_note(cantus_location).get_pitch())
					{
						// Check if unison is inappropriately placed
						if (counterpoint_location % time_signature != time_signature / 2 &&
							counterpoint_location >= time_signature &&
							counterpoint_location != total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an illegally placed vertical unison." + 
								" Vertical unisons are only permitted on weak beaks, except for on the" +
								" first beats of the first and last measures, where they may be present.");
						}
						else if (counterpoint_location % time_signature == time_signature / 2)
						{
							int direction_of_motion = 0; // is 1 for upwards and -1 for downwards
							int count = 0; // the number of notes detected
							int k = counterpoint_location + 1;
							while (count != 1 && k < total_sixteenth_notes) // find the note after the unison
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k++;
								}
								else k++;
								if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
							}
							if (count == 1) // check if the unison is left by step
							{
								if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch() - 
								              counterpoint.get_note(k).get_pitch()) > 2)
									errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains a vertical unison followed by a leap in the counterpoint." + 
										" Vertical unisons occuring on weak beats must be followed by" +
										" stepwise motion in the opposite direction to that by which" +
										" they were approached.");
								if (counterpoint.get_note(counterpoint_location).get_pitch() < 
									counterpoint.get_note(k).get_pitch())
									direction_of_motion = 1;
								else if (counterpoint.get_note(counterpoint_location).get_pitch() >
										 counterpoint.get_note(k).get_pitch())
									direction_of_motion = -1;
								else
									direction_of_motion = 0;
							}
							count = 0; // the number of notes detected
							k = counterpoint_location - 1;
							while (count != 1 && k >= 0) // find the note after the unison
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k--;
								}
								else k--;
								if (k < 0) count = 2; // to avoid going out of bounds
							}
							if (count == 1) // check if the unison is left in the proper direction
							{
								if ( ( direction_of_motion == 1 &&
									   counterpoint.get_note(counterpoint_location).get_pitch() >
									   counterpoint.get_note(k).get_pitch() ) ||
								     ( direction_of_motion == -1 &&
									   counterpoint.get_note(counterpoint_location).get_pitch() <
									   counterpoint.get_note(k).get_pitch() ) )
								{
									errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
										" contains a vertical unison followed by movement in the counterpoint" +
										" in the same direction as the motion by which the unison was approached." + 
										" Vertical unisons occuring on weak beats must be followed by" +
										" stepwise motion in the opposite direction to that by which" +
										" they were approached.");
								}
							} 
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_perfect_twelvthes()
	// Check to see if the following rule is violated:
	// Vertical perfect 12ths are only permitted in the climax of the counterpoint.
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Check for perfect twelfths
				if (counterpoint.get_note(counterpoint_location).get_pitch() != 0 &&
					cantus.get_note(cantus_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());
					if (difference == 19)
					{
						// Check if the perfect 12th occurs on the counterpoint's climax
						if (counterpoint_location != get_climax_position(counterpoint))
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains a vertical perfect 12th. This measure does not contain the" +
								" climax of the counterpoint. Vertical perfect 12ths are only permitted" +
								" on the climax of the counterpoint.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_perfect_approach()
	// Check to see if the following rule is violated:
	// Perfect intervals must be approached with contrary or oblique motion.
	// This rules out the possibility of parallel fifths or octaves, which are absolutely forbidden.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint
		int prev_counterpoint_location = 0; // the location of the previous note of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									prev_counterpoint = 0;
									prev_cantus = 0;
									prev_counterpoint_location = 0;


									// Find the previous counterpoint note
									for (int k = j - 1; k >= 0; k--)
									{
										if (counterpoint.get_note(k) != null)
										{
											prev_counterpoint = counterpoint.get_note(k).get_pitch();
											prev_counterpoint_location = k;
											k = -1;
										}
									}
									

									// Find the cantus note corresponding to this note
									if (cantus.get_note(prev_counterpoint_location) != null)
										prev_cantus = cantus.get_note(prev_counterpoint_location).get_pitch();
									else 
									{
										for (int k = j - 1; k >= 0; k--)
										{
											if (cantus.get_note(k) != null)
											{
												prev_cantus = cantus.get_note(k).get_pitch();
												k = -1;
											}
										}
									}


									// Generate an error if the motion isn't oblique or consonant
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										if (get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'c' &&
											get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'o')
										{
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains a perfect interval that is not approached with the proper type of" +
												" motion. Perfect intervals must be approached with oblique or contrary motion." +
												" This rules out the possibility of parallel fifths or octaves, which are absolutely forbidden.");
										}
									}
								}
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_successive_strong_beat_perfect_intervals()
	// Check to see if the following rule is violated:
	// Parallel octaves or fifths that occur on successive first beats should be avoided, regardless
	// of the half note separating them. Although these are permissible occasionally, they should not
	// be emphasized or used frequently.
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int next_difference = 0; // the difference in pitch between the cantus and the counterpoint on the next
		                         // first beat
		int j;

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == 7 || difference == 12)
					{
						j = i + time_signature;
						if (j < total_sixteenth_notes)
						{
							if (counterpoint.get_note(j) != null && cantus.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0 && cantus.get_note(j).get_pitch() != 0)
								{
									next_difference = Math.abs(counterpoint.get_note(j).get_pitch() -
									                           cantus.get_note(j).get_pitch());
									if (next_difference == difference)
										errors_found.insert_new_error("WARNING: The first beat of measure " +
											((j / time_signature) + 1) + " contains the same vertical perfect interval" +
											" as the interval on the first beat of the preceding measure." +
											" Parallel octaves or fifths that occur on successive first beats" +
											" should be avoided, regardless of the half note separating them." +
											" Although these are permissible occasionally, they should not be" +
											" emphasized or used frequently.");
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_vertical_intervals()
	// Check to see if the following rule is violated:
	// Should not have the same vertical interval occurring on the first beats of more than three
	// consecutive measures.
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int last_interval = 0; // the difference of the last pair of notes which were compared
		int count = 0; // the number of times that an interval has occured consecutively

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == last_interval) count++;
					else count = 0;
					last_interval = difference;
					
					if (count > 2)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains number " + (count + 1) + " of a series of consecutive identical" +
							" vertical intervals struck on strong beats. There should not be more than" +
							" three strong beats in a row that hold the same vertical interval.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leaps_from_perfect()
	// Check to see if the following rule is violated:
	// If there is a perfect vertical interval on the first beat of a measure, then the
	// counterpoint note should not be leaped to or from. This is sometimes permissible
	// if the leap is small and the direction of motion is changed, but it should
	// generally be avoided.
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int difference; // the difference in pitch between the cantus and the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for perfect intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0 &&
					counterpoint_location % time_signature != time_signature / 2)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());
					if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
					{
						// Find the next note
						int count = 0; // the number of notes detected
						int k = counterpoint_location + 1;
						while (count != 1 && k < total_sixteenth_notes) // find the note after the unison
						{
							if (counterpoint.get_note(k) != null)
							{
								if (counterpoint.get_note(k).get_pitch() != 0)
									count++;
								else k++;
							}
							else k++;
							if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
						}
						if (count == 1) // check if the unison is left by step
						{
							if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch() - 
							              counterpoint.get_note(k).get_pitch() ) > 2)
								errors_found.insert_new_error("WARNING: The first beat of measure " + 
									((i / time_signature) + 1) + " contains a vertical perfect interval" +
									" that is followed by a leap in the counterpoint." +
									" If there is a perfect vertical interval on the first beat of a measure," +
									" then the counterpoint note should not be leaped to or from. This is" +
									" sometimes permissible if the leap is small and the direction of motion" +
									" is changed, but it should generally be avoided.");
						}
						
						
						// Find the previous note
						count = 0;
						k = counterpoint_location - 1;
						while (count != 1 && k >= 0) // find the note after the unison
						{
							if (counterpoint.get_note(k) != null)
							{
								if (counterpoint.get_note(k).get_pitch() != 0)
									count++;
								else k--;
							}
							else k--;
							if (k < 0) count = 2; // to avoid going out of bounds
						}
						if (count == 1) // check if the unison is left in the proper direction
						{
							if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch() - 
							              counterpoint.get_note(k).get_pitch() ) > 2)
								errors_found.insert_new_error("WARNING: The first beat of measure " + 
									((i / time_signature) + 1) + " contains a vertical perfect interval" +
									" that is approached by leap in the counterpoint." +
									" If there is a perfect vertical interval on the first beat of a measure," +
									" then the counterpoint note should not be leaped to or from. This is" +
									" sometimes permissible if the leap is small and the direction of motion" +
									" is changed, but it should generally be avoided.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_simultaneous_climaxes()
	// Check to see if the following rule is violated:
	// The climax may not occur at the same time in both the cantus and the counterpoint
	{
		if (get_climax_position(cantus) == get_climax_position(counterpoint))
			errors_found.insert_new_error("WARNING: Both the cantus firmus and the counterpoint have" +
				" their climaxes occuring at the same place. The lines" +
				" should have their climaxes occuring at different locations.");
	}


	/***************************************************************************/


	public void check_voice_crossing()
	// Check to see if the following rule is violated:
	// A note in the bottom line may never be above the corresponding note of the top line and vice versa
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Deal with case where cantus is the upper voice
								if ( counterpoint.get_type_of_line() % 2 == 0 )
								{
									if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() < 0 )
										errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
											((i / time_signature) + 1) + ". A note in the bottom line may never be" +
											" above the corresponding note of the top line and vice versa.");
								}
								
								
								// Deal with case where cantus is lower voice
								else if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() > 0 )
									errors_found.insert_new_error("ERROR: There is voice crossing in measure " + 
										((i / time_signature) + 1) + ". A note in the bottom line may never be" +
										" above the corresponding note of the top line and vice versa.");
							}
						}
					}				
				}
			}
		}
	}


	/***************************************************************************/


	public void check_voice_overlap()
	// Check to see if the following rule is violated:
	// A note in the bottom line cannot move up past where the last note of the top line was just
	// previously and vice versa
	{
		int prev_cantus = -1; // the previous note of the cantus
		int prev_counterpoint = -1; // the previous note of the counerpoint
		boolean found_one = false; // set to true when the first counterpoint note in a measure is found
		boolean error_found = false; // set to true if voice overlap is found

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			error_found = false;
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					if (prev_cantus == -1)
					{
						prev_cantus = cantus.get_note(i).get_pitch();
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}		
					}
					else
					{
						// Check cantus for voice overlap in case where cantus is the upper voice
						if ( counterpoint.get_type_of_line() % 2 == 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint < 0 )
								error_found = true;
						}


						// Check cantus for voice overlap in case where cantus is the lower voice
						else if ( counterpoint.get_type_of_line() % 2 != 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint > 0 )
								error_found = true;
						}


						// Check counterpoint for voice overlap
						found_one = false;
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									if (found_one == false)
									{
										// Check counterpoint for voice overlap in case where cantus is the upper voice
										if ( counterpoint.get_type_of_line() % 2 == 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus > 0 )
												error_found = true;
										}


										// Check counterpoint for voice overlap in case where cantus is the lower voice
										else if ( counterpoint.get_type_of_line() % 2 != 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus < 0 )
												error_found = true;
										}
									}
									found_one = true;
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}
						prev_cantus = cantus.get_note(i).get_pitch();
					}
				}
			}
			
			if (error_found == true)
				errors_found.insert_new_error("ERROR: There is voice overlap in measure " + ((i / time_signature) + 1) + 
					".  A note in the bottom line cannot move up past where the last note of the top line was just previously" +
					" and vice versa.");
		}
	}


	/***************************************************************************/

	
	public void check_beginning_interval()
	// Check to see if the following rule is violated:
	// The first vertical interval must be a P1, P5 or P8. A P5 is only allowed when the counterpoint is
	// above the cantus.
	{
		int cantus_pitch = -1; // the first pitch in the cantus
		int counterpoint_pitch = -1; // the first pitch in the counterpoint
		
		
		// Find the first vertical interval
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					cantus_pitch = cantus.get_note(i).get_pitch();

					for (int j = 0; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								counterpoint_pitch = counterpoint.get_note(j).get_pitch();
								j = i + cantus.get_note(i).get_duration();
							}
						}
					}

					i = total_sixteenth_notes;
				}
			}
		}
		int interval = counterpoint_pitch - cantus_pitch; // the interval between the first two notes


		// Detect a missign first inteval
		if (cantus_pitch == -1 || counterpoint_pitch == -1)
			errors_found.insert_new_error("ERROR: The line has an improper first interval.");


		// Detect errors when the cantus is the upper voice
		else if ( counterpoint.get_type_of_line() % 2 == 0 )
		{
			if (interval != 0 && interval != -12)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the upper voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or an octave below it.");
			}
		}


		// Detect errors when the cantus is the lower voice
		else if ( counterpoint.get_type_of_line() % 2 != 0 )
		{
			if (interval != 0 && interval != 12 && interval != 7)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the lower voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or a fifth or an octave above it.");
			}
		}
	}


	/***************************************************************************/


	public void check_last_interval()
	// Check to see if the following rule is violated:
	// The interval of the notes struck on the first beat of the last measure of the piece must be a P1 or a P8.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int difference; // the interval between the two voices

		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			difference = Math.abs(counterpoint.get_note(index).get_pitch() - cantus.get_note(index).get_pitch());
			if (difference != 0 && difference != 12)
				errors_found.insert_new_error("ERROR: The vertical interval on the first beat of the last measure" +
					" is incorrect. It must be either a unison or an octave.");
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void check_last_approach()
	// Check to see if the following rule is violated:
	// The notes on the first beat of the last measure must be approached with contrary motion.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int last_cantus; // the pitch of the last note of the cantus
		int last_counterpoint; // the pitch of the last note of the counterpoint
		int approach_cantus = 0; // the pitch of the second to last note of the cantus
		int approach_counterpoint = 0; // the pitch of the second to last note of the counterpoint
		
		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			last_cantus = cantus.get_note(index).get_pitch();
			last_counterpoint = counterpoint.get_note(index).get_pitch();
			

			// Find the second to last note of the counterpoint
			int count = 0; // the number of notes detected
			int k = index - 1;
			while (count != 1 && k >= 0)
			{
				if (counterpoint.get_note(k) != null)
				{
					if (counterpoint.get_note(k).get_pitch() != 0)
					{
						approach_counterpoint = counterpoint.get_note(k).get_pitch();
						count++;
					}
					else k--;
				}
				else k--;
				if (k < index - time_signature) count = 2; // to avoid going out of bounds
			}
			if (count == 1) // find the second to last note of the cantus
			{
				count = 0;
				k = index - 1;
				while (count != 1 && k < total_sixteenth_notes)
				{
					if (cantus.get_note(k) != null)
					{
						if (cantus.get_note(k).get_pitch() != 0)
						{
							approach_cantus = cantus.get_note(k).get_pitch();
							count++;
						}
						else k--;
					}
					else k--;
					if (k < index - time_signature) count = 2; // to avoid going out of bounds
				}
				if (count == 1)
				{
					if ( get_type_of_motion(approach_cantus, last_cantus, approach_counterpoint,
					                        last_counterpoint) != 'c')
						errors_found.insert_new_error("ERROR: The notes on the first beat of the last" +
							" measure are approached incorrectly. They must be approached with contrary motion.");
				}
			}
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/* PRIVATE METHODS *********************************************************/


	private int get_number_notes(Voice voice_to_check)
	// Return the number of non-rest notes struck in the given Voice
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		return number_of_notes;
	}


	/***************************************************************************/


	private int get_climax_position(Voice voice_to_check)
	// Return the position of the climax in the voice_to_check voice. If the climax is repeated, the position
	// of its first occurence is returned. The climax is found to be the highest note of the line or, if the
	// first and last notes of the line are the hightes, the lowest note of the line.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note
		

		// Determine whether the number of notes in the line
		number_of_notes = get_number_notes(voice_to_check);
		
		
		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && voice_to_check.get_note(i).get_pitch() >= climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         voice_to_check.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;
		climax_note = 0;
		
		
		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;					
					}
					else if (high_or_low == 1 && voice_to_check.get_note(i).get_pitch() > climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && voice_to_check.get_note(i).get_pitch() < climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}
		
	
		// Return the position of the climax
		return climax_note_position;
	}


	/***************************************************************************/


	private char get_type_of_motion(int v1_first_pitch, int v1_second_pitch,
	                                int v2_first_pitch, int v2_second_pitch)
	// Return whether the motion from pitch v1_first_pitch to v1_second_pitch in one voice
	// and from v2_first_pitch to v2_second_pitch in the other voice is similar, parallel,
	// contrary or oblique. 's' means similar, 'p' means parallel, 'c' means contrary and
	// 'o' means oblique.
	// NOTE: the definition of parallel motion used here includes only exactly parallel motion.
	//       Movement from a minor third to a major third, for example, is considered similar.
	//       This diverges from some definitions of parallel motion.
	{
		if (v1_first_pitch == v1_second_pitch || v2_first_pitch == v2_second_pitch)
			return 'o';
		else if ( (v1_first_pitch - v1_second_pitch > 0 && v2_first_pitch - v2_second_pitch < 0) ||
		          (v1_first_pitch - v1_second_pitch < 0 && v2_first_pitch - v2_second_pitch > 0) )
			return 'c';
		else if (v1_first_pitch - v2_first_pitch == v1_second_pitch - v2_second_pitch)
			return 'p';
		else
			return 's';
	}
}