/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerVerticalThird
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if the individual rules relating
				to the vertical aspect of a third species counterpoint line have been violated.
				There is one method for each rule. AnalyzerVerticalThird objects should be eliminated
				after the analysis has been completed.
*/


class AnalyzerVerticalThird
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;			// the voice to be checked
	private Voice cantus;				// the voice corresponding to the counterpoint to be checked
	private ErrorList errors_found;		// the list of errors found
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the piece
	private int time_signature;			//  the time signature of the piece
	private int number_of_bars;			// the total number of bars in the piece
	private int total_sixteenth_notes;	// the total number of 16th note units in the piece


	/* CONSTRUCTOR *************************************************************/


	AnalyzerVerticalThird(Voice cant, Voice count, ErrorList err, int qual, int key, int tim, int num, int tot)
	{
		cantus = cant;
		counterpoint = count;
		errors_found = err;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;
		total_sixteenth_notes = tot;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_vertical_intervals()
	// Check to see if the following rule is violated:
	// Vertical intervals other than P1, m3, M3, P5, m6, M6, P8 and their multiples are only
	// permitted when:
	// - they are part of a double neighbour
	// - they are part of a nota cambiata
	// - they are part of a suspension that ends on the first beat of a measure
	// - they are not part of suspensions and are approached and left with stepwise
	//   motion on weak beats (i.e. second, third or fourth beat)
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int counter1 = -1; // the pitch of the first of four notes in the counterpoint
		int counter2 = -1; // the pitch of the second of four notes in the counterpoint
		int counter3 = -1; // the pitch of the third of four notes in the counterpoint
		int counter4 = -1; // the pitch of the fourth of four notes in the counterpoint
		int next_index; // where to go after each iteration of the leap
		int wait_until = -1; // the index to wait until before looking for errors


		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Update the counter notes if a new non-rest counterpoint note is detected
				counter1= -1;
				counter2= -1;
				counter3= -1;
				counter4= -1;
				next_index = i;
				if (i == counterpoint_location && counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					counter1 = counterpoint.get_note(counterpoint_location).get_pitch();
					next_index = get_next_note(counterpoint_location, counterpoint);
					if (next_index != -1)
						counter2 = counterpoint.get_note(next_index).get_pitch();
					next_index = get_next_note(next_index, counterpoint);
					if (next_index != -1)
						counter3 = counterpoint.get_note(next_index).get_pitch();
					next_index = get_next_note(next_index, counterpoint);
					if (next_index != -1)
						counter4 = counterpoint.get_note(next_index).get_pitch();
				}


				// Check for a dissonant interval on the first beat
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());
					difference = difference % 12;
					if ( difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						 difference != 9 && i % time_signature == 0 &&
						 counterpoint.get_note(counterpoint_location).get_tied() != 2 &&
						 counterpoint.get_note(counterpoint_location).get_tied() != 4)
						errors_found.insert_new_error("ERROR: The first beat of measure " + ((i / time_signature) + 1) +
							" contains an illegal interval between the cantus and the counterpoint." +
							" Vertical intervals other than unisons, minor and major thirds, perfect" +
							" fifths, minor and major sixths, octaves and the multiples of these" +
							" intervals are not permitted on the first beat of a measure.");
				}


				// Determine if a new counterpoint note is the first note of a double neighbour or nota cambiata
				if (counter1 != -1 && counter2 != -1 && counter3 != -1 && counter4 != -1)
				{
					if (is_nota_cambiata(counter1, counter2, counter3, counter4) == true)
						wait_until = next_index;
					if (is_double_neighbour(counter1, counter2, counter3, counter4)[0] == true)
						wait_until = next_index;
				}


				// Look for illegal vertical intervals if a non-rest note is found that is not part of a
				// double neighbour or nota cambiata
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0 &&
					i > wait_until)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());
					difference = difference % 12;


					// Check if the interval is dissonant
					if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						difference != 9)
					{
						// If a note is on a weak beat
						if (i % time_signature != 0)
						{
							int count = 0; // the number of notes detected
							int k = counterpoint_location - 1;
							while (count != 1 && k >= 0)
							{
								if (counterpoint.get_note(k) != null)
								{
									if (counterpoint.get_note(k).get_pitch() != 0)
										count++;
									else k--;
								}
								else k--;
								if (k < 0) count = 2; // to avoid going out of bounds
							}
							if (count == 1)
							{
								// Check if note is part of a suspension
								if (counterpoint.get_note(k).get_tied() != 1 ||	counterpoint.get_note(counterpoint_location).get_tied() != 1)
								{
									errors_found.insert_new_error("ERROR: The weak beat of measure " + ((i / time_signature) + 1) +
										" contains an illegal interval between the cantus and the counterpoint." +
										" Vertical intervals other than unisons, minor and major thirds, perfect" +
										" fifths, minor and major sixths, octaves and the multiples of these" +
										" intervals are only permitted on weak beats when they are not invovled" +
										" in any part of a suspension.");
								}


								// Check if a note is not approached and left stepwise if it is not part of a suspension
								else
								{
									count = 0; // the number of notes detected
									k = counterpoint_location + 1;
									while (count != 1 && k < total_sixteenth_notes)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
												count++;
											else k++;
										}
										else k++;
										if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
									}
									if (count == 1)
									{
										if (Math.abs( counterpoint.get_note(k).get_pitch() -
													  counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
										errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains a dissonant interval between the cantus and the counterpoint" +
											" that is not left with stepwise motion." +
											" Vertical intervals other than unisons, minor and major thirds, perfect" +
											" fifths, minor and major sixths, octaves and the multiples of these" +
											" intervals are only permitted on weak beats when they are" +
											" approached and left stepwise or are part of a double neighbour or" +
											" nota cambiata.");
									}
									count = 0; // the number of notes detected
									k = counterpoint_location - 1;
									while (count != 1 && k >= 0)
									{
										if (counterpoint.get_note(k) != null)
										{
											if (counterpoint.get_note(k).get_pitch() != 0)
												count++;
											else k--;
										}
										else k--;
										if (k < 0) count = 2; // to avoid going out of bounds
									}
									if (count == 1)
									{
										if (Math.abs( counterpoint.get_note(k).get_pitch() -
													  counterpoint.get_note(counterpoint_location).get_pitch()) > 2)
										errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
											" contains a dissonant interval between the cantus and the counterpoint" +
											" that is not approached with stepwise motion." +
											" Vertical intervals other than unisons, minor and major thirds, perfect" +
											" fifths, minor and major sixths, octaves and the multiples of these" +
											" intervals are only permitted on weak beats when they are" +
											" approached and left stepwise or are part of a double neighbour or" +
											" nota cambiata.");
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_unisons()
	// Check to see if the following rule is violated:
	// Vertical unisons are not permitted on the first beat of measures, except in the case of the first
	// and last measures, where they may be present.
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for unisons
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					if (counterpoint.get_note(counterpoint_location).get_pitch() ==
						cantus.get_note(cantus_location).get_pitch())
					{
						// Check if unison is inappropriately placed
						if (counterpoint_location % time_signature == 0 &&
							counterpoint_location >= time_signature &&
							counterpoint_location != total_sixteenth_notes - time_signature)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
								" contains an illegally placed vertical unison." +
								" Vertical unisons are not permitted on the first beat of measures, except" +
								" in the case of the first and last measures, where they may be present.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_nota_cambiata()
	// Check to see if the following rules are violated:
	// - Is only considered a nota cambiata if the second tone is dissonant
	// - The first tone of a nota cambiata must fall on the first beat of a measure
	//   if the second tone of the nota cambiata is dissonant.
	// - The first and third notes of a nota cambiata must be consonant.
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int counter1 = -1; // the index of the first of four notes in the counterpoint
		int counter2 = -1; // the index of the second of four notes in the counterpoint
		int counter3 = -1; // the index of the third of four notes in the counterpoint
		int counter4 = -1; // the index of the fourth of four notes in the counterpoint
		int cant1 = -1; // the index of the first of four notes in the cantus
		int cant2 = -1; // the index of the second of four notes in the cantus
		int cant3 = -1; // the index of the third of four notes in the cantus
		int cant4 = -1; // the index of the fourth of four notes in the cantus


		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Update the counter and cant notes
				counter1 = counter2;
				cant1= cant2;
				counter2 = counter3;
				cant2 = cant3;
				counter3 = counter4;
				cant3 = cant4;
				if (counterpoint.get_note(counterpoint_location).get_pitch() != 0)
					counter4 = counterpoint_location;
				else counter4 = -1;
				if (cantus.get_note(cantus_location).get_pitch() != 0)
					cant4 = cantus_location;
				else cant4 = -1;


				// Check if is a nota cambiata
				if (i == counterpoint_location && counterpoint.get_note(counterpoint_location).get_pitch() != 0
					&& counter1 != -1 && counter2 != -1 && counter3 != -1 && counter4 != -1)
				{
					if (is_nota_cambiata(counterpoint.get_note(counter1).get_pitch(),
					                     counterpoint.get_note(counter2).get_pitch(),
					                     counterpoint.get_note(counter3).get_pitch(),
						                 counterpoint.get_note(counter4).get_pitch()) == true)
					{
						if (counter1 % time_signature != 0)
						{
							if (interval_is_consonant(cantus.get_note(cant2).get_pitch(),
						                          counterpoint.get_note(counter2).get_pitch()) == false)
								errors_found.insert_new_error("WARNING: Measure " + ((counter1 / time_signature) + 1) +
									" contains the first tone of a nota cambiata that does not start on the first beat of the measure." +
									" The first tone of a nota cambiata should fall on the first beat of the measure.");
						}
						if (interval_is_consonant(cantus.get_note(cant1).get_pitch(),
						                          counterpoint.get_note(counter1).get_pitch()) == false)
						{
							if (interval_is_consonant(cantus.get_note(cant2).get_pitch(),
						                          counterpoint.get_note(counter2).get_pitch()) == false)
								errors_found.insert_new_error("ERROR: Measure " + ((counter1 / time_signature) + 1) +
									" contains the first tone of a nota cambiata with a dissonance on its first tone." +
									" The first and third tones of a nota cambiata must be consonant.");
						}
						if (interval_is_consonant(cantus.get_note(cant3).get_pitch(),
						                          counterpoint.get_note(counter3).get_pitch()) == false)
							errors_found.insert_new_error("ERROR: Measure " + ((counter3 / time_signature) + 1) +
								" contains the third note of a nota cambiata with a dissonance on its third tone." +
								" The first and third tones of a nota cambiata must be consonant.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_double_neighbour()
	// Check to see if the following rules are violated:
	// - Is only considered a double neighbour if the two middle tones are dissonant
	// - The first tone of a double neighbour must fall on the first beat of a measure
	// - The tone being decorated must be consonant.
	// - The motion immediately after a double neighbour must be stepwise and must continue
	//   the direction of motion.
	// - It is preferable for the upper neighbour to precede the lower neighbour.
	// - It is preferable if one of the neighbours is a semitone from the note being decorated.
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int counter1 = -1; // the index of the first of four notes in the counterpoint
		int counter2 = -1; // the index of the second of four notes in the counterpoint
		int counter3 = -1; // the index of the third of four notes in the counterpoint
		int counter4 = -1; // the index of the fourth of four notes in the counterpoint
		int cant1 = -1; // the index of the first of four notes in the cantus
		int cant2 = -1; // the index of the second of four notes in the cantus
		int cant3 = -1; // the index of the third of four notes in the cantus
		int cant4 = -1; // the index of the fourth of four notes in the cantus
		boolean[] results; // contains information about a double neighbour


		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Update the counter and cant notes
				counter1 = counter2;
				cant1= cant2;
				counter2 = counter3;
				cant2 = cant3;
				counter3 = counter4;
				cant3 = cant4;
				if (counterpoint.get_note(counterpoint_location).get_pitch() != 0)
					counter4 = counterpoint_location;
				else counter4 = -1;
				if (cantus.get_note(cantus_location).get_pitch() != 0)
					cant4 = cantus_location;
				else cant4 = -1;


				// Check if is a double neighbour
				if (i == counterpoint_location && counterpoint.get_note(counterpoint_location).get_pitch() != 0
					&& counter1 != -1 && counter2 != -1 && counter3 != -1 && counter4 != -1)
				{
					results = is_double_neighbour(counterpoint.get_note(counter1).get_pitch(),
												  counterpoint.get_note(counter2).get_pitch(),
												  counterpoint.get_note(counter3).get_pitch(),
												  counterpoint.get_note(counter4).get_pitch());

					if (interval_is_consonant(cantus.get_note(cant2).get_pitch(), counterpoint.get_note(counter2).get_pitch()) == true &&
					    interval_is_consonant(cantus.get_note(cant3).get_pitch(), counterpoint.get_note(counter3).get_pitch()) == true)
						results[0] = false; // do not count as a double neighbour if the middle 2 tones are consonant

					if (results[0] == true)
					{
						// Check that starts on first beat of measure
						if (counter1 % time_signature != 0)
						{
							errors_found.insert_new_error("ERROR: Measure " + ((counter1 / time_signature) + 1) +
								" contains the first tone of a double neighbour that does not start on the first beat of the measure." +
								" The first tone of a double neighbour must fall on the first beat of the measure.");
						}

						// Check that first and last note aren't dissonant
						if (interval_is_consonant(cantus.get_note(cant1).get_pitch(),
						                          counterpoint.get_note(counter1).get_pitch()) == false)
							errors_found.insert_new_error("ERROR: Measure " + ((counter1 / time_signature) + 1) +
								" contains the first tone of a double neighbour with a dissonance on the first statement of the" +
								" tone being decorated. Only the neighbour notes of a double neighbour" +
								" may be dissonant");
						if (interval_is_consonant(cantus.get_note(cant4).get_pitch(),
						                          counterpoint.get_note(counter4).get_pitch()) == false)
							errors_found.insert_new_error("ERROR: Measure " + ((counter4 / time_signature) + 1) +
								" contains the last tone of a double neighbour with a dissonance on the second statement of the" +
								" tone being decorated. Only the neighbour notes of a double neighbour" +
								" may be dissonant");


						// Check that the motion immediately after the double neighbour is stepwise and
						// continues the direction of motion
						int next_note = get_next_note(counter4, counterpoint);
						if (next_note != -1)
						{
							if (Math.abs(counterpoint.get_note(counter4).get_pitch()
								         - counterpoint.get_note(next_note).get_pitch()) > 2)
								errors_found.insert_new_error("ERROR: Measure " + ((counter4 / time_signature) + 1) +
									" contains the last note of a double neighbour that is followed by" +
									" a leap. The motion following a double neighbour must be stepwise.");
							if ( ( counterpoint.get_note(counter4).get_pitch() - counterpoint.get_note(counter3).get_pitch() > 0
							       && counterpoint.get_note(next_note).get_pitch() - counterpoint.get_note(counter4).get_pitch() < 0 ) ||
								 ( counterpoint.get_note(counter4).get_pitch() - counterpoint.get_note(counter3).get_pitch() < 0
							       && counterpoint.get_note(next_note).get_pitch() - counterpoint.get_note(counter4).get_pitch() > 0 ) )
								errors_found.insert_new_error("ERROR: Measure " + ((counter4 / time_signature) + 1) +
									" contains the last note of a double neighbour that is followed by" +
									" a change of direction. The motion following a double neighbour must" +
									" continue in the direction of the last motion of the double neighbour.");
						}


						// Check the type of double neighbour
						if (results[1] == false)
							errors_found.insert_new_error("WARNING: Measure " + ((counter2 / time_signature) + 1) +
								" contains the first note of a double neighbour where the lower neighbour" +
								" precedes the upper neighbour. While this is sometimes acceptable, it is" +
								" better to have the upper neighbour precede the lower neighbour.");


						// Check if one of the neighbour notes is a minor second from the tone being decorated
						if (results[2] == false)
							errors_found.insert_new_error("WARNING: Measure " + ((counter2 / time_signature) + 1) +
								" contains the first note of a double neighbour where neither neighbour" +
								" is a semitone from the note being derorated. It is preferable if one of the" +
								" neighbours is a semitone from the note being decorated.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_dissonant_neighbour_on_third_beat()
	// Check to see if the following rule is violated:
	// A dissonant neighbour note on the third beat of a measure should generally be avoided
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int next; // the index of the next note
		int prev; // the index of the previous note


		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Check that interval is dissonant and note falls on third beat
				if (i == counterpoint_location && i % time_signature == 8 &&
					interval_is_consonant(cantus.get_note(cantus_location).get_pitch(),
					                      counterpoint.get_note(counterpoint_location).get_pitch()) == false)
				{
					next = get_next_note(i, counterpoint);
					prev = get_previous_note(i, counterpoint);
					if ( ( Math.abs(counterpoint.get_note(i).get_pitch() - counterpoint.get_note(prev).get_pitch()) <= 2 ) &&
					     ( Math.abs(counterpoint.get_note(i).get_pitch() - counterpoint.get_note(next).get_pitch()) <= 2 ) &&
						 (( counterpoint.get_note(i).get_pitch() > counterpoint.get_note(prev).get_pitch() &&
						    counterpoint.get_note(i).get_pitch() > counterpoint.get_note(next).get_pitch() ) ||
						  ( counterpoint.get_note(i).get_pitch() < counterpoint.get_note(prev).get_pitch() &&
						    counterpoint.get_note(i).get_pitch() < counterpoint.get_note(next).get_pitch() )) )
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains a dissonant neighbour note on the third beat of the measure." +
							" Dissonant neighbour notes on the third beat of measures should generally" +
							" be avoided.");
				}
			}
		}
	}


	/***************************************************************************/


	public void check_distance_between_lines()
	// Check to see if the following rule is violated:
	// Intervals greater than perfect twelfths should generally be avoided
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for illegal vertical intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());


					// Check if a vertical interval is greater than a perfect 12th
					if (difference > 19)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains a large interval between the cantus and the counterpoint." +
							" Intervals greater than perfect twelfths are generally to be avoided.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_perfect_approach()
	// Check to see if the following rule is violated:
	// Perfect intervals should be approached with contrary or oblique motion.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_cantus_location = 0; // the location of the previous note of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint
		int prev_counterpoint_location = 0; // the location of the previous note of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0 && counterpoint.get_note(j).get_tied() == 1)
							{
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									prev_counterpoint = 0;
									prev_cantus = 0;
									prev_cantus_location = 0;
									prev_counterpoint_location = 0;


									// Find the previous counterpoint note and the cantus note corresponding
									// to this note
									prev_counterpoint_location = get_previous_note(j, counterpoint);
									if (prev_counterpoint_location != -1)
									{
										prev_counterpoint = counterpoint.get_note(prev_counterpoint_location).get_pitch();
										if (prev_counterpoint_location >= i)
											prev_cantus = cantus.get_note(i).get_pitch();
										else
										{
											prev_cantus_location = get_previous_note(i, cantus);
											if (prev_cantus_location != -1)
												prev_cantus = cantus.get_note(prev_cantus_location).get_pitch();
										}
									}


									// Generate an error if the motion isn't oblique or consonant
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										if (get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'c' &&
											get_type_of_motion(prev_cantus, cantus.get_note(i).get_pitch(), prev_counterpoint,
												counterpoint.get_note(j).get_pitch()) != 'o')
										{
											errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
												" contains a perfect interval that is not approached with the proper type of" +
												" motion. Perfect intervals should be approached with oblique or contrary motion.");
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_parallel_perfects()
	// Check to see if the following rule is violated:
	// Consecutive unisons, fifths and octaves are forbidden.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_difference; // the difference in pitch between the previous cantus and counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_cantus_location = 0; // the location of the previous note of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint
		int prev_counterpoint_location = 0; // the location of the previous note of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									prev_counterpoint = 0;
									prev_cantus = 0;
									prev_cantus_location = 0;
									prev_counterpoint_location = 0;


									// Find the previous counterpoint note and the cantus note corresponding
									// to this note
									prev_counterpoint_location = get_previous_note(j, counterpoint);
									if (prev_counterpoint_location != -1)
									{
										prev_counterpoint = counterpoint.get_note(prev_counterpoint_location).get_pitch();
										if (prev_counterpoint_location >= i)
											prev_cantus = cantus.get_note(i).get_pitch();
										else
										{
											prev_cantus_location = get_previous_note(i, cantus);
											if (prev_cantus_location != -1)
												prev_cantus = cantus.get_note(prev_cantus_location).get_pitch();
										}
									}


									// Generate an error
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										prev_difference = Math.abs(prev_counterpoint - prev_cantus);
										if (prev_difference == difference)
											errors_found.insert_new_error("ERROR: Measure " + ((i / time_signature) + 1) +
												" contains the second part of a parallel perfect interval." +
												" Consecutive unisons, fifths and octaves are forbidden.");
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_parallel_perfects_separated_by_one_tone()
	// Check to see if the following rule is violated:
	// Two vertical unisons, fifths or octaves separated by only one tone should be avoided.
	{
		int difference; // the difference in pitch between the cantus and the counterpoint
		int prev_difference; // the difference in pitch between the previous cantus and counterpoint
		int prev_cantus = 0; // the previous pitch of the cantus
		int prev_cantus_location = 0; // the location of the previous note of the cantus
		int prev_counterpoint = 0; // the previous pitch of the counterpoint
		int prev_counterpoint_location = 0; // the location of the previous note of the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Detect perfect intervals
								difference = Math.abs(counterpoint.get_note(j).get_pitch() - cantus.get_note(i).get_pitch());
								if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
								{
									prev_counterpoint = 0;
									prev_cantus = 0;
									prev_cantus_location = 0;
									prev_counterpoint_location = 0;


									// Find the 2nd previous counterpoint note and the cantus note corresponding
									// to this note
									prev_counterpoint_location = get_previous_note(j, counterpoint);
									if (prev_counterpoint_location != -1)
										prev_counterpoint_location = get_previous_note(prev_counterpoint_location, counterpoint);
									if (prev_counterpoint_location != -1)
									{
										prev_counterpoint = counterpoint.get_note(prev_counterpoint_location).get_pitch();
										if (prev_counterpoint_location >= i)
											prev_cantus = cantus.get_note(i).get_pitch();
										else
										{
											prev_cantus_location = get_previous_note(i, cantus);
											if (prev_cantus_location != -1)
												prev_cantus = cantus.get_note(prev_cantus_location).get_pitch();
										}
									}


									// Generate an error
									if (prev_cantus != 0 && prev_counterpoint != 0)
									{
										prev_difference = Math.abs(prev_counterpoint - prev_cantus);
										if (prev_difference == difference)
											errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
												" contains the second of two vertical unisons, fifths or" +
												" octaves that are only separated by one tone." +
												" Two vertical unisons, fifths or octaves should be avoided if" +
												" they are only separated by one tone.");
									}
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_successive_strong_beat_perfect_intervals()
	// Check to see if the following rule is violated:
	// Parallel octaves or fifths that occur on successive first beats should be avoided, regardless
	// of the notes separating them. Although these are permissible occasionally, they should not
	// be emphasized nor used frequently.
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int next_difference = 0; // the difference in pitch between the cantus and the counterpoint on the next
		                         // first beat
		int j;

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if (difference == 7 || difference == 12)
					{
						j = i + time_signature;
						if (j < total_sixteenth_notes)
						{
							if (counterpoint.get_note(j) != null && cantus.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0 && cantus.get_note(j).get_pitch() != 0)
								{
									next_difference = Math.abs(counterpoint.get_note(j).get_pitch() -
									                           cantus.get_note(j).get_pitch());
									if (next_difference == difference)
										errors_found.insert_new_error("WARNING: The first beat of measure " +
											((j / time_signature) + 1) + " contains the same vertical perfect interval" +
											" as the interval on the first beat of the preceding measure." +
											" Parallel octaves or fifths that occur on successive first beats" +
											" should be avoided, regardless of the notes separating them." +
											" Although these are permissible occasionally, they should not be" +
											" emphasized nor used frequently.");
								}
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_consecutive_vertical_intervals()
	// Check to see if the following rule is violated:
	// Should not have the same vertical perfect interval occurring on the first beats of more than two
	// consecutive measures.
	{
		int difference = 0; // the difference in pitch between the cantus and the counterpoint
		int last_interval = 0; // the difference of the last pair of notes which were compared
		int count = 0; // the number of times that an interval has occured consecutively

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null && cantus.get_note(i) != null)
			{
				if (counterpoint.get_note(i).get_pitch() != 0 && cantus.get_note(i).get_pitch() != 0)
				{
					difference = Math.abs(counterpoint.get_note(i).get_pitch() - cantus.get_note(i).get_pitch());
					if ( (difference == 0 || difference == 7 || difference == 12 || difference == 19) &&
					     difference == last_interval)
						count++;
					else count = 0;
					last_interval = difference;

					if (count > 1)
					{
						errors_found.insert_new_error("WARNING: Measure " + ((i / time_signature) + 1) +
							" contains number " + (count + 1) + " of a series of consecutive identical" +
							" perfect vertical intervals struck on strong beats. It is better not to" +
							" have more than" +
							" two strong beats in a row that hold the same vertical perfect interval.");
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_leaps_from_perfect()
	// Check to see if the following rule is violated:
	// If there is a perfect vertical interval on the first beat of a measure, then the
	// counterpoint note should not be leaped to or from. This is sometimes permissible
	// if the leap is small and the direction of motion is changed, but it should
	// generally be avoided.
	{
		int cantus_location = 0; // the location of the last note struck in the cantus
		int counterpoint_location = 0; // the location of the last note struck in the counterpoint
		int difference; // the difference in pitch between the cantus and the counterpoint

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (counterpoint.get_note(i) != null || cantus.get_note(i) != null)
			{
				// Update the pitches
				if (counterpoint.get_note(i) != null)
					counterpoint_location = i;
				if (cantus.get_note(i) != null)
					cantus_location = i;


				// Look for perfect intervals
				if (cantus.get_note(cantus_location).get_pitch() != 0 &&
					counterpoint.get_note(counterpoint_location).get_pitch() != 0 &&
					counterpoint_location % time_signature == 0)
				{
					difference = Math.abs(counterpoint.get_note(counterpoint_location).get_pitch() -
					                      cantus.get_note(cantus_location).get_pitch());
					if (difference == 0 || difference == 7 || difference == 12 || difference == 19)
					{
						// Find the next note
						int count = 0; // the number of notes detected
						int k = counterpoint_location + 1;
						while (count != 1 && k < total_sixteenth_notes) // find the note after the unison
						{
							if (counterpoint.get_note(k) != null)
							{
								if (counterpoint.get_note(k).get_pitch() != 0)
									count++;
								else k++;
							}
							else k++;
							if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
						}
						if (count == 1) // check if the unison is left by step
						{
							if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch() -
							              counterpoint.get_note(k).get_pitch() ) > 2)
								errors_found.insert_new_error("WARNING: The first beat of measure " +
									((i / time_signature) + 1) + " contains a vertical perfect interval" +
									" that is followed by a leap in the counterpoint." +
									" If there is a perfect vertical interval on the first beat of a measure," +
									" then the counterpoint note should not be leaped to or from. This is" +
									" sometimes permissible if the leap is small and the direction of motion" +
									" is changed, but it should generally be avoided.");
						}


						// Find the previous note
						count = 0;
						k = counterpoint_location - 1;
						while (count != 1 && k >= 0) // find the note after the unison
						{
							if (counterpoint.get_note(k) != null)
							{
								if (counterpoint.get_note(k).get_pitch() != 0)
									count++;
								else k--;
							}
							else k--;
							if (k < 0) count = 2; // to avoid going out of bounds
						}
						if (count == 1) // check if the unison is left in the proper direction
						{
							if (Math.abs( counterpoint.get_note(counterpoint_location).get_pitch() -
							              counterpoint.get_note(k).get_pitch() ) > 2)
								errors_found.insert_new_error("WARNING: The first beat of measure " +
									((i / time_signature) + 1) + " contains a vertical perfect interval" +
									" that is approached by leap in the counterpoint." +
									" If there is a perfect vertical interval on the first beat of a measure," +
									" then the counterpoint note should not be leaped to or from. This is" +
									" sometimes permissible if the leap is small and the direction of motion" +
									" is changed, but it should generally be avoided.");
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_simultaneous_climaxes()
	// Check to see if the following rule is violated:
	// The climax may not occur at the same time in both the cantus and the counterpoint
	{
		if (get_climax_position(cantus) == get_climax_position(counterpoint))
			errors_found.insert_new_error("WARNING: Both the cantus firmus and the counterpoint have" +
				" their climaxes occuring at the same place. The lines" +
				" should have their climaxes occuring at different locations.");
	}


	/***************************************************************************/


	public void check_voice_crossing()
	// Check to see if the following rule is violated:
	// A note in the bottom line may never be above the corresponding note of the top line and vice versa
	{
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								// Deal with case where cantus is the upper voice
								if ( counterpoint.get_type_of_line() % 2 == 0 )
								{
									if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() < 0 )
										errors_found.insert_new_error("ERROR: There is voice crossing in measure " +
											((i / time_signature) + 1) + ". A note in the bottom line may never be" +
											" above the corresponding note of the top line and vice versa.");
								}


								// Deal with case where cantus is lower voice
								else if ( cantus.get_note(i).get_pitch() - counterpoint.get_note(j).get_pitch() > 0 )
									errors_found.insert_new_error("ERROR: There is voice crossing in measure " +
										((i / time_signature) + 1) + ". A note in the bottom line may never be" +
										" above the corresponding note of the top line and vice versa.");
							}
						}
					}
				}
			}
		}
	}


	/***************************************************************************/


	public void check_voice_overlap()
	// Check to see if the following rule is violated:
	// A note in the bottom line cannot move up past where the last note of the top line was just
	// previously and vice versa
	{
		int prev_cantus = -1; // the previous note of the cantus
		int prev_counterpoint = -1; // the previous note of the counerpoint
		boolean found_one = false; // set to true when the first counterpoint note in a measure is found
		boolean error_found = false; // set to true if voice overlap is found

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			error_found = false;
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					if (prev_cantus == -1)
					{
						prev_cantus = cantus.get_note(i).get_pitch();
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}
					}
					else
					{
						// Check cantus for voice overlap in case where cantus is the upper voice
						if ( counterpoint.get_type_of_line() % 2 == 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint < 0 )
								error_found = true;
						}


						// Check cantus for voice overlap in case where cantus is the lower voice
						else if ( counterpoint.get_type_of_line() % 2 != 0 )
						{
							if ( cantus.get_note(i).get_pitch() - prev_counterpoint > 0 )
								error_found = true;
						}


						// Check counterpoint for voice overlap
						found_one = false;
						for (int j = i; j < i + cantus.get_note(i).get_duration(); j++)
						{
							if (counterpoint.get_note(j) != null)
							{
								if (counterpoint.get_note(j).get_pitch() != 0)
								{
									if (found_one == false)
									{
										// Check counterpoint for voice overlap in case where cantus is the upper voice
										if ( counterpoint.get_type_of_line() % 2 == 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus > 0 )
												error_found = true;
										}


										// Check counterpoint for voice overlap in case where cantus is the lower voice
										else if ( counterpoint.get_type_of_line() % 2 != 0 )
										{
											if ( counterpoint.get_note(i).get_pitch() - prev_cantus < 0 )
												error_found = true;
										}
									}
									found_one = true;
									prev_counterpoint = counterpoint.get_note(j).get_pitch();
								}
							}
						}
						prev_cantus = cantus.get_note(i).get_pitch();
					}
				}
			}

			if (error_found == true)
				errors_found.insert_new_error("ERROR: There is voice overlap in measure " + ((i / time_signature) + 1) +
					".  A note in the bottom line cannot move up past where the last note of the top line was just previously" +
					" and vice versa.");
		}
	}


	/***************************************************************************/


	public void check_beginning_interval()
	// Check to see if the following rule is violated:
	// The first vertical interval must be a P1, P5, P8 or P12. A P5 or P12 are only allowed when the counterpoint is
	// above the cantus.
	{
		int cantus_pitch = -1; // the first pitch in the cantus
		int counterpoint_pitch = -1; // the first pitch in the counterpoint


		// Find the first vertical interval
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (cantus.get_note(i) != null)
			{
				if (cantus.get_note(i).get_pitch() != 0)
				{
					cantus_pitch = cantus.get_note(i).get_pitch();

					for (int j = 0; j < i + cantus.get_note(i).get_duration(); j++)
					{
						if (counterpoint.get_note(j) != null)
						{
							if (counterpoint.get_note(j).get_pitch() != 0)
							{
								counterpoint_pitch = counterpoint.get_note(j).get_pitch();
								j = i + cantus.get_note(i).get_duration();
							}
						}
					}

					i = total_sixteenth_notes;
				}
			}
		}
		int interval = counterpoint_pitch - cantus_pitch; // the interval between the first two notes


		// Detect a missign first inteval
		if (cantus_pitch == -1 || counterpoint_pitch == -1)
			errors_found.insert_new_error("ERROR: The line has an improper first interval.");


		// Detect errors when the cantus is the upper voice
		else if ( counterpoint.get_type_of_line() % 2 == 0 )
		{
			if (interval != 0 && interval != -12)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the upper voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or an octave below it.");
			}
		}


		// Detect errors when the cantus is the lower voice
		else if ( counterpoint.get_type_of_line() % 2 != 0 )
		{
			if (interval != 0 && interval != 7 && interval != 12 && interval != 19)
			{
				errors_found.insert_new_error("ERROR: The line has an improper first interval. When the cantus is" +
					" the lower voice, the first note of the counterpoint must be in unison with the first note" +
					" of the cantus or a fifth or an octave above it.");
			}
		}
	}


	/***************************************************************************/


	public void check_last_interval()
	// Check to see if the following rule is violated:
	// The interval of the notes struck on the first beat of the last measure of the piece must be a P1 or a P8.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int difference; // the interval between the two voices

		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			difference = Math.abs(counterpoint.get_note(index).get_pitch() - cantus.get_note(index).get_pitch());
			if (difference != 0 && difference != 12)
				errors_found.insert_new_error("ERROR: The vertical interval on the first beat of the last measure" +
					" is incorrect. It must be either a unison or an octave.");
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/***************************************************************************/


	public void check_last_approach()
	// Check to see if the following rule is violated:
	// The notes on the first beat of the last measure must be approached with contrary motion.
	{
		int index = total_sixteenth_notes - time_signature; // the index of the first beat of the last measure
		int last_cantus; // the pitch of the last note of the cantus
		int last_counterpoint; // the pitch of the last note of the counterpoint
		int approach_cantus = 0; // the pitch of the second to last note of the cantus
		int approach_counterpoint = 0; // the pitch of the second to last note of the counterpoint

		if (counterpoint.get_note(index) != null && cantus.get_note(index) != null)
		{
			last_cantus = cantus.get_note(index).get_pitch();
			last_counterpoint = counterpoint.get_note(index).get_pitch();


			// Find the second to last note of the counterpoint
			int count = 0; // the number of notes detected
			int k = index - 1;
			while (count != 1 && k >= 0)
			{
				if (counterpoint.get_note(k) != null)
				{
					if (counterpoint.get_note(k).get_pitch() != 0)
					{
						approach_counterpoint = counterpoint.get_note(k).get_pitch();
						count++;
					}
					else k--;
				}
				else k--;
				if (k < index - time_signature) count = 2; // to avoid going out of bounds
			}
			if (count == 1) // find the second to last note of the cantus
			{
				count = 0;
				k = index - 1;
				while (count != 1 && k < total_sixteenth_notes)
				{
					if (cantus.get_note(k) != null)
					{
						if (cantus.get_note(k).get_pitch() != 0)
						{
							approach_cantus = cantus.get_note(k).get_pitch();
							count++;
						}
						else k--;
					}
					else k--;
					if (k < index - time_signature) count = 2; // to avoid going out of bounds
				}
				if (count == 1)
				{
					if ( get_type_of_motion(approach_cantus, last_cantus, approach_counterpoint,
					                        last_counterpoint) != 'c')
						errors_found.insert_new_error("ERROR: The notes on the first beat of the last" +
							" measure are approached incorrectly. They must be approached with contrary motion.");
				}
			}
		}
		else
			errors_found.insert_new_error("ERROR: Notes must be struck in both the cantus and the counterpoint" +
				" on the first beat of the last measure.");
	}


	/* PRIVATE METHODS *********************************************************/


	private boolean[] is_double_neighbour(int pitch1, int pitch2, int pitch3, int pitch4)
	// Takes in four pitches and returns an array with 3 cells.
	// The first cell is set to true if the four notes outline a double neighbour and false if it doesn't
	// The second cell is set to true if the upper neigbour precedes the lower neighbour
	//	and to false if the lower neighbour precedes the upper neighbour
	// The third cell is set to true if at least one of the neighbour notes is a semitone away from
	//	the note being decorated, and false otherwise.
	{
		boolean[] results = new boolean[3];
		results[0] = false;
		results[1] = false;
		results[2] = false;

		if (pitch1 == pitch4)
		{
			if ( ((pitch2 - pitch1 == 1) || (pitch2 - pitch1 == 2)) &&
			     ((pitch3 - pitch2 == -3) || (pitch3 - pitch2 == -4)) &&
				 ((pitch4 - pitch3 == 1) || (pitch4 - pitch3 == 2)) )
			{
				results[0] = true;
				results[1] = true;
			}
			else if ( ((pitch2 - pitch1 == -1) || (pitch2 - pitch1 == -2)) &&
			          ((pitch3 - pitch2 == 3) || (pitch3 - pitch2 == 4)) &&
				      ((pitch4 - pitch3 == -1) || (pitch4 - pitch3 == -2)) )
			{
				results[0] = true;
				results[1] = false;
			}
		}
		else results[0] = false;

		if ( Math.abs(pitch2 - pitch1) == 1 || Math.abs(pitch4 - pitch3) == 1 )
			results[2] = true;
		else
			results[2] = false;

		return results;
	}


	/***************************************************************************/


	private boolean is_nota_cambiata(int pitch1, int pitch2, int pitch3, int pitch4)
	// Takes in four pitches and returns true if they outline a nota cambiata
	{
		if ( ( ((pitch2 - pitch1 == 1) || (pitch2 - pitch1 == 2)) &&
		       ((pitch3 - pitch2 == 3) || (pitch3 - pitch2 == 4)) &&
		       ((pitch4 - pitch3 == -1) || (pitch4 - pitch3 == -2)) ) ||
		     ( ((pitch2 - pitch1 == -1) || (pitch2 - pitch1 == -2)) &&
		       ((pitch3 - pitch2 == -3) || (pitch3 - pitch2 == -4)) &&
		       ((pitch4 - pitch3 == 1) || (pitch4 - pitch3 == 2)) ) )
			return true;
		else
			return false;
	}


	/***************************************************************************/


	private boolean interval_is_consonant(int cantus_note, int counterpoint_note)
	// Returns true if the vertical interval between the two given notes is consonant
	{
		int difference = Math.abs(cantus_note - counterpoint_note);
		difference = difference % 12;
		if (difference != 0 && difference != 3 && difference != 4 && difference != 7 && difference != 8 &&
						difference != 9)
			return false;
		else
			return true;
	}


	/***************************************************************************/


	private int get_number_notes(Voice voice_to_check)
	// Return the number of non-rest notes struck in the given Voice
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
					number_of_notes++;
			}
		}
		return number_of_notes;
	}


	/***************************************************************************/


	private int get_climax_position(Voice voice_to_check)
	// Return the position of the climax in the voice_to_check voice. If the climax is repeated, the position
	// of its first occurence is returned. The climax is found to be the highest note of the line or, if the
	// first and last notes of the line are the hightes, the lowest note of the line.
	{
		int number_of_notes = 0; // the total number of notes that are struck in the piece
		int notes_so_far = 0; // the number of notes that have been detected so far
		int high_or_low = 0; // 1 means the climax is the highest note, -1 means it's the lowest, 0 indicates an error
		int climax_note = 0; // the pitch of the climax note of the line
		int climax_note_position = 0; // the location of the climax note


		// Determine whether the number of notes in the line
		number_of_notes = get_number_notes(voice_to_check);


		// Determine whether the climax is the lowest note or the highest note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (notes_so_far == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						notes_so_far++;
					}
					else if (notes_so_far < number_of_notes - 1 && voice_to_check.get_note(i).get_pitch() >= climax_note)
					{
						high_or_low = 1;
						i = total_sixteenth_notes; // exit the loop
					}
					else if (notes_so_far == number_of_notes - 1 && high_or_low != 1 &&
					         voice_to_check.get_note(i).get_pitch() == climax_note)
					{
						high_or_low = -1;
						i = total_sixteenth_notes; // exit the loop
					}
					else
						notes_so_far++;
				}
			}
		}
		notes_so_far = 0;
		climax_note = 0;


		// Find the pitch and position of the climax note
		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			if (voice_to_check.get_note(i) != null)
			{
				if (voice_to_check.get_note(i).get_pitch() != 0)
				{
					if (climax_note == 0)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == 1 && voice_to_check.get_note(i).get_pitch() > climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
					else if (high_or_low == -1 && voice_to_check.get_note(i).get_pitch() < climax_note)
					{
						climax_note = voice_to_check.get_note(i).get_pitch();
						climax_note_position = i;
					}
				}
			}
		}


		// Return the position of the climax
		return climax_note_position;
	}


	/***************************************************************************/


	private char get_type_of_motion(int v1_first_pitch, int v1_second_pitch,
	                                int v2_first_pitch, int v2_second_pitch)
	// Return whether the motion from pitch v1_first_pitch to v1_second_pitch in one voice
	// and from v2_first_pitch to v2_second_pitch in the other voice is similar, parallel,
	// contrary or oblique. 's' means similar, 'p' means parallel, 'c' means contrary and
	// 'o' means oblique.
	// NOTE: the definition of parallel motion used here includes only exactly parallel motion.
	//       Movement from a minor third to a major third, for example, is considered similar.
	//       This diverges from some definitions of parallel motion.
	{
		if (v1_first_pitch == v1_second_pitch || v2_first_pitch == v2_second_pitch)
			return 'o';
		else if ( (v1_first_pitch - v1_second_pitch > 0 && v2_first_pitch - v2_second_pitch < 0) ||
		          (v1_first_pitch - v1_second_pitch < 0 && v2_first_pitch - v2_second_pitch > 0) )
			return 'c';
		else if (v1_first_pitch - v2_first_pitch == v1_second_pitch - v2_second_pitch)
			return 'p';
		else
			return 's';
	}


	/***************************************************************************/


	private int get_next_note(int given_index, Voice given_voice)
	// Get the index of the first non-rest note after given_index in given_voice
	// Does not count notes that are tied back
	// Returns -1 if this note cannot be found
	{
		int k = given_index + 1; // the index of the next note
		int count = 0; // the number of notes detected

		while (count != 1 && k < total_sixteenth_notes)
		{
			if (given_voice.get_note(k) != null)
			{
				if (given_voice.get_note(k).get_pitch() != 0 &&
					( given_voice.get_note(k).get_tied() == 1 || given_voice.get_note(k).get_tied() == 3 ))
					count++;
				else k++;
			}
			else k++;
			if (k == total_sixteenth_notes) count = 2; // to avoid going out of bounds
		}

		if (count == 1)
			return k;
		else
			return -1;
	}


	/***************************************************************************/


	private int get_previous_note(int given_index, Voice given_voice)
	// Get the index of the first non-rest note before given_index in given_voice
	// Does not count notes that are tied back
	// Returns -1 if this note cannot be found
	{
		int k = given_index - 1; // the index of the previous note
		int count = 0; // the number of notes detected

		while (count != 1 && k >= 0)
		{
			if (given_voice.get_note(k) != null)
			{
				if (given_voice.get_note(k).get_pitch() != 0 &&
					( given_voice.get_note(k).get_tied() == 1 || given_voice.get_note(k).get_tied() == 3 ))
					count++;
				else k--;
			}
			else k--;
			if (k < 0) count = 2; // to avoid going out of bounds
		}

		if (count == 1)
			return k;
		else
			return -1;
	}
}