/*
SpeciesChecker
by Cory McKay

CLASS NAME:		Composition
CLASS TYPE:		Fundamental data structure
DESCRIPTION:	Objects of this class represent full musical pieces with one to four voices.
				Appropriate for use in species counterpoint and simple harmony.
				See bottom of file for a key to the field contents.
LIMITATION:		Unable to differentiate between key signatures that are enharmonically equivalent.
				These are Db maj / C # maj, Bb min / A# min, Gb maj / F# maj, Eb min / D# min,
				B maj / Cb maj, G# min / Ab min
*/


class Composition
{
	/* FIELDS ******************************************************************/


	private int type_of_composition;	// the type of composition
	private int quality;				// whether the composition's home key is major or minor
	private int key_signature;			// the key signature of the composition
	private int time_signature;			// the time signature of the composition
	private int number_of_bars;			// the total number of bars in the composition
	private int total_sixteenth_notes;	// the total number of 16th notes in a line
	private Voice[] voices;				// the voices that are contained in the composition


	/* CONSTRUCTOR *************************************************************/


	Composition(int typ, int qual, int key, int tim, int num)
	// Automatically sets all voices to null, as there are not any at initiation
	{
		type_of_composition = typ;
		quality = qual;
		key_signature = key;
		time_signature = tim;
		number_of_bars = num;


		// Calculate the total number of sixteenth notes in the piece
		total_sixteenth_notes = number_of_bars * time_signature;


		// Set up the proper amount of voices for the type of composition
		// and initialize them to null
		if (type_of_composition == 0)
		{
			voices = new Voice[1];
			voices[0] = null;
		}
		else if (type_of_composition == 6)
		{
			voices = new Voice[4];
			voices[0] = null;
			voices[1] = null;
			voices[2] = null;
			voices[3] = null;
		}
		else
		{
			voices = new Voice[2];
			voices[0] = null;
			voices[1] = null;
		}
	}


	/* PUBLIC METHODS **********************************************************/


	public int get_type_of_composition()
	// Returns the type of composition
	{
		return type_of_composition;
	}


	/***************************************************************************/


	public int get_quality()
	// Return whether the composition is major or minor
	{
		return quality;
	}


	/***************************************************************************/


	public int get_key_signature()
	// Returns the key signature
	{
		return key_signature;
	}


	/***************************************************************************/


	public int get_time_signature()
	// Returns the time signature
	{
		return time_signature;
	}


	/***************************************************************************/


	public int get_number_of_bars()
	// Returns the total number of bars in the piece
	{
		return number_of_bars;
	}


	/***************************************************************************/


	public int get_total_sixteenth_notes()
	// Returns the total number of 16th note units in the piece
	{
		return total_sixteenth_notes;
	}


	/***************************************************************************/


	public Voice get_voice(int index)
	// Returns the requested voice object
	{
		return voices[index];
	}


	/***************************************************************************/


	public int get_number_of_voices()
	// Returns the number of voices that the voices array can hold
	{
		return voices.length;
	}


	/***************************************************************************/


	public void insert_voice(int index, int type_of_line, int clef)
	// Inserts an empty voice into the composition
	// NOTE: this method overwrites any voice at the given index
	{
		voices[index] = new Voice(type_of_line, clef, total_sixteenth_notes);
	}


	/***************************************************************************/


	public void insert_ready_voice(int index, Voice voice_to_add)
	// Inserts a voice into the given index
	// NOTE: this method overwrites any voice at the given index
	{
		voices[index] = voice_to_add;
	}
}


/*
KEY

type_of_composition
	0 = cantus only
	1 = first species
	2 = second species
	3 = third species
	4 = forth species
	5 = fifth species
	6 = four-part composition

quality
	1 = major
	2 = minor

key_signature
	1 = C major / A minor
	2 = Db major / Bb minor (C# major / A# minor)
	3 = D major / B minor
	4 = Eb major / C minor
	5 = E major / C# minor
	6 = F major / D minor
	7 = Gb major / Eb minor (F# major / D# minor)
	8 = G major / E minor
	9 = Ab major / F minor
	10 = A major / F# minor
	11 = Bb major / G minor
	12 = B major / G# minor (Cb major / Ab minor)

time_signature (note that the number indicates number of 16th note units per measure)
	8 = 2/4
	12 = 3/4
	16 = 4/4
	24 = 12/8

number_of_bars
	# = how many bars are in the composition in total

total_sixteenth_notes
	# = the total number of sixteenth note rhythmic units in a line

voices[i]
	pointer to a Voice = the voice in the ith position
	null = the voice i is currently empty
*/