/*
SpeciesChecker
by Cory McKay

CLASS NAME:		ControllerCantus
CLASS TYPE:		Applier of analysis rules
DESCRIPTION:	Objects of this class oversee the application of rules relating to cantus firmi
				contained in Composition objects. The rules that are applied are contained in
				AnalyzerHorizontalCantus objects. ControllerCantus objects should be eliminated
				after the check_cantus() method has been called.
*/


class ControllerCantus
{
	/* FIELDS ******************************************************************/


	private Voice cantus;								// the piece to be checked
	private ErrorList errors_found;						// the list of errors found
	private AnalyzerHorizontalCantus horizontal_rules;	// an object containing the rules to be applied


	/* CONSTRUCTOR *************************************************************/


	ControllerCantus(Composition piece)
	// piece should contain a cantus
	{
		// Set cantus to the given cantus, or set it to null if given Composition does not contain a cantus
		cantus = null;
		for (int i = 0; i < piece.get_number_of_voices(); i++)
		{
			if (piece.get_voice(i).get_type_of_line() == 0)
			{
				cantus = piece.get_voice(i);
				i = 4; // exit the loop
			}
		}

		
		// Create the list used to store errors
		errors_found = new ErrorList();
		

		// Create the class containing the rules to be applied
		if (cantus != null)
		{
			horizontal_rules = new AnalyzerHorizontalCantus(cantus, errors_found, piece.get_quality(),
									piece.get_key_signature(), piece.get_time_signature(),
									piece.get_number_of_bars(), piece.get_total_sixteenth_notes());
		}
	}


	/* PUBLIC METHODS **********************************************************/


	public ErrorList check_cantus(int[] rules_to_be_applied)
	// Performs the analysis of the cantus and returns the list of errors that were found.
	// Only the rules indicated in rules_to_be_applied are applied.
	{
		// Return the appropriate error if the given piece does not contain a cantus
		if (cantus == null)
		{
			errors_found.insert_new_error("The given music does not have a cantus firmus.");
			return errors_found;
		}


		// Analyze the cantus and add error information to errors_found
		if (rules_to_be_applied[0] == 1)
			horizontal_rules.check_key();
		if (rules_to_be_applied[1] == 1)
			horizontal_rules.check_note_durations();
		if (rules_to_be_applied[2] == 1)
			horizontal_rules.check_note_placement();
		if (rules_to_be_applied[3] == 1)
			horizontal_rules.check_tied_notes();
		if (rules_to_be_applied[4] == 1)
			horizontal_rules.check_rests();
		if (rules_to_be_applied[5] == 1)
			horizontal_rules.check_range();
		if (rules_to_be_applied[6] == 1)
			horizontal_rules.check_repeated_note();
		if (rules_to_be_applied[7] == 1)
			horizontal_rules.check_horizontal_intervals();
		if (rules_to_be_applied[8] == 1)
			horizontal_rules.check_leading_tone_leap();
		if (rules_to_be_applied[9] == 1)
			horizontal_rules.check_total_number_of_leaps();
		if (rules_to_be_applied[10] == 1)
			horizontal_rules.check_a_large_leap();
		if (rules_to_be_applied[11] == 1)
			horizontal_rules.check_total_number_of_large_leaps();
		if (rules_to_be_applied[12] == 1)
			horizontal_rules.check_number_consecutive_leaps();
		if (rules_to_be_applied[13] == 1)
			horizontal_rules.check_consecutive_leaps_in_same_direction();
		if (rules_to_be_applied[14] == 1)
			horizontal_rules.check_leap_filled_in();
		if (rules_to_be_applied[15] == 1)
			horizontal_rules.check_large_leap_change_direction();
		if (rules_to_be_applied[16] == 1)
			horizontal_rules.check_change_direction_after_leap();
		if (rules_to_be_applied[17] == 1)
			horizontal_rules.check_first_note_is_tonic();
		if (rules_to_be_applied[18] == 1)
			horizontal_rules.check_last_note_is_tonic();
		if (rules_to_be_applied[19] == 1)
			horizontal_rules.compare_first_and_last_notes();
		if (rules_to_be_applied[20] == 1)
			horizontal_rules.check_second_to_last_note();
		if (rules_to_be_applied[21] == 1)
			horizontal_rules.check_last_note_approached_stepwise();
		if (rules_to_be_applied[22] == 1)
			horizontal_rules.check_raised_sixth_and_seventh_in_minor();
		if (rules_to_be_applied[23] == 1)
			horizontal_rules.check_climax();
		if (rules_to_be_applied[24] == 1)
			horizontal_rules.check_note_occurs_too_often();
		if (rules_to_be_applied[25] == 1)
			horizontal_rules.check_note_occurs_too_often_strict();
		if (rules_to_be_applied[26] == 1)
			horizontal_rules.check_interval_of_series_of_notes();
		if (rules_to_be_applied[27] == 1)
			horizontal_rules.check_too_may_stepwise();
		if (rules_to_be_applied[28] == 1)
			horizontal_rules.check_too_many_in_same_direction();
		if (rules_to_be_applied[29] == 1)
			horizontal_rules.check_sufficient_changes_of_direction();
		if (rules_to_be_applied[30] == 1)
			horizontal_rules.check_repetition_of_groups_of_tones();
		if (rules_to_be_applied[31] == 1)
			horizontal_rules.check_ornamentation();
		if (rules_to_be_applied[32] == 1)
			horizontal_rules.check_length();

		
		horizontal_rules = null;


		// Return the list of errors that was found
		return errors_found;
	}
}