/*
SpeciesChecker
by Cory McKay

CLASS NAME:		ControllerSecond
CLASS TYPE:		Applier of analysis rules
DESCRIPTION:	Objects of this class oversee the application of rules relating to counterpoint
				contained in Composition objects. The rules that are applied are contained in
				AnalyzerHorizontalSecond and AnalyzerVerticalSecond objects. ControllerSecond objects
				should be eliminated after the check_counterpoint() method has been called.
*/


class ControllerSecond
{
	/* FIELDS ******************************************************************/


	private Voice counterpoint;							// the voice to be checked
	private Voice cantus;								// the cantus corresponding to the counterpoint
	private ErrorList errors_found;						// the list of errors found
	private AnalyzerHorizontalSecond horizontal_rules;	// an object containing the horizontal rules to be applied
	private AnalyzerVerticalSecond vertical_rules;		// an object containing the vertical rules to be applied


	/* CONSTRUCTOR *************************************************************/


	ControllerSecond(Composition piece)
	// piece should contain a cantus and a counterpoint
	{
		// Set cantus to the given cantus, or set it to null if given Composition does not contain a cantus
		// Set counterpoint to the given counterpoint, or set it to null if given Composition does not contain
		// a counterpoint.
		cantus = null;
		counterpoint = null;
		for (int i = 0; i < piece.get_number_of_voices(); i++)
		{
			if (piece.get_voice(i).get_type_of_line() == 0)
			{
				cantus = piece.get_voice(i);
			}
			if (piece.get_voice(i).get_type_of_line() == 3 || piece.get_voice(i).get_type_of_line() == 4)
			{
				counterpoint = piece.get_voice(i);
			}
		}

		
		// Create the list used to store errors
		errors_found = new ErrorList();
		

		// Create the classes containing the rules to be applied
		if (cantus != null && counterpoint != null)
		{
			horizontal_rules = new AnalyzerHorizontalSecond(counterpoint, errors_found, piece.get_quality(),
									piece.get_key_signature(), piece.get_time_signature(),
									piece.get_number_of_bars(), piece.get_total_sixteenth_notes());
			vertical_rules = new AnalyzerVerticalSecond(cantus, counterpoint, errors_found, piece.get_quality(),
									piece.get_key_signature(), piece.get_time_signature(),
									piece.get_number_of_bars(), piece.get_total_sixteenth_notes());
		}
	}


	/* PUBLIC METHODS **********************************************************/


	public ErrorList check_counterpoint(int[] rules_to_be_applied)
	// Performs the analysis of the counterpoint and returns the list of errors that were found.
	// Only the rules indicated in rules_to_be_applied are applied.
	{
		// Return the appropriate error if the given piece does not contain a cantus or a counterpoint
		if (cantus == null)
		{
			errors_found.insert_new_error("The given music does not have a cantus firmus.");
			return errors_found;
		}
		if (counterpoint == null)
		{
			errors_found.insert_new_error("The given music does not have a counterpoint.");
			return errors_found;
		}


		// Analyze the counterpoint and add error information to errors_found
		if (rules_to_be_applied[0] == 1)
			horizontal_rules.check_key();
		if (rules_to_be_applied[1] == 1)
			horizontal_rules.check_note_durations();
		if (rules_to_be_applied[2] == 1)
			horizontal_rules.check_note_placement();
		if (rules_to_be_applied[3] == 1)
			horizontal_rules.check_tied_notes();
		if (rules_to_be_applied[4] == 1)
			horizontal_rules.check_rests();
		if (rules_to_be_applied[5] == 1)
			horizontal_rules.check_range();
		if (rules_to_be_applied[6] == 1)
			horizontal_rules.check_repeated_note();
		if (rules_to_be_applied[7] == 1)
			horizontal_rules.check_horizontal_intervals();
		if (rules_to_be_applied[8] == 1)
			horizontal_rules.check_leading_tone_leap();
		if (rules_to_be_applied[9] == 1)
			horizontal_rules.check_a_large_leap();
		if (rules_to_be_applied[10] == 1)
			horizontal_rules.check_total_number_of_large_leaps();
		if (rules_to_be_applied[11] == 1)
			horizontal_rules.check_number_consecutive_leaps();
		if (rules_to_be_applied[12] == 1)
			horizontal_rules.check_consecutive_leaps_in_same_direction();
		if (rules_to_be_applied[13] == 1)
			horizontal_rules.check_leaps_accross_measure_lines();
		if (rules_to_be_applied[14] == 1)
			horizontal_rules.check_leap_filled_in();
		if (rules_to_be_applied[15] == 1)
			horizontal_rules.check_fourths_properly_treated();
		if (rules_to_be_applied[16] == 1)
			horizontal_rules.check_large_leap_change_direction();
		if (rules_to_be_applied[17] == 1)
			horizontal_rules.check_last_note_is_tonic();
		if (rules_to_be_applied[18] == 1)
			horizontal_rules.compare_first_and_last_notes();
		if (rules_to_be_applied[19] == 1)
			horizontal_rules.check_second_to_last_note();
		if (rules_to_be_applied[20] == 1)
			horizontal_rules.check_last_note_approached_stepwise();
		if (rules_to_be_applied[21] == 1)
			horizontal_rules.check_raised_sixth_and_seventh_in_minor();
		if (rules_to_be_applied[22] == 1)
			horizontal_rules.check_climax();
		if (rules_to_be_applied[23] == 1)
			horizontal_rules.check_interval_of_series_of_notes();
		if (rules_to_be_applied[24] == 1)
			horizontal_rules.check_too_may_stepwise();
		if (rules_to_be_applied[25] == 1)
			horizontal_rules.check_too_many_in_same_direction();
		if (rules_to_be_applied[26] == 1)
			horizontal_rules.check_sufficient_changes_of_direction();
		if (rules_to_be_applied[27] == 1)
			horizontal_rules.check_repetition_of_groups_of_tones();
		if (rules_to_be_applied[28] == 1)
			horizontal_rules.check_ornamentation();
		if (rules_to_be_applied[29] == 1)
			vertical_rules.check_vertical_intervals();
		if (rules_to_be_applied[30] == 1)
			vertical_rules.check_unisons();
		if (rules_to_be_applied[31] == 1)
			vertical_rules.check_perfect_twelvthes();
		if (rules_to_be_applied[32] == 1)
			vertical_rules.check_perfect_approach();
		if (rules_to_be_applied[33] == 1)
			vertical_rules.check_successive_strong_beat_perfect_intervals();
		if (rules_to_be_applied[34] == 1)
			vertical_rules.check_consecutive_vertical_intervals();
		if (rules_to_be_applied[35] == 1)
			vertical_rules.check_leaps_from_perfect();
		if (rules_to_be_applied[36] == 1)
			vertical_rules.check_simultaneous_climaxes();
		if (rules_to_be_applied[37] == 1)
			vertical_rules.check_voice_crossing();
		if (rules_to_be_applied[38] == 1)
			vertical_rules.check_voice_overlap();
		if (rules_to_be_applied[39] == 1)
			vertical_rules.check_beginning_interval();
		if (rules_to_be_applied[40] == 1)
			vertical_rules.check_last_interval();
		if (rules_to_be_applied[41] == 1)
			vertical_rules.check_last_approach();


		// Erase the Analyzer objects
		horizontal_rules = null;
		vertical_rules = null;


		// Return the list of errors that was found
		return errors_found;
	}
}