/*
SpeciesChecker
by Cory McKay

CLASS NAME:		FReader
CLASS TYPE:		File I/O
DESCRIPTION:	Used to read and parse music saved as SpeciesChecker files.
				Appropriate for use in species counterpoint and simple harmony.
				See FormatSavedFile.txt guide for explanation of file format.
*/


import java.io.*;
import java.util.StringTokenizer;


class FReader
{
	/* FIELDS ******************************************************************/


	private char[] chars;				// an array that holds all of the characters that are read in through
										// the file
	private int next_index;				// the current location in the chars array
	private Composition opened_piece;	// the composition that is currently being read and parsed


	/* CONSTRUCTOR *************************************************************/


	FReader()
	{
		chars = null;
		next_index = 0;
		opened_piece = null;
	}


	/* PUBLIC METHODS **********************************************************/


	public Composition read_file(String filename)
		throws IOException
	// Reads in the specified SpeciesChecker file and returns a Composition object
	// that contains all of the data on the file. Throws an IOException if problems
	// occured while reading the file.
	{
		opened_piece = null; // delete any previously opened piece
		access_file(filename); // read the file
		parse_composition(); // parse the file
		next_index = 0; // prepare to read another file
		chars = null; // prepare to read another file
		return opened_piece; // return the new Composition object
	}

	
	/* PRIVATE METHODS *********************************************************/


	private void access_file(String filename) 
		throws IOException
	// Instantiates chars and reads the content of the specified file into chars
	{
		File src = new File(filename);
		chars = new char[(int) src.length()];
		src = null;
		
		InputStream in;
		in = new FileInputStream(filename);
		int ch;
		for (int i = 0; (ch = in.read()) != -1; i++)
			chars[i] = (char)ch;
		in.close();
	}
	

	/***************************************************************************/


	private int get_next_number()
	// Reads in one line from chars and returns it as an integer. Also moves next_index to the next_location
	// in the file.
	{
		int i;
		for (i=next_index; chars[i] != '\n'; i++);
		String temp = new String(chars, next_index, i-next_index);
		next_index = i + 1;
		return Integer.parseInt(temp);
	}
	

	/***************************************************************************/


	private String get_next_string()
	// Reads in one line from chars and returns it as a string. Also moves next_index to
	// the next_location in the file.
	{
		int i;
		for (i=next_index; chars[i] != '\n'; i++);
		String temp = new String(chars, next_index, i-next_index);
		next_index = i + 1;
		return temp;
	}


	/***************************************************************************/


	private void parse_voice(int voice_index)
	// Inserts the specified voice into opened_piece. Also inserts the notes that are read
	// in into this voice.
	{
		// Read in the data for the voice
		int type_of_line = get_next_number();
		int clef = get_next_number();
		

		// Insert the voice into opened_piece and get a reference to it
		opened_piece.insert_voice(voice_index, type_of_line, clef);
		Voice current_voice = opened_piece.get_voice(voice_index);


		// Set up the variables to read in the notes for the voice
		int beat_counter = 0; // keeps track of what beat is being processed
		String current_beat = null; // the current line of information being read
		StringTokenizer tokens; // used to parse each line of information
		Note current_note = null; // the note to be inserted
		int duration = 0, tied = 0, pitch = 0; // information for each note
		int done = 0; // is 1 if all notes for the voice have been read. 0 otherwise.


		// Read and insert each note in the voice		
		while ( done != 1 )
		{
			current_beat = get_next_string();		
			if (current_beat.equals("end"))
				done = 1;
			else if (current_beat.equals("X"))
				beat_counter++;
			else if ( !(current_beat.equals("-")) )
			{
				tokens = new StringTokenizer(current_beat, " ");
				duration = Integer.parseInt(tokens.nextToken());
				tied = Integer.parseInt(tokens.nextToken());
				pitch = Integer.parseInt(tokens.nextToken());
				current_note = new Note(duration, tied, pitch);
				current_voice.insert_note(current_note, beat_counter);
				beat_counter++;
			}
		}
	}


	/***************************************************************************/


	private void parse_composition()
	// Creates opened_piece and inserts the configuration data into it. Also inserts
	// the voices and notes into opened_piece by calling parse_voice.
	{
		// Read in the data for the composition
		int type_of_composition = get_next_number();
		int quality = get_next_number();
		int key_signature = get_next_number();
		int time_signature = get_next_number();
		int number_of_bars = get_next_number();


		// Instantiate the new composition
		opened_piece = new Composition(type_of_composition, quality, key_signature, time_signature, number_of_bars);


		// Read in which voices are present
		int voice0 = get_next_number();
		int voice1 = get_next_number();
		int voice2 = get_next_number();
		int voice3 = get_next_number();

		
		// Process the voices which are present
		if (voice0 == 1)
		{
			while ( !(get_next_string().equals("VOICE 0")) );
			parse_voice(0);
		}
		if (voice1 == 1)
		{
			while ( !(get_next_string().equals("VOICE 1")) );
			parse_voice(1);
		}
		if (voice2 == 1)
		{
			while ( !(get_next_string().equals("VOICE 2")) );
			parse_voice(2);
		}
		if (voice3 == 1)
		{
			while ( !(get_next_string().equals("VOICE 3")) );
			parse_voice(3);
		}
	}
}