/*
SpeciesChecker
by Cory McKay

CLASS NAME:		FWriter
CLASS TYPE:		File I/O
DESCRIPTION:	Used to save SpeciesChecker compositions to disk.
				Appropriate for use in species counterpoint and simple harmony.
				See FormatSavedFile.txt guide for explanation of file format.
*/


import java.io.*;


class FWriter
{
	/* PUBLIC METHODS **********************************************************/


	public void write_file(String filename, Composition piece_to_save)
		throws IOException
	// Writes the date for the given Composition to a file with the given filename
	{
		DataOutputStream writer = open_file(filename); // prepare the file to be written to
		int[] voices_present = determine_voices(piece_to_save); // determine which voices need to be written

		write_header(piece_to_save, voices_present, writer); // write the header of the file
		for (int i = 0; i < voices_present.length; i++) // write the voice info and notes for each voice
		{
			if (voices_present[i] == 1)
				write_voice_info(piece_to_save, i, writer);
			else
				write_blank_voice(i, writer);
		}
		try {writer.close();} catch (IOException e) {;} // close the output stream
	}

	
	/* PRIVATE METHODS *********************************************************/


	private DataOutputStream open_file(String filename)
		throws IOException
	// Opens a DataOutputStream to the given file
	{
		File to_file = new File(filename);
		FileOutputStream to = new FileOutputStream(to_file);
		DataOutputStream writer = new DataOutputStream(to);
		return writer;
	}
	

	/***************************************************************************/


	private int[] determine_voices(Composition piece_to_save)
	// Determines which voices are present in the composition and need to be written to file
	{
		int[] voices_present = {0, 0, 0, 0}; // the voices for which data will be written
						     // 1 indicates that the voice is present, 0 means it isn't

		
		// Determine which voices are present
		int type_of_composition = piece_to_save.get_type_of_composition();
		if (type_of_composition == 0) // if the piece is a cantus
		{
			if (piece_to_save.get_voice(0) != null)
				voices_present[0] = 1;
		}
		else if (type_of_composition == 6) // if the piece is a four-part composition
		{
			if (piece_to_save.get_voice(0) != null)
				voices_present[0] = 1;
			if (piece_to_save.get_voice(1) != null)
				voices_present[1] = 1;
			if (piece_to_save.get_voice(2) != null)
				voices_present[2] = 1;
			if (piece_to_save.get_voice(3) != null)
				voices_present[3] = 1;
		}
		else // if the piece is a two-part composition
		{
			if (piece_to_save.get_voice(0) != null)
				voices_present[0] = 1;
			if (piece_to_save.get_voice(1) != null)
				voices_present[1] = 1;
		}

		
		// Return the voices present
		return voices_present;
	}
	

	/***************************************************************************/


	private void write_header(Composition piece_to_save, int[] voices_present, DataOutputStream writer)
		throws IOException
	// Writes the header of the file that is being saved to
	{
		writer.writeBytes(piece_to_save.get_type_of_composition() + "\n");
		writer.writeBytes(piece_to_save.get_quality() + "\n");
		writer.writeBytes(piece_to_save.get_key_signature() + "\n");
		writer.writeBytes(piece_to_save.get_time_signature() + "\n");
		writer.writeBytes(piece_to_save.get_number_of_bars() + "\n");
		for (int i = 0; i < voices_present.length; i++)
			writer.writeBytes(voices_present[i] + "\n");
		writer.writeBytes("\n"); // insert blank line to denote end of header
	}


	/***************************************************************************/


	private void write_voice_info(Composition piece_to_save, int voice_to_write, DataOutputStream writer)
		throws IOException
	// Writes the voice information and notes for voice_to_write
	{
		Voice voice_to_save = piece_to_save.get_voice(voice_to_write); // the voice that is being saved
	
		
		// Write the header for voice_to_save
		writer.writeBytes("VOICE " + voice_to_write + "\n");
		writer.writeBytes(voice_to_save.get_type_of_line() + "\n");
		writer.writeBytes(voice_to_save.get_clef() + "\n");
		

		// Write the note information for the voice
		Note current_note = null; // the note currently being written
		int time_signature = piece_to_save.get_time_signature(); // the time signature of the piece
		int note_index = 0; // the number of sixteenth note units that have been written
		int number_of_bars = piece_to_save.get_number_of_bars(); // the number of bars in the piece
		while (number_of_bars > 0) // write the information for each measure
		{
			writer.writeBytes("-\n"); // write the - to denote beginning of a measure
			for(int i = 0; i < time_signature; i++) // write the notes for one measure
			{
				current_note = voice_to_save.get_note(note_index);
				if (current_note == null)
					writer.writeBytes("X\n");
				else
					writer.writeBytes(current_note.get_duration() + " "
										+ current_note.get_tied() + " "
										+ current_note.get_pitch() + "\n");
				note_index++;
 			}
			number_of_bars--;
		}
		

		// Write the voice information ended indicator
		writer.writeBytes("end" + "\n");
	}


	/***************************************************************************/


	private void write_blank_voice(int voice_index, DataOutputStream writer)
		throws IOException
	// Writes the filler information for a blank voice
	{
		writer.writeBytes("VOICE " + voice_index + "\n");
		writer.writeBytes("end" + "\n");
	}
}