/*
SpeciesChecker
by Cory McKay

CLASS NAME:		GeneratorCantus
CLASS TYPE:		Music generator
DESCRIPTION:	Used to randomly generate cantus firmi. Does this by randomly generating
				candidate cantus firmi until one is generated which does not violate the
				rules contained in objects of the AnalyzerHorizontalCantus class.
*/


class GeneratorCantus
{
	/* PUBLIC METHODS **********************************************************/

	
	public Composition get_random_cantus(int[] rules_to_be_applied)
	// Returns a Composition object that contains a randomly generated cantus firmus.
	// The rules that are applied depend on the rules that are given in rules_to_be_applied
	// Currently disabled:Prints a status line to standard out while doing this.
	{
		int incorrect = 1; // a flag indicating how many trial generations are done
		Composition trial_cantus = null; // the candidate cantus
		ControllerCantus cantus_controller; // used to check the candidate cantus
		ErrorList errors_found; // will hold any errors found in the candidate cantus

		// System.out.print("Generating a cantus");

		while (incorrect != 0)
		{
			trial_cantus = generate_trial_line();
			incorrect++;
			// if (incorrect % 3000 == 1)
			//	System.out.print(".");
			cantus_controller = new ControllerCantus(trial_cantus);
			errors_found = cantus_controller.check_cantus(rules_to_be_applied);

			if (errors_found.is_there_an_error() == 0)
				incorrect = 0;
		}
		// System.out.println();
		return trial_cantus;
	}


	/* PRIVATE METHODS *********************************************************/

	
	private Composition generate_trial_line()
	// Returns a randomly generated trial cantus that may be incorrect
	{
		Composition random_composition = create_composition();
		create_voice(random_composition, 0);
		add_notes(random_composition, 0);
		return random_composition;
	}


	/***************************************************************************/


	private Composition create_composition()
	// Returns a randomly configured cantus Composition object
	// The following constraints are placed on this line:
	// - The Composition is a cantus only type Composition
	// - The time signature is 4/4
	// - The length is between 8 and 16 measures
	{
		int type_of_composition = 0; // is a cantus only Composition
		int quality = generate_random_number(2); // is either major or minor
		int key_signature = generate_random_number(12); // has any standard key signature
		int time_signature = 16; // is in 4/4
		int number_of_bars = 7 + generate_random_number(9); // is between 8 and 16 measures long
		return new Composition(type_of_composition, quality, key_signature, time_signature, number_of_bars);
	}
	
	
	/***************************************************************************/


	private void create_voice(Composition random_composition, int index)
	// Inserts a randomly generated cantus Voice object into location index random_composition
	// The following constraints are placed on this line:
	// - The line is a cantus
	// - The line has a C clef
	{
		int type_of_line = 0; // is a cantus
		int clef = 2; // cantus will have C clef
		random_composition.insert_voice(index, type_of_line, clef);
	}
	
	
	/***************************************************************************/


	private void add_notes(Composition random_composition, int index)
	// Adds randomly generated notes to random_voice
	// The following constraints are placed on this line:
	// - The first note will be between Gb small and F1 and will be the tonic of the given key
	// - All notes are whole notes and appear on the first beat of each bar
	// - All notes are not tied
	// - The interval between the lowest note and the highest note will be a tenth or less.
	// - All notes will belong to the key signature of the composition. In a minor line, if the second to last
	//		measure contains the seventh, it will be raised. Likewise for the 6th in the third to last measure
	// - Notes are not permitted to appear twice in a row
	{
		int beat_counter = 0; // the number of 16th note long rhythmic units that have gone by
		int pitch = 0; // the pitch of the next note to be inserted
		int tonic = 0; // the pitch of the first note


		// Insert the first note of the line
		if (random_composition.get_quality() == 1) // if the piece is major
		{
			if (random_composition.get_key_signature() <= 6)
				tonic = random_composition.get_key_signature() + 24;
			else
				tonic = random_composition.get_key_signature() + 12;
		}
		else // if the piece is minor
		{
			if (random_composition.get_key_signature() <= 9)
				tonic = random_composition.get_key_signature() + 21;
			else
				tonic = random_composition.get_key_signature() + 9;
		}
		random_composition.get_voice(index).insert_note(new Note(16, 1, tonic), beat_counter);
		beat_counter = beat_counter + 16;

	
		// Insert the remaining notes
		int valid_pitches[] = generate_valid_notes(tonic, random_composition.get_quality());
						// an array of pitches that are valid for the given key signature
		int pitch_index; // the index of the current note in valid_pitches
		int min_pitch_index = 0; // the minimum index of valid_pitches that fits within the range
		int max_pitch_index = 18; // the maximum index of valid_pitches that fits within the range
		int min_pitch = tonic; // the lowest pitch so far
		int max_pitch = tonic; // the highest pitch so far
		int pitch_in_range = 0; // is 1 when the chosen pitch does not cause the range of the line
		                        // to exceed a 10th. Is 0 otherwise.
		int last_pitch = 0; // the previous pitch that was succesfully selected
		while (beat_counter < random_composition.get_total_sixteenth_notes())
		{
			pitch_in_range = 0;
			while (pitch_in_range == 0)
			{
				pitch_index = generate_random_number(1 + max_pitch_index - min_pitch_index) + min_pitch_index - 1;
				pitch = valid_pitches[pitch_index];
				if (pitch < min_pitch && max_pitch - pitch > 16)
					min_pitch_index = pitch_index + 1;
				else if (pitch > max_pitch && pitch - min_pitch > 16)
					max_pitch_index = pitch_index - 1;
				else if (pitch != last_pitch)
					pitch_in_range = 1;
			}
			
			random_composition.get_voice(index).insert_note(new Note(16, 1, pitch), beat_counter);
			
			last_pitch = pitch;

			if (pitch > max_pitch)
				max_pitch = pitch;
			else if (pitch < min_pitch)
				min_pitch = pitch;
			
			beat_counter = beat_counter + 16;
		}

		
		// Make the last note the same as the first note
		random_composition.get_voice(index).insert_note(new Note(16, 1, tonic), beat_counter - 16);

		
		// Raise the 6th and 7th in the third and second to last measures if minor and necessary
		if (random_composition.get_quality() == 2) 
		{
			if (random_composition.get_voice(index).get_note(beat_counter - 32).get_pitch() == tonic - 2)
			{
				random_composition.get_voice(index).insert_note(new Note(16, 1, tonic - 1), beat_counter - 32);
				if (random_composition.get_voice(index).get_note(beat_counter - 48).get_pitch() == tonic - 4)
				{
					random_composition.get_voice(index).insert_note(new Note(16, 1, tonic - 3),
					                                                beat_counter - 48);
				}
			}
		}
	}

	
	/***************************************************************************/


	private int[] generate_valid_notes(int tonic, int quality)
	// Returns an array containing the pitches (in ascending order) of all notes that fall
	// within the range of a tenth below and above the tonic.
	{
		int valid_notes[] = new int[19];
		if (quality == 1)
		{
			valid_notes[0] = tonic - 15;
			valid_notes[1] = tonic - 13;
			valid_notes[2] = tonic - 12;
			valid_notes[3] = tonic - 10;
			valid_notes[4] = tonic - 8;
			valid_notes[5] = tonic - 7;
			valid_notes[6] = tonic - 5;
			valid_notes[7] = tonic - 3;
			valid_notes[8] = tonic - 1;
			valid_notes[9] = tonic;
			valid_notes[10] = tonic + 2;
			valid_notes[11] = tonic + 4;
			valid_notes[12] = tonic + 5;
			valid_notes[13] = tonic + 7;
			valid_notes[14] = tonic + 9;
			valid_notes[15] = tonic + 11;
			valid_notes[16] = tonic + 12;
			valid_notes[17] = tonic + 14;
			valid_notes[18] = tonic + 16;
		}
		else if (quality == 2)
		{
			valid_notes[0] = tonic - 16;
			valid_notes[1] = tonic - 14;
			valid_notes[2] = tonic - 12;
			valid_notes[3] = tonic - 10;
			valid_notes[4] = tonic - 9;
			valid_notes[5] = tonic - 7;
			valid_notes[6] = tonic - 5;
			valid_notes[7] = tonic - 4;
			valid_notes[8] = tonic - 2;
			valid_notes[9] = tonic;
			valid_notes[10] = tonic + 2;
			valid_notes[11] = tonic + 3;
			valid_notes[12] = tonic + 5;
			valid_notes[13] = tonic + 7;
			valid_notes[14] = tonic + 8;
			valid_notes[15] = tonic + 10;
			valid_notes[16] = tonic + 12;
			valid_notes[17] = tonic + 14;
			valid_notes[18] = tonic + 15;
		}
		return valid_notes;
	}

	
	/***************************************************************************/


	private int generate_random_number(int max) 
	// Returns a random integer from 1 to max, based on the uniform distribution.
	// WARNING: max must be lest than 10000
	{
		int random_number = (int) (10000.0 * Math.random());
		return (random_number % max) + 1;
	}
}