/*
SpeciesChecker
by Cory McKay

CLASS NAME:		InsertRowDialog
CLASS TYPE:		GUI - dialog box
DESCRIPTION:	This dialog box is used to choose the measure and beat where a new row is to be inserted on a table.
				New rows are only allowed to be inserted if a row does not already exist at that location.
				The was_updated method can be used to see if the OK button was pressed (as opposed to the
				Cancel button or the close box).
LIMITATIONS:	Only compatible with species counterpoint, not 4-part harmonies.
				Assumes that time signature is 4/4 and that is dealing with a fifth species composition.
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


class InsertRowDialog extends JDialog
{
	/* FIELDS ******************************************************************/


	private JLabel measure_label;		// a label for the measure_combo combo box
	private JComboBox measure_combo;	// a combo box to store the measure to insert the row in
	private JLabel beat_label;			// a label for the beat_combo combo box
	private JComboBox beat_combo;		// a combo box to store the beat to insert the row in
	private boolean update;				// is set to true after the OK button is pressed


	/* CONSTRUCTOR *************************************************************/


	public InsertRowDialog(JFrame owner, int measures)
	// measures is the number of measures in the piece that the row is being added to
	{
		// Give the dialog box its owner, its title and make it modal
		super(owner, "Add Row", true);


		// Initialize the dialog box as not having been updated (to distinguish between ok and cancel)
		update = false;


		// Configure a non-editable combo box to allow the user to choose the measure where the row is
		// to be inserted
		measure_label = new JLabel("Measure");
		measure_combo = new JComboBox();
		measure_combo.setEditable(false);
		for (int i = 1; i <= measures; i++)
		{
			measure_combo.addItem(new Integer(i));
		}


		// Configure a non-editable combo box to allow the user to choose the beat where the row is
		// to be inserted
		beat_label = new JLabel("Beat");
		beat_combo = new JComboBox();
		beat_combo.setEditable(false);
		beat_combo.addItem("1.5");
		beat_combo.addItem("2");
		beat_combo.addItem("2.5");
		beat_combo.addItem("3");
		beat_combo.addItem("3.5");
		beat_combo.addItem("4");
		beat_combo.addItem("4.5");


		// Configure an OK button to close the dialog box
		// Sets update to true so that the calling class can check to see if OK was pressed
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				update = true;
				setVisible(false);
			}
		});


		// Configure a cancel dialog box to close the dialog
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				setVisible(false);
			}
		});


		// Display the dialog box elements and set the size of the dialog box
		Container contentPane = getContentPane();
		JPanel outer_panel = new JPanel();
		JPanel combo_panel = new JPanel();
		JPanel button_panel = new JPanel();
		combo_panel.setLayout(new GridLayout(2,2));
		combo_panel.add(measure_label);
		combo_panel.add(measure_combo);
		combo_panel.add(beat_label);
		combo_panel.add(beat_combo);
		button_panel.add(ok);
		button_panel.add(cancel);
		outer_panel.add(combo_panel, BorderLayout.CENTER);
		outer_panel.add(button_panel, BorderLayout.SOUTH);
		contentPane.add(outer_panel);
		setSize(160, 130);
	}


	/* PUBLIC METHODS **********************************************************/


	public boolean was_updated()
	// Returns true if the OK button was pressed the last time the dialog box was visible
	// Returns false otherwise. Resets the update field to false.
	{
		boolean temp = update;
		update = false;
		return temp;
	}


	/***************************************************************************/


	public int insert_row(NotesTableModel table_model)
	// Insert a row into the table_model table at the location specified on this dialog box. Do not do anything
	// if this row already exists.
	{
		int number_of_rows = table_model.getRowCount(); // the total number of rows in the table


		// Do not do anything if the selected row already exists
		for (int i = 0; i < number_of_rows; i++)
		{
			if (String.valueOf(table_model.getValueAt(i, 0)).equals(String.valueOf(measure_combo.getSelectedItem())))
			{
				if (table_model.getValueAt(i, 1).equals(beat_combo.getSelectedItem()))
					return -1;
			}
		}


		// Configure the default row to insert into the table
		Object[] row = new Object[8]; // the default note data for the row
		row[0] = new Integer(Integer.parseInt(String.valueOf(measure_combo.getSelectedItem())));
		row[1] = new String((String) beat_combo.getSelectedItem());
		row[2] = null;
		row[3] = null;
		row[4] = new String("C");
		row[5] = new Integer(1);
		row[6] = new String("quarter");
		row[7] = new String("not tied ahead");


		// Insert the row in the proper place
		boolean found_measure = false;
		for (int i = 0; i < number_of_rows; i++)
		{
			if (String.valueOf(table_model.getValueAt(i, 0)).equals(String.valueOf(measure_combo.getSelectedItem())))
			{
				found_measure = true;
				if (new Float((String) table_model.getValueAt(i, 1)).floatValue()
				    > new Float((String) beat_combo.getSelectedItem()).floatValue())
				{
					table_model.insertRow(i, row);
					i = number_of_rows;
				}
			}
			else if ( (found_measure == true)
				&& (Integer.parseInt(String.valueOf(table_model.getValueAt(i, 0))) > Integer.parseInt(String.valueOf(measure_combo.getSelectedItem()))))
			{
				table_model.insertRow(i, row);
				i = number_of_rows;
			}
			if (i == number_of_rows - 1)
				table_model.insertRow(i + 1, row);
		}


		// Let the calling method know that there were no problems
		return 1;
	}
}