/*
SpeciesChecker
by Cory McKay

CLASS NAME:		NewDialog
CLASS TYPE:		GUI - dialog box
DESCRIPTION:	This dialog box is used to get the basic settings for a new composition from the user.
				The user is able to set the piece's type, key signature, time signature (currently disabled)
				and length. The was_updated method can be used to see if the OK button was pressed (as
				opposed to the Cancel button or the close box). The get_new_composition method can be called
				to get a NewComposition object that contains a new Composition object, its blank new Voice objects
				and formatted human readable display information specifying the choices selected in this dialog box.
LIMITATIONS:	Only compatible with species counterpoint, not 4-part harmonies.
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


class NewDialog extends JDialog
{
	/* FIELDS ******************************************************************/


	private JLabel type_label;		// a label for the type_combo combo box
	private JComboBox type_combo;	// a combo box to store the type of composition
	private JLabel key_label;		// a label for the key_combo combo box
	private JComboBox key_combo;	// a combo box to store the key signature of the composition
	private JLabel time_label;		// a label for the time_combo combo box
	private JComboBox time_combo;	// a combo box to store the meter of the composition
	private JLabel length_label;	// a label for the length_combo combo box
	private JComboBox length_combo;	// a combo box to store the lenght of the composition
	private boolean update;			// is set to true after the OK button is pressed, is false otherwise


	/* CONSTRUCTOR *************************************************************/


	public NewDialog(JFrame owner)
	// Constructor
	{
		// Give the dialog box its owner, its title and make it modal
		super(owner, "Create New Composition", true);


		// Initialize the dialog box as not having been updated (to distinguish between ok and cancel)
		update = false;


		// Configure a non-editable combo box to allow the user to choose the type of composition
		type_label = new JLabel("Composition type:");
		type_combo = new JComboBox();
		type_combo.setEditable(false);
		type_combo.addItem("Cantus firmus only");
		type_combo.addItem("First species, cantus lower voice");
		type_combo.addItem("First species, cantus upper voice");
		type_combo.addItem("Second species, cantus lower voice");
		type_combo.addItem("Second species, cantus upper voice");
		type_combo.addItem("Third species, cantus lower voice");
		type_combo.addItem("Third species, cantus upper voice");
		type_combo.addItem("Fourth species, cantus lower voice");
		type_combo.addItem("Fourth species, cantus upper voice");
		type_combo.addItem("Fifth species, cantus lower voice");
		type_combo.addItem("Fifth species, cantus upper voice");


		// Configure a non-editable combo box to allow the user to choose the key of the composition
		key_label = new JLabel("Key of composition:");
		key_combo = new JComboBox();
		key_combo.setEditable(false);
		key_combo.addItem("C major");
		key_combo.addItem("Db major");
		key_combo.addItem("D major");
		key_combo.addItem("Eb major");
		key_combo.addItem("E major");
		key_combo.addItem("F major");
		key_combo.addItem("Gb major");
		key_combo.addItem("G major");
		key_combo.addItem("Ab major");
		key_combo.addItem("A major");
		key_combo.addItem("Bb major");
		key_combo.addItem("B major");
		key_combo.addItem("A minor");
		key_combo.addItem("Bb minor");
		key_combo.addItem("B minor");
		key_combo.addItem("C minor");
		key_combo.addItem("C# minor");
		key_combo.addItem("D minor");
		key_combo.addItem("Eb minor");
		key_combo.addItem("E minor");
		key_combo.addItem("F minor");
		key_combo.addItem("F# minor");
		key_combo.addItem("G minor");
		key_combo.addItem("G# minor");


		// Configure a non-editable combo box to allow the user to choose the time signature of the composition
		time_label = new JLabel("Meter of composition:");
		time_combo = new JComboBox();
		time_combo.setEditable(false);
		time_combo.addItem("4 / 4");
		time_combo.addItem("3 / 4");
		time_combo.addItem("2 / 4");
		time_combo.addItem("12 / 8");


		// Configure a non-editable combo box to allow the user to choose the number of measures in the composition
		// Also set the default selection to 10
		length_label = new JLabel("Length in measures:");
		length_combo = new JComboBox();
		length_combo.setEditable(false);
		length_combo.addItem("4");
		length_combo.addItem("5");
		length_combo.addItem("6");
		length_combo.addItem("7");
		length_combo.addItem("8");
		length_combo.addItem("9");
		length_combo.addItem("10");
		length_combo.addItem("11");
		length_combo.addItem("12");
		length_combo.addItem("13");
		length_combo.addItem("14");
		length_combo.addItem("15");
		length_combo.addItem("16");
		length_combo.addItem("17");
		length_combo.addItem("18");
		length_combo.addItem("19");
		length_combo.addItem("20");
		length_combo.setSelectedIndex(6);


		// Configure an OK button to close the dialog box
		// Sets update to true so that the calling class can check to see if OK was pressed
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				update = true;
				setVisible(false);
			}
		});


		// Configure a cancel dialog box to close the dialog
		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				setVisible(false);
			}
		});


		// Display the dialog box elements and set the size of the dialog box
		Container contentPane = getContentPane();
		JPanel outer_panel = new JPanel();
		outer_panel.setLayout(new BorderLayout());
		JPanel combo_panel = new JPanel();
		combo_panel.setLayout(new GridLayout(3,2));
		combo_panel.add(type_label);
		combo_panel.add(type_combo);
		combo_panel.add(key_label);
		combo_panel.add(key_combo);
		/* DO NOT ALLOW TIME SIGNATURE SELECTION SINCE IT IS ALWAYS 4/4 FOR SPECIES COUNTERPOINT
		combo_panel.add(time_label);
		combo_panel.add(time_combo);
		*/
		combo_panel.add(length_label);
		combo_panel.add(length_combo);
		JPanel button_panel = new JPanel();
		button_panel.add(ok);
		button_panel.add(cancel);
		outer_panel.add(combo_panel, BorderLayout.CENTER);
		outer_panel.add(button_panel, BorderLayout.SOUTH);
		contentPane.add(outer_panel);
		setSize(480, 150);
	}


	/* PUBLIC METHODS **********************************************************/


	public boolean was_updated()
	// Returns true if the OK button was pressed the last time the dialog box was visible
	// Returns false otherwise. Resets the update field to false.
	{
		boolean temp = update;
		update = false;
		return temp;
	}


	/***************************************************************************/


	public NewComposition get_new_composition()
	// Returns the information selected in this dialog box. Return information includes a generated
	// Composition and its blank Voice objects along with a human readable string containing the items selected
	// in the dialog box.
	{
		// Determine what type of line is desired and if the cantus is the upper or lower voice
		// NOTE: this is determined by looking at the type of line that was chosen by the user
		int composition_type_of_line; // the type of line, using the code of Composition objects
		int voice_type_of_line; // the type of line, using the code of Voice objects
		if (get_type().identifier > 5)
			composition_type_of_line = get_type().identifier - 5;
		else
			composition_type_of_line = get_type().identifier;
		if (get_type().identifier == 1) voice_type_of_line = 1;
		else if (get_type().identifier == 2) voice_type_of_line = 3;
		else if (get_type().identifier == 3) voice_type_of_line = 5;
		else if (get_type().identifier == 4) voice_type_of_line = 7;
		else if (get_type().identifier == 5) voice_type_of_line = 9;
		else if (get_type().identifier == 6) voice_type_of_line = 2;
		else if (get_type().identifier == 7) voice_type_of_line = 4;
		else if (get_type().identifier == 8) voice_type_of_line = 6;
		else if (get_type().identifier == 9) voice_type_of_line = 8;
		else if (get_type().identifier == 10) voice_type_of_line = 10;
		else voice_type_of_line = 0;


		// Determine if the composition is major or minor and what key it's in
		// NOTE: this is determined by looking at the key that was chosen by the user
		int quality; // whether the new composition in major or minor, in Composition object encoding
		int key_sig; // the key of the new composition, in Composition object encoding
		String quality_name; // whether the new composition is major or minor, in English
		if (get_key().identifier < 13)
		{
			quality = 1;
			quality_name = new String("Major");
			key_sig = get_key().identifier;
		}
		else
		{
			quality = 2;
			quality_name = new String("Minor");
			key_sig = get_key().identifier - 12;
		}


		// Create the new composition. Configure the appropriate number of voices for it.
		Composition piece; // the new Composition object
		Voice cantus; // the new cantus Voice object
		Voice counterpoint; // the new counterpoint Voice object
		piece = new Composition(composition_type_of_line, quality, key_sig,
			get_time_sig().identifier, get_length().identifier);
		piece.insert_voice(0, 0, 2); // set up the cantus voice with a C clef
		cantus = piece.get_voice(0);
		if (composition_type_of_line != 0)
		{
			piece.insert_voice(1, voice_type_of_line, 2); // set up the counterpoint with a C clef
			counterpoint = piece.get_voice(1);
		}
		else
			counterpoint = null;


		// Return the NewComposition object that contains all of the appropriate information
		return new NewComposition(get_type().text, get_key().text, get_time_sig().text,
			get_length().text, piece, cantus, counterpoint);
	}


	/* PRIVATE METHODS *********************************************************/


	private NewReturn get_type()
	// Returns a NewReturn object that contains the text for the type of composition that is selected
	// in the type_combo combo box and the code for this selection that will be used by a Composition object.
	// NOTE: the code that is returned is used by the calling class to determine if the counterpoint is
	//       above or below the cantus. 5 must be subtracted from the codes of cantus upper voice types
	//       in order to get the true code used by composition objects.
	{
		String choice = (String)type_combo.getSelectedItem();
		if (choice.equals("First species, cantus lower voice")) return new NewReturn(choice, 1);
		else if (choice.equals("Second species, cantus lower voice")) return new NewReturn(choice, 2);
		else if (choice.equals("Third species, cantus lower voice")) return new NewReturn(choice, 3);
		else if (choice.equals("Fourth species, cantus lower voice")) return new NewReturn(choice, 4);
		else if (choice.equals("Fifth species, cantus lower voice")) return new NewReturn(choice, 5);
		else if (choice.equals("First species, cantus upper voice")) return new NewReturn(choice, 6);
		else if (choice.equals("Second species, cantus upper voice")) return new NewReturn(choice, 7);
		else if (choice.equals("Third species, cantus upper voice")) return new NewReturn(choice, 8);
		else if (choice.equals("Fourth species, cantus upper voice")) return new NewReturn(choice, 9);
		else if (choice.equals("Fifth species, cantus upper voice")) return new NewReturn(choice, 10);
		else return new NewReturn("Cantus firmus only", 0);
	}


	/***************************************************************************/


	private NewReturn get_key()
	// Returns a NewReturn object that contains the text for the key signature that is selected
	// in the key_combo combo box and the code for this selection that will be used by a Composition object.
	// NOTE: the code that is returned is used by the calling class to determine if the composition is major
	//       or minor. 12 must be subtracted from the codes of minor keys in order to get the true
	//       code used by composition objects.
	{
		String choice = (String)key_combo.getSelectedItem();
		if (choice.equals("C major")) return new NewReturn(choice, 1);
		else if (choice.equals("Db major")) return new NewReturn(choice, 2);
		else if (choice.equals("D major")) return new NewReturn(choice, 3);
		else if (choice.equals("Eb major")) return new NewReturn(choice, 4);
		else if (choice.equals("E major")) return new NewReturn(choice, 5);
		else if (choice.equals("F major")) return new NewReturn(choice, 6);
		else if (choice.equals("Gb major")) return new NewReturn(choice, 7);
		else if (choice.equals("G major")) return new NewReturn(choice, 8);
		else if (choice.equals("Ab major")) return new NewReturn(choice, 9);
		else if (choice.equals("A major")) return new NewReturn(choice, 10);
		else if (choice.equals("Bb major")) return new NewReturn(choice, 11);
		else if (choice.equals("B major")) return new NewReturn(choice, 12);
		else if (choice.equals("A minor")) return new NewReturn(choice, 13);
		else if (choice.equals("Bb minor")) return new NewReturn(choice, 14);
		else if (choice.equals("B minor")) return new NewReturn(choice, 15);
		else if (choice.equals("C minor")) return new NewReturn(choice, 16);
		else if (choice.equals("C# minor")) return new NewReturn(choice, 17);
		else if (choice.equals("D minor")) return new NewReturn(choice, 18);
		else if (choice.equals("Eb minor")) return new NewReturn(choice, 19);
		else if (choice.equals("E minor")) return new NewReturn(choice, 20);
		else if (choice.equals("F minor")) return new NewReturn(choice, 21);
		else if (choice.equals("F# minor")) return new NewReturn(choice, 22);
		else if (choice.equals("G minor")) return new NewReturn(choice, 23);
		else if (choice.equals("G# minor")) return new NewReturn(choice, 24);
		else return new NewReturn("C major", 1);
	}


	/***************************************************************************/


	private NewReturn get_time_sig()
	// Returns a NewReturn object that contains the text for the time signature that is selected
	// in the time_combo combo box and the code for this selection that will be used by a Composition object.
	{
		String choice = (String)time_combo.getSelectedItem();
		if (choice.equals("3 / 4")) return new NewReturn(choice, 12);
		if (choice.equals("2 / 4")) return new NewReturn(choice, 8);
		if (choice.equals("12 / 8")) return new NewReturn(choice, 24);
		else return new NewReturn("4 / 4", 16);
	}


	/***************************************************************************/


	private NewReturn get_length()
	// Returns a NewReturn object that contains the text for the composition length that is selected
	// in the length_combo combo box and the code for this selection that will be used by a Composition object.
	{
		String choice = (String)length_combo.getSelectedItem();
		if (choice.equals("4")) return new NewReturn(choice, 4);
		if (choice.equals("5")) return new NewReturn(choice, 5);
		if (choice.equals("6")) return new NewReturn(choice, 6);
		if (choice.equals("7")) return new NewReturn(choice, 7);
		if (choice.equals("8")) return new NewReturn(choice, 8);
		if (choice.equals("9")) return new NewReturn(choice, 9);
		if (choice.equals("10")) return new NewReturn(choice, 10);
		if (choice.equals("11")) return new NewReturn(choice, 11);
		if (choice.equals("12")) return new NewReturn(choice, 12);
		if (choice.equals("13")) return new NewReturn(choice, 13);
		if (choice.equals("14")) return new NewReturn(choice, 14);
		if (choice.equals("15")) return new NewReturn(choice, 15);
		if (choice.equals("16")) return new NewReturn(choice, 16);
		if (choice.equals("17")) return new NewReturn(choice, 17);
		if (choice.equals("18")) return new NewReturn(choice, 18);
		if (choice.equals("19")) return new NewReturn(choice, 19);
		if (choice.equals("20")) return new NewReturn(choice, 20);
		else return new NewReturn("11", 11);
	}


	/* INTERNAL CLASSES ********************************************************/


	class NewReturn
	// Objects of this class are used by NewDialog mehtods to communicate internally with each other.
	// The text field holds the text of a selection in a combo box and the identifier holds the
	// code that is used by Composition objects to identify the selection.
	{
		public NewReturn(String a, int b)
		{
			text = a;
			identifier = b;
		}

		public String text;
		public int identifier;
	}
}