/*
SpeciesChecker
by Cory McKay

CLASS NAME:		NotesTableModel
CLASS TYPE:		GUI - table
DESCRIPTION:	The part of the GUI dealing with the displaying and entering of information about
				individual notes in compositions. Each row represents a beat unit.
				The getValuesFromTable should always be called before analyzing Composition
				information in orger to update the stored Composition information with the
				data on the table. The setupTable method should be called to populate and display
				new compositions and the importIntoTable method should be called to display already
				popluated compositions.
LIMITATIONS:	Only compatible with species counterpoint, not 4-part harmonies.
				Assumes that time signature is 4/4.
				Assumes that cantus notes are untied whole notes.
*/


import javax.swing.*;
import javax.swing.table.*;
import java.util.*;


class NotesTableModel extends DefaultTableModel
{
	/* FIELDS ******************************************************************/


	private boolean cantus_present;			// indicates whether or not a cantus is present
	private boolean counterpoint_present;	// indicates whether or not a cournterpoint is present


	/* CONSTRUCTOR *************************************************************/


	NotesTableModel(Object[] columnNames, int rows)
	// Identical constructor as DefaultTableModel
	{
		super(columnNames, rows);
	}


	/* PUBLIC METHODS **********************************************************/


	public boolean isCellEditable(int row, int column)
	// Returns whether or not the given cell is editable
	// Sets the cantus pitch and octave columns to editable if a cantus is present and sets
	// the counterpoint columns to editable if a counterpoint is present. All other
	// columns are non-editable.
	// Will only allow cantus cells to be edited if they correspond to the first beat of
	// a measure.
	{
		boolean answer = false;
		if (cantus_present == true)
		{
			if (column == 2 || column == 3)
			{
				if (Integer.parseInt(String.valueOf(getValueAt(row, 1))) == 1)
						answer = true;
			}
		}
		if (counterpoint_present == true)
		{
			if (column > 3)
				answer = true;
		}

		return answer;
	}


	/***************************************************************************/


	public void clearTable()
	// Remove all data on the table
	{
		while (getRowCount() != 0)
		{
			removeRow(0);
		}
		cantus_present = false;
		counterpoint_present = false;
	}


	/***************************************************************************/


	public void setupTable(Composition piece, Voice cantus, Voice counterpoint)
	// Set up a default table for new unpopulated compositions, based on the type of composition.
	// NOTE: Assumes that time signature is 4/4
	{
		int number_measures = piece.get_number_of_bars(); // the number of measures in the composition
		Object[] row = new Object[8]; // the default note data for each row
		for (int i =0; i < 8; i++)
			row[i] = null;


		// Deal with a cantus only
		if (piece.get_type_of_composition() == 0)
		{
			cantus_present = true;
			counterpoint_present = false;

			for (int i = 0; i < number_measures; i++)
			{
				row[0] = new Integer( i + 1 );
				row[1] = new String("1");
				row[2] = new String("C");
				row[3] = new Integer(1);
				insertRow(i, row);
			}
		}


		// Deal with a first species composition
		if (piece.get_type_of_composition() == 1)
		{
			cantus_present = true;
			counterpoint_present = true;

			for (int i = 0; i < number_measures; i++)
			{
				row[0] = new Integer( i + 1 );
				row[1] = new String("1");
				row[2] = new String("C");
				row[3] = new Integer(1);
				row[4] = new String("C");
				row[5] = new Integer(1);
				row[6] = new String("whole");
				row[7] = new String("not tied ahead");
				insertRow(i, row);
			}
		}


		// Deal with a second or forth species composition
		if (piece.get_type_of_composition() == 2 || piece.get_type_of_composition() == 4)
		{
			cantus_present = true;
			counterpoint_present = true;

			for (int i = 0; i < 2 * (number_measures - 1); i++)
			{
				if (i%2 == 0)
				{
					row[0] = new Integer( (i / 2) + 1 );
					row[1] = new String("1");
					row[2] = new String("C");
					row[3] = new Integer(1);
				}
				else
				{
					row[0] = new Integer( (i / 2) + 1 );
					row[1] = new String("3");
					row[2] = null;
					row[3] = null;
				}
				row[4] = new String("C");
				row[5] = new Integer(1);
				row[6] = new String("half");
				row[7] = new String("not tied ahead");
				insertRow(i, row);
			}


			// Set up last measure
			row[0] = new Integer(number_measures);
			row[1] = new String("1");
			row[2] = new String("C");
			row[3] = new Integer(1);
			row[4] = new String("C");
			row[5] = new Integer(1);
			row[6] = new String("whole");
			row[7] = new String("not tied ahead");
			insertRow(2 * number_measures - 2, row);
		}


		// Deal with a third or fifth species composition
		if (piece.get_type_of_composition() == 3 || piece.get_type_of_composition() == 5)
		{
			cantus_present = true;
			counterpoint_present = true;

			for (int i = 0; i < 4 * (number_measures - 1); i++)
			{
				if (i%4 == 0)
				{
					row[0] = new Integer( (i / 4) + 1 );
					row[1] = new String("1");
					row[2] = new String("C");
					row[3] = new Integer(1);
				}
				else
				{
					row[0] = new Integer( (i / 4) + 1 );
					row[1] = new String(String.valueOf(1 + i%4));
					row[2] = null;
					row[3] = null;
				}
				row[4] = new String("C");
				row[5] = new Integer(1);
				row[6] = new String("quarter");
				row[7] = new String("not tied ahead");
				insertRow(i, row);
			}


			// Set up last measure
			row[0] = new Integer(number_measures);
			row[1] = new String("1");
			row[2] = new String("C");
			row[3] = new Integer(1);
			row[4] = new String("C");
			row[5] = new Integer(1);
			row[6] = new String("whole");
			row[7] = new String("not tied ahead");
			insertRow(4 * number_measures - 4, row);
		}
	}


	/***************************************************************************/


	public void importIntoTable(Composition piece, Voice cantus, Voice counterpoint)
	// Sets up the table for populated compositions.
	// NOTE: works for any time signature except 12/8
	// NOTE: assumes that cantus notes are untied whole notes
	{
		int total_sixteenth_notes = piece.get_total_sixteenth_notes();
		int time_signature = piece.get_time_signature();
		int key_signature = piece.get_key_signature();
		int quality = piece.get_quality();
		int type_of_composition = piece.get_type_of_composition();
		boolean found_note;
		int rows_so_far = 0;
		Object[] row = new Object[8]; // the default note data for each row

		for (int i = 0; i < total_sixteenth_notes; i++)
		{
			for (int j = 0; j < 8; j++)
				row[j] = null;
			found_note = false;
			if (cantus.get_note(i) != null)
			{
				found_note = true;
				row[2] = new String(decodePitch(cantus.get_note(i).get_pitch(), key_signature, quality, false));
				row[3] = new Integer(Integer.parseInt(decodePitch(cantus.get_note(i).get_pitch(),
				                     key_signature, quality, true)));
			}
			if (type_of_composition != 0)
			{
				if (counterpoint.get_note(i) != null)
				{
					found_note = true;
					row[4] = new String(decodePitch(counterpoint.get_note(i).get_pitch(), key_signature, quality, false));
					row[5] = new Integer(Integer.parseInt(decodePitch(counterpoint.get_note(i).get_pitch(),
					                    key_signature, quality, true)));
					row[6] = new String(decodeDuration(counterpoint.get_note(i).get_duration()));
					row[7] = new String(decodeTied(counterpoint.get_note(i).get_tied()));
				}
			}

			if (found_note == true)
			{
				row[0] = new Integer( (i / time_signature) + 1);
				String beat_symbol = String.valueOf( ( ( (double) i % (double) time_signature) / 4.0) + 1.0 );
				if (beat_symbol.charAt(2) == '0') // deal with notes that fall between beats
				{
					char[] beat_char = new char[1];
					beat_char[0] = beat_symbol.charAt(0);
					beat_symbol = new String(beat_char);
				}
				row[1] = new String(beat_symbol);
				insertRow(rows_so_far, row);
				rows_so_far++;
			}
		}

		cantus_present = true;
		if (type_of_composition != 0) counterpoint_present = true;
		else counterpoint_present = false;
	}


	/***************************************************************************/


	public void getValuesFromTable(Composition piece, Voice cantus, Voice counterpoint)
	// Delete any notes in cantus or counterpoint and fill them in with the notes displayed on the table
	// NOTE: works for any time signature
	{
		int counterpoint_type_of_line; // the type of line of the counterpoint
		int measure; // the measure as indicated on the table
		int beat; // the beat of the measure as indicated on the table
		int rhythm_index; // the proper index to insert a note in a Voice object
		char[] beat_chars; // used to determine the beat value
		String beat_string; // used to determine the beat value
		String pitch; // the of a note as indicated on the table
		int octave; // the register of a note as indicated on the table
		String duration; // the duration of a counterpoint note as indicated on the table
		String tied; // whether or not a counterpoint note is tied, as indicated on the table
		int previous_tied_status = 1; // how the preceeding note was tied
		Note note_to_insert; // a note to insert into a voice


		// Determine what type of line the counterpoint is
		counterpoint_type_of_line = 15; // dummy value
		if (counterpoint != null)
			counterpoint_type_of_line = counterpoint.get_type_of_line();


		// Clear any pre-existing information in the cantus and counterpoint voices
		piece.insert_voice(0, 0, 2); // set up the cantus voice with a C clef
		cantus = piece.get_voice(0);
		counterpoint = null;
		if (piece.get_type_of_composition() != 0)
		{
			piece.insert_voice(1, counterpoint_type_of_line, 2); // set up the counterpoint with a C clef
			counterpoint = piece.get_voice(1);
		}


		// Insert notes into the voices
		for (int i = 0; i < getRowCount(); i++)
		{
			// Calculate where to insert the notes rhytmically
			measure = Integer.parseInt(String.valueOf(getValueAt(i, 0)));
			beat_string = String.valueOf(getValueAt(i, 1));
			if (beat_string.length() == 1) // case where note doesn't fall in between beats
			{
				beat = Integer.parseInt(beat_string);
				rhythm_index = ((measure - 1) * piece.get_time_signature()) + ((beat - 1) * 4);
			}
			else // case where note falls in between beats
			{
				beat_chars = new char[1];
				beat_chars[0] = beat_string.charAt(0);
				beat = Integer.parseInt(new String(beat_chars));
				rhythm_index = ((measure - 1) * piece.get_time_signature()) + ((beat - 1) * 4) + 2;
			}

			// Insert notes belonging to the cantus
			// These notes are set automatically to untied whole notes
			if (getValueAt(i, 2) != null )
			{
				pitch = String.valueOf(getValueAt(i, 2));
				if ( !(pitch.equals(" ")) )
				{
					octave = Integer.parseInt(String.valueOf(getValueAt(i, 3)));
					note_to_insert = new Note(16, 1, encodePitch(pitch, octave));
					cantus.insert_note(note_to_insert, rhythm_index);
				}
			}


			// Insert notes belonging to the the counterpoint
			if (getValueAt(i, 4) != null )
			{
				pitch = String.valueOf(getValueAt(i, 4));
				if ( !(pitch.equals(" ")) )
				{
					octave = Integer.parseInt(String.valueOf(getValueAt(i, 5)));
					duration = String.valueOf(getValueAt(i, 6));
					tied = String.valueOf(getValueAt(i, 7));
					note_to_insert = new Note(encodeDuration(duration), encodeTied(tied, previous_tied_status),
						encodePitch(pitch, octave));
					counterpoint.insert_note(note_to_insert, rhythm_index);
					previous_tied_status = encodeTied(tied, previous_tied_status);
				}
			}
		}
	}


	/***************************************************************************/


	public void deleteRow(int row_to_delete)
	// Delete the given row
	{
		removeRow(row_to_delete);
	}


	/***************************************************************************/


	public void labelPitchCombo(JComboBox pitch_combo)
	// Add selection items to combo boxes dealing with pitch of notes
	{
		pitch_combo.addItem("C");
		pitch_combo.addItem("C#");
		pitch_combo.addItem("Db");
		pitch_combo.addItem("D");
		pitch_combo.addItem("D#");
		pitch_combo.addItem("Eb");
		pitch_combo.addItem("E");
		pitch_combo.addItem("E#");
		pitch_combo.addItem("Fb");
		pitch_combo.addItem("F");
		pitch_combo.addItem("F#");
		pitch_combo.addItem("Gb");
		pitch_combo.addItem("G");
		pitch_combo.addItem("G#");
		pitch_combo.addItem("Ab");
		pitch_combo.addItem("A");
		pitch_combo.addItem("A#");
		pitch_combo.addItem("Bb");
		pitch_combo.addItem("B");
		pitch_combo.addItem("Rest");
		/* DISABLE B# AND Bb BECAUSE B AND C ARE ENHARMONIC EQUIVALENTS AND THEY ARE DIFFICULT TO PROCESS
		   DISABLE BLANK SPACE BECAUSE IT IS TAKEN CARE OF AUTOMATICALLY BY GUI. CAN ERASE A ROW IF WANT.
		pitch_combo.addItem("B#");
		pitch_combo.addItem("Cb");
		pitch_combo.addItem(" ");
		*/
	}


	/***************************************************************************/


	public void labelOctaveCombo(JComboBox octave_combo)
	// Add selection items to combo boxes dealing with register of notes
	{
		octave_combo.addItem(new Integer(-2));
		octave_combo.addItem(new Integer(-1));
		octave_combo.addItem(new Integer(1));
		octave_combo.addItem(new Integer(2));
		octave_combo.addItem(new Integer(3));
	}


	/***************************************************************************/


	public void labelDurationCombo(JComboBox duration_combo)
	// Add selection items to combo boxes dealing with duration of notes or rests
	{
		duration_combo.addItem("eighth");
		duration_combo.addItem("quarter");
		duration_combo.addItem("half");
		duration_combo.addItem("whole");
	}


	/***************************************************************************/


	public void labelTiedCombo(JComboBox tied_combo)
	// Add selection items to combo boxes dealing with notes being tied ahead or not
	{
		tied_combo.addItem("tied ahead");
		tied_combo.addItem("not tied ahead");
	}


	/* PRIVATE METHODS *********************************************************/


	private int encodePitch(String pitch, int octave)
	// Returns the code used by Note objects that corresponds to the given
	// table codes for pitch and octave
	{
		int coded_pitch = 0; // the pitch to be retruned

		if (pitch.equals("C") || pitch.equals("B#")) coded_pitch = 1;
		else if (pitch.equals("C#") || pitch.equals("Db")) coded_pitch = 2;
		else if (pitch.equals("D")) coded_pitch = 3;
		else if (pitch.equals("D#") || pitch.equals("Eb")) coded_pitch = 4;
		else if (pitch.equals("E") || pitch.equals("Fb")) coded_pitch = 5;
		else if (pitch.equals("E#") || pitch.equals("F")) coded_pitch = 6;
		else if (pitch.equals("F#") || pitch.equals("Gb")) coded_pitch = 7;
		else if (pitch.equals("G")) coded_pitch = 8;
		else if (pitch.equals("G#") || pitch.equals("Ab")) coded_pitch = 9;
		else if (pitch.equals("A")) coded_pitch = 10;
		else if (pitch.equals("A#") || pitch.equals("Bb")) coded_pitch = 11;
		else if (pitch.equals("B")  || pitch.equals("Cb")) coded_pitch = 12;
		// else if (pitch.equals("B#") || pitch.equals("Cb")) coded_pitch = 2;

		if (octave >= 0) octave = octave - 1;
		coded_pitch = coded_pitch + 12 * (octave + 2);
		if (pitch.equals("Rest")) coded_pitch = 0;

		return coded_pitch;
	}


	/***************************************************************************/


	private int encodeDuration(String duration)
	// Returns the code used by Note objects that corresponds to the given
	// table code for duration
	{
		int coded_duration = 0;

		if (duration.equals("eighth")) coded_duration = 2;
		else if (duration.equals("quarter")) coded_duration = 4;
		else if (duration.equals("half")) coded_duration = 8;
		else if (duration.equals("whole")) coded_duration = 16;

		return coded_duration;
	}


	/***************************************************************************/


	private int encodeTied(String tied, int previous)
	// Returns the code used by Note objects that corresponds to the given
	// table code for whether or not a note is tied to previous or subsequent notes
	{
		int coded_tied = 0;

		if (tied.equals("not tied ahead") && previous < 3) coded_tied = 1;
		else if (tied.equals("not tied ahead") && previous > 2) coded_tied = 2;
		else if (tied.equals("tied ahead") && previous < 3) coded_tied = 3;
		else if (tied.equals("tied ahead") && previous > 2) coded_tied = 4;

		return coded_tied;
	}


	/***************************************************************************/


	private String decodePitch(int pitch, int key_signature, int quality, boolean want_octave)
	// Takes the key_signature and pitch as encoded by Composition and Note objects,
	// and returns the note name (if want_octave is false) and the register (if octave
	// is true) as encoded on the display table. An invalid pitch is simply stored as
	// a blank note.
	{
		MusicTranslator translator = new MusicTranslator();
		String results = translator.interperet_pitch(pitch, key_signature, quality);
		if (results.equals("Rest") && want_octave == false)
			return results;
		else if (results.equals("Rest") && want_octave == true)
			return "1";
		else if (results.equals("Invalid Pitch"))
			return new String(" ");
		else
		{
			StringTokenizer tokens = new StringTokenizer(results);
			results = tokens.nextToken();
			if (want_octave == false)
				return results;
			else
			{
				results = tokens.nextToken();
				return results;
			}
		}
	}


	/***************************************************************************/


	private String decodeDuration(int duration)
	// Takes a note duration as encoded by Note objects,
	// and returns the duration name as encoded on the display table.
	// An invalid duration causes a quarter note to be returned.
	{
		String duration_name;

		if (duration == 2) duration_name = new String("eighth");
		else if (duration == 4) duration_name = new String("quarter");
		else if (duration == 8) duration_name = new String("half");
		else if (duration == 16) duration_name = new String("whole");
		else duration_name = new String("quarter");

		return duration_name;
	}


	/***************************************************************************/


	private String decodeTied(int tied)
	// Takes a note's tied status as encoded by Note objects,
	// and returns the tied name as encoded on the display table.
	// An invalid tied status causes a not tied to be returned.
	{
		String tied_name;

		if (tied == 1 || tied == 2) tied_name = new String("not tied ahead");
		else if (tied == 3 || tied == 4) tied_name = new String("tied ahead");
		else tied_name = new String("not tied ahead");

		return tied_name;
	}
}