/*
SpeciesChecker
by Cory McKay

CLASS NAME:		PreferencesDialog
CLASS TYPE:		GUI - dialog box
DESCRIPTION:	A dialog box that allows the user to select which rules will be applied
				when analyses are done for the various species. These preferences are
				saved to disk (in a file called SCPref.spr) so that they will persist.
				The file consists of a sequence of 1's and 0's, one for each rule, starting
				at the general rules and ending at fifth species. 1 means that the rule
				should be applied and 0 means that it shouldn't. A default SCPref file
				is generated if none is present. The SCPref file is automatically loaded
				when an object of this class is instantiated.
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;


class PreferencesDialog extends JDialog
{
	/* FIELDS ******************************************************************/


	final static int NUM_CANTUS_RULES = 33;		// the number of cantus firmus rules that can be applied
	final static int NUM_FIRST_RULES = 48;		// the number of first species rules that can be applied
	final static int NUM_SECOND_RULES = 42;		// the number of second species rules that can be applied
	final static int NUM_THIRD_RULES = 44;		// the number of third species rules that can be applied
	final static int NUM_FORTH_RULES = 44;		// the number of forth species rules that can be applied
	final static int NUM_FIFTH_RULES = 58;		// the number of fifth species rules that can be applied
	private JCheckBox[] cantus_rules;			// the array of checkboxes for the cantus rules
	private JCheckBox[] first_species_rules;	// the array of checkboxes for the cantus rules
	private JCheckBox[] second_species_rules;	// the array of checkboxes for the cantus rules
	private JCheckBox[] third_species_rules;	// the array of checkboxes for the cantus rules
	private JCheckBox[] forth_species_rules;	// the array of checkboxes for the cantus rules
	private JCheckBox[] fifth_species_rules;	// the array of checkboxes for the cantus rules
	private int next_index; 					// used to maintain position when reading from the disk
	private HelpDialog help_dialog;				// the dialog box to get help from


	/* CONSTRUCTOR *************************************************************/


	public PreferencesDialog(JFrame owner, HelpDialog helpdialog)
	{
		// Give the dialog box its owner, its title, its size and make it non-modal
		super(owner, "Preferences", false);
		setSize(650, 545);
		help_dialog = helpdialog;


		// Set up the panel for choosing the cantus firmus rules
		JPanel cantusPanel = new JPanel(new GridLayout(NUM_CANTUS_RULES,1));
		JCheckBox[] cantus_rules_1 =           // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check that correct note durations are used"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check for tied notes"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check total number of leaps"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check that leaps are filled in"),
			new JCheckBox("Check that large leaps change direction"),
			new JCheckBox("Check that direction is changed after a leap"),
			new JCheckBox("Check that the first note is the tonic"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that second to last note is approached properly"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check that a note does not reoccur too often"),
			new JCheckBox("Check rigorously that a note does not reoccur too often"),
			new JCheckBox("Check interval outlined by a series of notes"),
			new JCheckBox("Check for too many consecutive stepwise notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for repetitions of groups of tones"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check length of piece")
		};
		cantus_rules = cantus_rules_1;
		cantus_rules_1 = null;
		for (int i = 0; i < NUM_CANTUS_RULES; i++)
			cantusPanel.add(cantus_rules[i]);


		// Set up the panel for choosing the first species rules
		JPanel firstSpeciesPanel = new JPanel(new GridLayout(NUM_FIRST_RULES + 3,1));
		JLabel first_horizontal_label = new JLabel("Rules relating to the counterpoint line only");
		JLabel first_blank_label = new JLabel();
		JLabel first_vertical_label = new JLabel("Rules relating to both lines");
		first_horizontal_label.setFont(new Font("Serif", Font.BOLD, 16));
		first_vertical_label.setFont(new Font("Serif", Font.BOLD, 16));
		JCheckBox[] first_species_rules_1 =          // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check that correct note durations are used"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check that tied notes are used correctly"),
			new JCheckBox("Check for too many consecutive tied notes"),
			new JCheckBox("Check for too many tied notes in total"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check total number of leaps"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check that leaps are filled in"),
			new JCheckBox("Check that large leaps change direction"),
			new JCheckBox("Check that direction is changed after a leap"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that second to last note is approached properly"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check that a note does not reoccur too often"),
			new JCheckBox("Check rigorously that a note does not reoccur too often"),
			new JCheckBox("Check interval outlined by a series of notes"),
			new JCheckBox("Check for too many consecutive stepwise notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for repetitions of groups of tones"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check vertical intervals"),
			new JCheckBox("Check unisons"),
			new JCheckBox("Check that vertical twelfths are treated properly"),
			new JCheckBox("Check that perfect intervals are approached properly"),
			new JCheckBox("Check successive unisons and octaves"),
			new JCheckBox("Check for too many consecutive identical intervals"),
			new JCheckBox("Check for too much consecutive parallel motion"),
			new JCheckBox("Check leaps from perfect intervals"),
			new JCheckBox("Check lines for improper simultaneous leaps"),
			new JCheckBox("Check for simultaneous climaxes"),
			new JCheckBox("Check for voice crossing"),
			new JCheckBox("Check for voice overlap"),
			new JCheckBox("Check the beginning interval"),
			new JCheckBox("Check the last interval"),
			new JCheckBox("Check the approach to the last interval")
			};
		first_species_rules = first_species_rules_1;
		first_species_rules_1 = null;
		firstSpeciesPanel.add(first_horizontal_label);
		for (int i = 0; i < NUM_FIRST_RULES; i++)
		{
			if (i == 33)
			{
				firstSpeciesPanel.add(first_blank_label);
				firstSpeciesPanel.add(first_vertical_label);
			}
			firstSpeciesPanel.add(first_species_rules[i]);
		}


		// Set up the panel for choosing the second species rules
		JPanel secondSpeciesPanel = new JPanel(new GridLayout(NUM_SECOND_RULES + 3,1));
		JLabel second_horizontal_label = new JLabel("Rules relating to the counterpoint line only");
		JLabel second_blank_label = new JLabel();
		JLabel second_vertical_label = new JLabel("Rules relating to both lines");
		second_horizontal_label.setFont(new Font("Serif", Font.BOLD, 16));
		second_vertical_label.setFont(new Font("Serif", Font.BOLD, 16));
		JCheckBox[] second_species_rules_1 =          // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check that correct note durations are used"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check for tied notes"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check for leaps across measure lines"),
			new JCheckBox("Check that leaps are filled in"),
			new JCheckBox("Check that leaps of a fourth are treated properly"),
			new JCheckBox("Check that large leaps change direction"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that second to last note is approached properly"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check interval outlined by a series of notes"),
			new JCheckBox("Check for too many consecutive stepwise notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for repetitions of groups of tones"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check vertical intervals"),
			new JCheckBox("Check unisons"),
			new JCheckBox("Check that vertical twelfths are treated properly"),
			new JCheckBox("Check that perfect intervals are approached properly"),
			new JCheckBox("Check for parallel perfect intervals on successive strong beats"),
			new JCheckBox("Check for too many consecutive identical intervals"),
			new JCheckBox("Check leaps from perfect intervals"),
			new JCheckBox("Check for simultaneous climaxes"),
			new JCheckBox("Check for voice crossing"),
			new JCheckBox("Check for voice overlap"),
			new JCheckBox("Check the beginning interval"),
			new JCheckBox("Check the last interval"),
			new JCheckBox("Check the approach to the last interval")
		};
		second_species_rules = second_species_rules_1;
		second_species_rules_1 = null;
		secondSpeciesPanel.add(second_horizontal_label);
		for (int i = 0; i < NUM_SECOND_RULES; i++)
		{
			if (i == 29)
			{
				secondSpeciesPanel.add(second_blank_label);
				secondSpeciesPanel.add(second_vertical_label);
			}
			secondSpeciesPanel.add(second_species_rules[i]);
		}


		// Set up the panel for choosing the third species rules
		JPanel thirdSpeciesPanel = new JPanel(new GridLayout(NUM_THIRD_RULES + 3,1));
		JLabel third_horizontal_label = new JLabel("Rules relating to the counterpoint line only");
		JLabel third_blank_label = new JLabel();
		JLabel third_vertical_label = new JLabel("Rules relating to both lines");
		third_horizontal_label.setFont(new Font("Serif", Font.BOLD, 16));
		third_vertical_label.setFont(new Font("Serif", Font.BOLD, 16));
		JCheckBox[] third_species_rules_1 =          // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check that correct note durations are used"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check for tied notes"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check for leaps across measure lines"),
			new JCheckBox("Check that leaps are filled in"),
			new JCheckBox("Check that leaps change direction"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that second to last note is approached properly"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check interval outlined by a series of notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check vertical intervals"),
			new JCheckBox("Check unisons"),
			new JCheckBox("Check nota cambiata"),
			new JCheckBox("Check double neighbour"),
			new JCheckBox("Check dissonant neighbour notes"),
			new JCheckBox("Check distance between lines"),
			new JCheckBox("Check that perfect intervals are approached properly"),
			new JCheckBox("Check for parallel perfect intervals"),
			new JCheckBox("Check separation between perfect intervals"),
			new JCheckBox("Check for parallel perfect intervals on successive strong beats"),
			new JCheckBox("Check for too many consecutive identical intervals"),
			new JCheckBox("Check leaps from perfect intervals"),
			new JCheckBox("Check for simultaneous climaxes"),
			new JCheckBox("Check for voice crossing"),
			new JCheckBox("Check for voice overlap"),
			new JCheckBox("Check the beginning interval"),
			new JCheckBox("Check the last interval"),
			new JCheckBox("Check the approach to the last interval")
		};
		third_species_rules = third_species_rules_1;
		third_species_rules_1 = null;
		thirdSpeciesPanel.add(third_horizontal_label);
		for (int i = 0; i < NUM_THIRD_RULES; i++)
		{
			if (i == 26)
			{
				thirdSpeciesPanel.add(third_blank_label);
				thirdSpeciesPanel.add(third_vertical_label);
			}
			thirdSpeciesPanel.add(third_species_rules[i]);
		}


		// Set up the panel for choosing the fourth species rules
		JPanel forthSpeciesPanel = new JPanel(new GridLayout(NUM_FORTH_RULES + 3,1));
		JLabel forth_horizontal_label = new JLabel("Rules relating to the counterpoint line only");
		JLabel forth_blank_label = new JLabel();
		JLabel forth_vertical_label = new JLabel("Rules relating to both lines");
		forth_horizontal_label.setFont(new Font("Serif", Font.BOLD, 16));
		forth_vertical_label.setFont(new Font("Serif", Font.BOLD, 16));
		JCheckBox[] forth_species_rules_1 =          // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check that correct note durations are used"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check that tied notes are used correctly"),
			new JCheckBox("Check for consecutive tied notes"),
			new JCheckBox("Check that ties are properly placed"),
			new JCheckBox("Check that species is not broken too frequently"),
			new JCheckBox("Check consecutive notes involved in broken species"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check for leaps across measure lines"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check for too many consecutive stepwise notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check vertical intervals"),
			new JCheckBox("Check distance between lines"),
			new JCheckBox("Check that perfect intervals are approached properly"),
			new JCheckBox("Check for voice crossing"),
			new JCheckBox("Check for voice overlap"),
			new JCheckBox("Check the beginning interval"),
			new JCheckBox("Check the last interval"),
			new JCheckBox("Check the approach to the last interval"),
			new JCheckBox("Check that dissonant suspensions resolve properly"),
			new JCheckBox("Check series of suspensions"),
			new JCheckBox("Check consecutive 9-8 suspensions"),
			new JCheckBox("Check consecutive 2-1 suspensions"),
			new JCheckBox("Check consecutive 4-5 suspensions"),
			new JCheckBox("Check for 7-8 suspensions"),
			new JCheckBox("Check for 2-1 suspensions"),
			new JCheckBox("Check for enough consonant suspensions"),
			new JCheckBox("Check for enough dissonant suspensions")
		};
		forth_species_rules = forth_species_rules_1;
		forth_species_rules_1 = null;
		forthSpeciesPanel.add(forth_horizontal_label);
		for (int i = 0; i < NUM_FORTH_RULES; i++)
		{
			if (i == 27)
			{
				forthSpeciesPanel.add(forth_blank_label);
				forthSpeciesPanel.add(forth_vertical_label);
			}
			forthSpeciesPanel.add(forth_species_rules[i]);
		}


		// Set up the panel for choosing the fifth species rules
		JPanel fifthSpeciesPanel = new JPanel(new GridLayout(NUM_FIFTH_RULES + 3,1));
		JLabel fifth_horizontal_label = new JLabel("Rules relating to the counterpoint line only");
		JLabel fifth_blank_label = new JLabel();
		JLabel fifth_vertical_label = new JLabel("Rules relating to both lines");
		fifth_horizontal_label.setFont(new Font("Serif", Font.BOLD, 16));
		fifth_vertical_label.setFont(new Font("Serif", Font.BOLD, 16));
		JCheckBox[] fifth_species_rules_1 =          // a dummy variable used to initialize the array
		{
			new JCheckBox("Check that notes are in correct key"),
			new JCheckBox("Check for whole notes"),
			new JCheckBox("Check that notes are struck at correct places"),
			new JCheckBox("Check rhythmic configuration of first measure"),
			new JCheckBox("Check rhythmic configuration of other measures"),
			new JCheckBox("Check for prolonged use of a single rhythmic value"),
			new JCheckBox("Check rhythmic variation"),
			new JCheckBox("Check for changes of rhythmic configuration"),
			new JCheckBox("Check eighth notes"),
			new JCheckBox("Check that tied notes are used correctly"),
			new JCheckBox("Check for consecutive tied notes"),
			new JCheckBox("Check that ties are rhythmically correct"),
			new JCheckBox("Check if rests are present"),
			new JCheckBox("Check range of line"),
			new JCheckBox("Check for repeated notes"),
			new JCheckBox("Check horizontal intervals"),
			new JCheckBox("Check for leaps from the leading tone"),
			new JCheckBox("Check for enough large leaps"),
			new JCheckBox("Check for too many large leaps"),
			new JCheckBox("Check for too many consecutive leaps"),
			new JCheckBox("Check for consecutive leaps in the same direction"),
			new JCheckBox("Check for leaps across bar lines"),
			new JCheckBox("Check that leaps are filled in"),
			new JCheckBox("Check that large leaps change direction"),
			new JCheckBox("Check that the last note is the tonic"),
			new JCheckBox("Compare first and last notes"),
			new JCheckBox("Check that second to last note is approached properly"),
			new JCheckBox("Check that the last note is approached stepwise"),
			new JCheckBox("Check that endings are properly treated in minor keys"),
			new JCheckBox("Check for properly treated climax"),
			new JCheckBox("Check interval outlined by a series of notes"),
			new JCheckBox("Check number of consecutive notes moving in the same direction"),
			new JCheckBox("Check number of changes of direction"),
			new JCheckBox("Check for ornamentation"),
			new JCheckBox("Check vertical intervals"),
			new JCheckBox("Check unisons"),
			new JCheckBox("Check nota cambiata"),
			new JCheckBox("Check double neighbour"),
			new JCheckBox("Check dissonant neighbour notes"),
			new JCheckBox("Check distance between lines"),
			new JCheckBox("Check that perfect intervals are approached properly"),
			new JCheckBox("Check for parallel perfect intervals"),
			new JCheckBox("Check separation between perfect intervals"),
			new JCheckBox("Check for too many consecutive identical intervals"),
			new JCheckBox("Check leaps from perfect intervals"),
			new JCheckBox("Check for simultaneous climaxes"),
			new JCheckBox("Check for voice crossing"),
			new JCheckBox("Check for voice overlap"),
			new JCheckBox("Check the beginning interval"),
			new JCheckBox("Check the last interval"),
			new JCheckBox("Check the approach to the last interval"),
			new JCheckBox("Check that dissonant suspensions resolve properly"),
			new JCheckBox("Check series of suspensions"),
			new JCheckBox("Check consecutive 9-8 suspensions"),
			new JCheckBox("Check consecutive 2-1 suspensions"),
			new JCheckBox("Check consecutive 4-5 suspensions"),
			new JCheckBox("Check for 7-8 suspensions"),
			new JCheckBox("Check for 2-1 suspensions")
		};
		fifth_species_rules = fifth_species_rules_1;
		fifth_species_rules_1 = null;
		fifthSpeciesPanel.add(fifth_horizontal_label);
		for (int i = 0; i < NUM_FIFTH_RULES; i++)
		{
			if (i == 34)
			{
				fifthSpeciesPanel.add(fifth_blank_label);
				fifthSpeciesPanel.add(fifth_vertical_label);
			}
			fifthSpeciesPanel.add(fifth_species_rules[i]);
		}


		// Get saved values for all the check boxes. Set to default if not saved.
		try {initialize_check_boxes();}
		catch (IOException e) {use_defaults();}


		// Set up the tabbed pane
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Cantus Firmus", new JScrollPane(cantusPanel));
		tabbedPane.addTab("First Species", new JScrollPane(firstSpeciesPanel));
		tabbedPane.addTab("Second Species", new JScrollPane(secondSpeciesPanel));
		tabbedPane.addTab("Third Species", new JScrollPane(thirdSpeciesPanel));
		tabbedPane.addTab("Fourth Species", new JScrollPane(forthSpeciesPanel));
		tabbedPane.addTab("Fifth Species", new JScrollPane(fifthSpeciesPanel));


		// Configure a Save Preferences button to save the preferences to disk and close the dialog box
		JButton ok = new JButton("Save Preferences");
		ok.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				try
				{
					File to_file = new File("SCPref.spr");
					FileOutputStream to = new FileOutputStream(to_file);
					DataOutputStream writer = new DataOutputStream(to);
					for (int i = 0; i < NUM_CANTUS_RULES; i++)
					{
						if (cantus_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
					for (int i = 0; i < NUM_FIRST_RULES; i++)
					{
						if (first_species_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
					for (int i = 0; i < NUM_SECOND_RULES; i++)
					{
						if (second_species_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
					for (int i = 0; i < NUM_THIRD_RULES; i++)
					{
						if (third_species_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
					for (int i = 0; i < NUM_FORTH_RULES; i++)
					{
						if (forth_species_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
					for (int i = 0; i < NUM_FIFTH_RULES; i++)
					{
						if (fifth_species_rules[i].isSelected() == true) writer.writeBytes("1");
						else writer.writeBytes("0");
					}
				} catch (IOException e) {}

				setVisible(false);
			}
		});


		// Configure a Help button to bring up the Help Dialog Box
		JButton help = new JButton("Help");
		help.addActionListener(new HelpAction());


		// Set up the panel of buttons
		JPanel panel = new JPanel();
		panel.add(help);
		panel.add(ok);


		// Display items
		Container contentPane = getContentPane();
		contentPane.add(tabbedPane,	BorderLayout.CENTER);
		contentPane.add(panel, BorderLayout.SOUTH);
	}


	/* PUBLIC METHODS **********************************************************/


	public int[] get_cantus_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_CANTUS_RULES];
		for (int i = 0; i < NUM_CANTUS_RULES; i++)
		{
			if (cantus_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/***************************************************************************/


	public int[] get_first_species_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_FIRST_RULES];
		for (int i = 0; i < NUM_FIRST_RULES; i++)
		{
			if (first_species_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/***************************************************************************/


	public int[] get_second_species_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_SECOND_RULES];
		for (int i = 0; i < NUM_SECOND_RULES; i++)
		{
			if (second_species_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/***************************************************************************/


	public int[] get_third_species_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_THIRD_RULES];
		for (int i = 0; i < NUM_THIRD_RULES; i++)
		{
			if (third_species_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/***************************************************************************/


	public int[] get_forth_species_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_FORTH_RULES];
		for (int i = 0; i < NUM_FORTH_RULES; i++)
		{
			if (forth_species_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/***************************************************************************/


	public int[] get_fifth_species_rules()
	// Return the array used to determine which rules will be applied. 1 means that the
	// rule should be applied and 0 means that it shouldn't.
	{
		int[] return_array = new int[NUM_FIFTH_RULES];
		for (int i = 0; i < NUM_FIFTH_RULES; i++)
		{
			if (fifth_species_rules[i].isSelected() == true)  return_array[i] = 1;
			else return_array[i] = 0;
		}
		return return_array;
	}


	/* PRIVATE METHODS *********************************************************/


	private void initialize_check_boxes()
		throws IOException
	// Set the value of the check boxes based on the contents of the SCPref.spr file
	{
		File src = new File("SCPref.spr");
		char[] chars = new char[(int) src.length()];
		src = null;

		InputStream in;
		in = new FileInputStream("SCPref.spr");
		int ch;
		for (int i = 0; (ch = in.read()) != -1; i++)
			chars[i] = (char)ch;
		in.close();

		next_index = 0;
		for (int i = 0; i < NUM_CANTUS_RULES; i++)
			cantus_rules[i].setSelected(get_next_value(chars));
		for (int i = 0; i < NUM_FIRST_RULES; i++)
			first_species_rules[i].setSelected(get_next_value(chars));
		for (int i = 0; i < NUM_SECOND_RULES; i++)
			second_species_rules[i].setSelected(get_next_value(chars));
		for (int i = 0; i < NUM_THIRD_RULES; i++)
			third_species_rules[i].setSelected(get_next_value(chars));
		for (int i = 0; i < NUM_FORTH_RULES; i++)
			forth_species_rules[i].setSelected(get_next_value(chars));
		for (int i = 0; i < NUM_FIFTH_RULES; i++)
			fifth_species_rules[i].setSelected(get_next_value(chars));
		next_index = 0;
	}


	/***************************************************************************/


	private void use_defaults()
	// Set the values of the checkboxes to apply all rules except
	// those indicated
	{
		for (int i = 0; i < NUM_CANTUS_RULES; i++)
			cantus_rules[i].setSelected(true);
		cantus_rules[25].setSelected(false);
		cantus_rules[28].setSelected(false);
		for (int i = 0; i < NUM_FIRST_RULES; i++)
			first_species_rules[i].setSelected(true);
		first_species_rules[26].setSelected(false);
		first_species_rules[29].setSelected(false);
		first_species_rules[39].setSelected(false);
		for (int i = 0; i < NUM_SECOND_RULES; i++)
			second_species_rules[i].setSelected(true);
		second_species_rules[23].setSelected(false);
		second_species_rules[25].setSelected(false);
		second_species_rules[27].setSelected(false);
		second_species_rules[31].setSelected(false);
		for (int i = 0; i < NUM_THIRD_RULES; i++)
			third_species_rules[i].setSelected(true);
		third_species_rules[22].setSelected(false);
		third_species_rules[23].setSelected(false);
		third_species_rules[36].setSelected(false);
		for (int i = 0; i < NUM_FORTH_RULES; i++)
			forth_species_rules[i].setSelected(true);
		forth_species_rules[16].setSelected(false);
		for (int i = 0; i < NUM_FIFTH_RULES; i++)
			fifth_species_rules[i].setSelected(true);
		fifth_species_rules[22].setSelected(false);
		fifth_species_rules[30].setSelected(false);
		fifth_species_rules[31].setSelected(false);
	}


	/***************************************************************************/


	private boolean get_next_value(char[] chars)
	// Get the next value in the character array, and return true if it
	// is 1 and false if it is 0
	{
		boolean return_value;
		if (chars[next_index] == '1') return_value = true;
		else return_value = false;
		next_index++;
		return return_value;
	}


	/* INTERNAL CLASSES ********************************************************/


	class HelpAction extends AbstractAction
	// Display a dialog box containing the on-line help
	{
		public HelpAction() {super("Help Topics");}
		public void actionPerformed(ActionEvent event)
		{
			if (help_dialog == null)
				help_dialog = new HelpDialog();
			help_dialog.show();
		}
	}
}