/*
SpeciesChecker
by Cory McKay

CLASS NAME:		SplitPaneFrame
CLASS TYPE:		GUI - main portion
DESCRIPTION:	This is the main portion of the SpeciesChecker GUI.
LIMITATIONS:	Only compatible with species counterpoint, not 4-part harmonies.
*/


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.io.File;


class SplitPaneFrame extends JFrame
{
	/* FIELDS ******************************************************************/


	// GUI Regions
	private JTextArea top_area;		// displays the global information about the current composition
	private JTable middle_area;		// displays the notes of the current composition
	private JTextArea bottom_area;	// displays the results of analyses and messages to the user
	private JSplitPane inner_pane;	// displays middle_area and bottom_area
	private JSplitPane outer_pane;	// displays tope_area and inner_pane


	// Dialog Boxes
	private NewDialog new_dialog;							// for entering information to create a new composition
	private JFileChooser opener;							// to open files
	private JFileChooser saver;								// to save files
	private InsertRowDialog insert_row_dialog;				// to enter info about inserting a new row into a composition
	private AddCounterpointDialog add_counterpoint_dialog;	// to enter info on adding a counterpoint to a cantus
	private PreferencesDialog preferences_dialog;			// to select which analysis rules are applied
	private HelpDialog help_dialog;							// displays help information
	private AboutDialog about_dialog;						// displays basic program information


	// Menu Fields
	private JMenuItem new_item;						// configure a new composition
	private JMenuItem open_item;					// open a saved composition
	private JMenuItem close_item;					// close the current composition
	private JMenuItem save_item;					// save the current composition under its current name
	private JMenuItem save_as_item;					// save the current composition under a new name
	private JMenuItem exit_item;					// quit the program
	private JMenuItem insert_item;					// insert a new row into the composition
	private JMenuItem delete_item;					// remove a row from the composition
	private JMenuItem add_item;						// add a counterpoint to a cantus
	private JMenuItem remove_item;					// remove the counterpoint from a composition, leaving the cantus
	private JMenuItem preferences_item;				// choose which rules will be applied
	private JMenuItem analyze_composition_item;		// analyze the entirety of the current composition
	private JMenuItem analyze_counterpoint_item;	// analyze only the counterpoint of the current composition
	private JMenuItem analyze_cantus_item;			// analyze only the cantus of the current composition
	private JMenuItem generate_item;				// generate a new cantus
	private JMenuItem help_item;					// view the on-line help
	private JMenuItem about_item;					// view information about the program and its author


	// Table Fields
	private NotesTableModel table_model;				// the model for the middle_area table
	private TableColumnModel column_model;				// a model of the column of the table
	private TableColumn cantus_pitch_column;			// the cantus pitch column of the table
	private TableColumn cantus_octave_column;			// the cantus octave column of the table
	private TableColumn counterpoint_pitch_column;		// the counterpoint pitch column of the table
	private TableColumn counterpoint_octave_column;		// the counterpoint octave column of the table
	private TableColumn counterpoint_duration_column;	// the counterpoint duration column of the table
	private TableColumn counterpoint_tied_column;		// the counterpoint tied ahead column of the table
	private JComboBox cantus_pitch_combo;				// the combo box used to select the cantus note pitch
	private JComboBox cantus_octave_combo;				// the combo box used to select the cantus note octave
	private JComboBox counterpoint_pitch_combo;			// the combo box used to select the counterpoint note pitch
	private JComboBox counterpoint_octave_combo;		// the combo box used to select the counterpoint note octave
	private JComboBox counterpoint_duration_combo;		// the combo box used to select the counterpoint note duration
	private JComboBox tied_combo;						// the combo box used to select whether a note is tied


	// Musical Data Structures
	private Composition piece;	// the composition that is currently being dealt with
	private Voice cantus;		// the voice in the composition that is designated as the cantus
	private Voice counterpoint;	// the voice in the composition that is designated as the counterpoint


	// Other Fields
	String filename;	// the path of the currently open file. Null if no file is open.


	/* CONSTRUCTOR *************************************************************/


	public SplitPaneFrame()
	{
		// Set the initial data structures to empty
		piece = null;
		cantus = null;
		counterpoint = null;
		filename = null;


		// Set the size and title of the frame
		setSize(800, 572);
		setTitle("SpeciesChecker 1.0");


		// Quit the program when the exit box is pressed. Ask user if wants to save.
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				if (piece != null)
				{
					int response = JOptionPane.showConfirmDialog(SplitPaneFrame.this, "Do you wish " +
						"to save your work first?", "Warning",
						JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
					if (response == JOptionPane.YES_OPTION)
						save();
				}
				System.exit(0);
			}
		});


		// Create and populate the menus. Add separators where appropriate.
		JMenu file_menu = new JMenu("File");
		new_item = file_menu.add(new NewAction());
		open_item = file_menu.add(new OpenAction());
		close_item = file_menu.add(new CloseAction());
		file_menu.addSeparator();
		save_item = file_menu.add(new SaveAction());
		save_as_item = file_menu.add(new SaveAsAction());
		file_menu.addSeparator();
		exit_item = file_menu.add(new ExitAction());
		JMenu edit_menu = new JMenu("Edit");
		insert_item = edit_menu.add(new InsertRowAction());
		delete_item = edit_menu.add(new DeleteRowAction());
		edit_menu.addSeparator();
		add_item = edit_menu.add(new AddCounterpointAction());
		remove_item = edit_menu.add(new RemoveCounterpointAction());
		edit_menu.addSeparator();
		preferences_item = edit_menu.add(new ChangePreferencesAction());
		JMenu action_menu = new JMenu("Action");
		analyze_composition_item = action_menu.add(new AnalyzeCompositionAction());
		analyze_counterpoint_item = action_menu.add(new AnalyzeCounterpointAction());
		analyze_cantus_item = action_menu.add(new AnalyzeCantusAction());
		action_menu.addSeparator();
		generate_item = action_menu.add(new GenerateCantusAction());
		JMenu help_menu = new JMenu("Help");
		help_item = help_menu.add(new HelpAction());
		about_item = help_menu.add(new AboutAction());


		// Add keyboard mnemonics and accelerators to menus and menu items
		file_menu.setMnemonic('F');
		edit_menu.setMnemonic('E');
		action_menu.setMnemonic('A');
		help_menu.setMnemonic('H');
		new_item.setMnemonic('N');
		open_item.setMnemonic('O');
		close_item.setMnemonic('C');
		save_item.setMnemonic('S');
		save_as_item.setMnemonic('A');
		exit_item.setMnemonic('X');
		insert_item.setMnemonic('I');
		delete_item.setMnemonic('D');
		add_item.setMnemonic('A');
		remove_item.setMnemonic('R');
		preferences_item.setMnemonic('P');
		analyze_composition_item.setMnemonic('T');
		analyze_counterpoint_item.setMnemonic('P');
		analyze_cantus_item.setMnemonic('C');
		generate_item.setMnemonic('G');
		help_item.setMnemonic('T');
		about_item.setMnemonic('A');
		new_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		open_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		save_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		exit_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
		analyze_composition_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, InputEvent.CTRL_MASK));
		analyze_counterpoint_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, InputEvent.CTRL_MASK));
		analyze_cantus_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, InputEvent.CTRL_MASK));
		generate_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK));
		help_item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.CTRL_MASK));


		// Disable appropriate menu items at startup
		update_menus();


		// Set up the menu bar
		JMenuBar menu_bar = new JMenuBar();
		menu_bar = new JMenuBar();
		setJMenuBar(menu_bar);
		menu_bar.add(file_menu);
		menu_bar.add(edit_menu);
		menu_bar.add(action_menu);
		menu_bar.add(help_menu);


		// Set up the top section of the frame, which displays global information about the composition
		// This is a text area, ines do not wrap, the default size is four rows and the frame is not editable
		top_area = new JTextArea();
		top_area.setLineWrap(false);
		top_area.setRows(4);
		top_area.setEditable(false);
		top_area.setText("There is currently no loaded composition.");


		// Set up the middle section of the frame, which holds information about the notes in the composition
		// This is a table with the column titles specified in columnNames
		// Adds and sets up the combo boxes in the table
		Object[] columnNames = {new String("Measure"), new String("Beat"), new String("Cantus Pitch"),
			new String("Cantus Register"), new String("CP Pitch"), new String("CP Register"),
			new String("CP Duration"), new String ("CP Tied")};
		table_model = new NotesTableModel(columnNames, 0);
		middle_area = new JTable(table_model);
		middle_area.setColumnSelectionAllowed(false);
		middle_area.setRowSelectionAllowed(false);
		cantus_pitch_combo = new JComboBox();
		cantus_octave_combo = new JComboBox();
		counterpoint_pitch_combo = new JComboBox();
		counterpoint_octave_combo = new JComboBox();
		counterpoint_duration_combo = new JComboBox();
		tied_combo = new JComboBox();
		table_model.labelPitchCombo(cantus_pitch_combo);
		table_model.labelOctaveCombo(cantus_octave_combo);
		table_model.labelPitchCombo(counterpoint_pitch_combo);
		table_model.labelOctaveCombo(counterpoint_octave_combo);
		table_model.labelDurationCombo(counterpoint_duration_combo);
		table_model.labelTiedCombo(tied_combo);
		column_model = middle_area.getColumnModel();
		cantus_pitch_column = column_model.getColumn(2);
		cantus_octave_column = column_model.getColumn(3);
		counterpoint_pitch_column = column_model.getColumn(4);
		counterpoint_octave_column = column_model.getColumn(5);
		counterpoint_duration_column = column_model.getColumn(6);
		counterpoint_tied_column = column_model.getColumn(7);
		cantus_pitch_column.setCellEditor(new DefaultCellEditor(cantus_pitch_combo));
		cantus_octave_column.setCellEditor(new DefaultCellEditor(cantus_octave_combo));
		counterpoint_pitch_column.setCellEditor(new DefaultCellEditor(counterpoint_pitch_combo));
		counterpoint_octave_column.setCellEditor(new DefaultCellEditor(counterpoint_octave_combo));
		counterpoint_duration_column.setCellEditor(new DefaultCellEditor(counterpoint_duration_combo));
		counterpoint_tied_column.setCellEditor(new DefaultCellEditor(tied_combo));


		// Set up the bottom section of the frame, which holds the results of analyses
		// This is a text area, lines wrap based on words and the frame is not editable
		bottom_area = new JTextArea();
		bottom_area.setRows(7);
		bottom_area.setLineWrap(true);
		bottom_area.setWrapStyleWord(true);
		bottom_area.setEditable(false);


		// Put the middle and bottom sections in split frames and give them scroll bars
		// Set up the split frames so that they do not update size on the fly and are one-touch expandable
		inner_pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			new JScrollPane(middle_area), new JScrollPane(bottom_area));
		inner_pane.setResizeWeight(0.65);
		inner_pane.setContinuousLayout(false);
		inner_pane.setOneTouchExpandable(true);
		outer_pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top_area, inner_pane);
		outer_pane.setContinuousLayout(false);
		outer_pane.setOneTouchExpandable(true);


		// Put the split frames onto the main pane
		getContentPane().add(outer_pane, "Center");


		// Initialize the dialog boxes
		new_dialog = null;
		opener = null;
		saver = null;
		insert_row_dialog = null;
		add_counterpoint_dialog = null;
		help_dialog = new HelpDialog();
		preferences_dialog = new PreferencesDialog(this, help_dialog);
		about_dialog = null;
	}

	
	/* PRIVATE METHODS *********************************************************/


	private void update_menus()
	// Adjust the menu bar so that the appropriate items are enabled and disabled
	{
		// Disable certain menu items if there is no composition currently in memory
		if (piece == null)
		{
			close_item.setEnabled(false);
			save_item.setEnabled(false);
			save_as_item.setEnabled(false);
			insert_item.setEnabled(false);
			delete_item.setEnabled(false);
			add_item.setEnabled(false);
			remove_item.setEnabled(false);
			analyze_composition_item.setEnabled(false);
			analyze_counterpoint_item.setEnabled(false);
			analyze_cantus_item.setEnabled(false);
		}


		// Disable certain menu items if there is a composition in memory
		else
		{
			close_item.setEnabled(true);
			save_item.setEnabled(true);
			save_as_item.setEnabled(true);
			insert_item.setEnabled(true);
			delete_item.setEnabled(true);
			analyze_cantus_item.setEnabled(true);


			// Disable the Analyze Composition, Analyze Counterpoint and Remove Counterpoint commands
			// if only a cantus is present. Disable the Add Counterpoint command if there is always
			// a counterpoint present.
			if (piece.get_type_of_composition() != 0)
			{
				analyze_composition_item.setEnabled(true);
				analyze_counterpoint_item.setEnabled(true);
				add_item.setEnabled(false);
				remove_item.setEnabled(true);
			}
			else
			{
				analyze_composition_item.setEnabled(false);
				analyze_counterpoint_item.setEnabled(false);
				add_item.setEnabled(true);
				remove_item.setEnabled(false);
			}


			// Disable the Insert Note and Delete Noted commands if the composition is not second or fifth species
			if (piece.get_type_of_composition() != 2 && piece.get_type_of_composition() != 5)
			{
				insert_item.setEnabled(false);
				delete_item.setEnabled(false);
			}
			else
			{
				insert_item.setEnabled(true);
				delete_item.setEnabled(true);
			}
		}
	}


	/***************************************************************************/


	private void clear_displays()
	// Remove all data from the three main displays of the GUI
	{
		table_model.clearTable();
		top_area.setText("");
		bottom_area.setText("");
	}


	/***************************************************************************/


	private boolean check_save()
	// Creates a dialog box asking the user if he wishes to save his work.
	// Returns false if the cancel option is chosen, returns true otherwise.
	{
		if (piece != null)
		{
			int response = JOptionPane.showConfirmDialog(SplitPaneFrame.this, "Do you wish " +
				"to save your work first?", "Warning",
				JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
			if (response == JOptionPane.YES_OPTION)
			{
				save();
				return true;
			}
			if (response == JOptionPane.CANCEL_OPTION)
				return false;
		}
		return true;
	}


	/***************************************************************************/


	private void save()
	// Save the currently open Composition to disk. Use the current filename if one is already
	// selected. Open the Save As dialog box if there is no current filename.
	{
		if (filename == null) save_as();
		else carry_out_save(true, filename);
	}


	/***************************************************************************/


	private void save_as()
	// Open the file chooser dialogue box and save the currently opened Composition to disk.
	{
		int saver_return; // what is returned by the JFileChooser object


		// Create a new JFileChooser if one doesn't exist already
		if (saver == null)
		{
			saver = new JFileChooser();
			saver.setCurrentDirectory(new File("."));
		}


		// Display the Save As dialog box and save the file
		// If the file already exists, ask the user if he wants to overwrite it
		saver_return = saver.showSaveDialog(SplitPaneFrame.this);
		if (saver_return == JFileChooser.APPROVE_OPTION) // only do if OK chosen
		{
			String new_filename = saver.getSelectedFile().getPath();
			carry_out_save(false, new_filename);
		}
	}


	/***************************************************************************/


	private void carry_out_save(boolean can_erase, String new_filename)
	// Save the currently open Compostion to disk using the name new_filename.
	// If the file already exists and can_erase is false, ask the user if he wants to
	// overwrite it. If the file is succesfully saved, change the value of filename
	// to new_filename.
	{
		table_model.getValuesFromTable(piece, cantus, counterpoint);
		FWriter writer = new FWriter();

		boolean go_ahead = true;
		File test_file = new File(new_filename);
		if (test_file.exists() && can_erase == false)
		{
			int response = JOptionPane.showConfirmDialog(SplitPaneFrame.this, "A file " +
				"by that name already exists. Do you wish to overwrite it?", "Warning",
				JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
			if (response != JOptionPane.YES_OPTION)
				go_ahead = false;
		}

		if (go_ahead == true)
		{
			try
			{
				writer.write_file(new_filename, piece);
				filename = new_filename;
				bottom_area.setText("");
			}
			catch (Exception e) {bottom_area.setText("UNABLE TO SAVE YOUR WORK.\n\n");}
		}
	}


	/***************************************************************************/


	private void perform_cantus_analysis()
	// Checks the cantus in the piece for errors using rules from the AnalyzerHorizontalCantus class.
	// Appends the results of the error analysis to bottom_area.
	// Applies only the rules selected in the preferences dialog box.
	{
		ControllerCantus cantus_controller = new ControllerCantus(piece);
		ErrorList errors_found = cantus_controller.check_cantus(preferences_dialog.get_cantus_rules());
		bottom_area.append("CHECKING CANTUS FIRMUS FOR ERRORS...\n\n");
		String an_error = errors_found.get_error();
		int error_count = 0;
		while (an_error != null)
		{
			bottom_area.append(an_error + "\n");
			an_error = errors_found.get_error();
			error_count++;
		}
		if (error_count == 0) bottom_area.append("No errors detected in cantus firmus.\n");
		bottom_area.append("\nFINISHED CHECKING CANTUS FIRMUS FOR ERRORS.");
	}


	/***************************************************************************/


	private void perform_counterpoint_analysis()
	// Checks the counterpoint in the piece for errors  using rules from the appropriate Analyzer classes
	// for the type of species. Appends the results of the error analysis to bottom_area.
	// Applies only the rules selected in the preferences dialog box.
	{
		ErrorList errors_found = null; // the list of errors detecte
		
		
		// Peform the error analysis, based on the kind of species
		if (piece.get_type_of_composition() == 1)
		{
			ControllerFirst counterpoint_controller = new ControllerFirst(piece);
			errors_found = counterpoint_controller.check_counterpoint(preferences_dialog.get_first_species_rules());
		}
		else if (piece.get_type_of_composition() == 2)
		{
			ControllerSecond counterpoint_controller = new ControllerSecond(piece);
			errors_found = counterpoint_controller.check_counterpoint(preferences_dialog.get_second_species_rules());
		}
		else if (piece.get_type_of_composition() == 3)
		{
			ControllerThird counterpoint_controller = new ControllerThird(piece);
			errors_found = counterpoint_controller.check_counterpoint(preferences_dialog.get_third_species_rules());
		}
		else if (piece.get_type_of_composition() == 4)
		{
			ControllerForth counterpoint_controller = new ControllerForth(piece);
			errors_found = counterpoint_controller.check_counterpoint(preferences_dialog.get_forth_species_rules());
		}
		else if (piece.get_type_of_composition() == 5)
		{
			ControllerFifth counterpoint_controller = new ControllerFifth(piece);
			errors_found = counterpoint_controller.check_counterpoint(preferences_dialog.get_fifth_species_rules());
		}

	
		// Display the results of the error analysis
		bottom_area.append("CHECKING COUNTERPOINT FOR ERRORS...\n\n");
		String an_error = errors_found.get_error();
		int error_count = 0;
		while (an_error != null)
		{
			bottom_area.append(an_error + "\n");
			an_error = errors_found.get_error();
			error_count++;
		}
		if (error_count == 0) bottom_area.append("No errors detected in the counterpoint.\n");
		bottom_area.append("\nFINISHED CHECKING COUNTERPOINT FOR ERRORS.");
	}


	/* INTERNAL CLASSES ********************************************************/


	class NewAction extends AbstractAction
	// Display a dialog box allowing the user to enter information about creating a new composition.
	// If the user presses OK, then the deal_with_new_composition method is called to load
	// the new composition and associated information into memory and update the display.
	// The composition will be populated with arbitrary but appropriately placed notes.
	{
		public NewAction() {super("New Composition");}
		public void actionPerformed(ActionEvent event)
		{
			if (check_save() == true) // check to see if the user is willing to overwrite a Composition
			                          // that is already loaded into memory
			{
				if (new_dialog == null)
					new_dialog = new NewDialog(SplitPaneFrame.this);
				new_dialog.show();
				if (new_dialog.was_updated() == true) // take action if the OK button was pressed
				{
					NewComposition new_dialog_results = new_dialog.get_new_composition();
					piece = new_dialog_results.new_composition;
					cantus = new_dialog_results.new_cantus_voice;
					counterpoint = new_dialog_results.new_counterpoint_voice;
					clear_displays();
					filename = null;
					update_menus();
					top_area.setText(new_dialog_results.composition_display);
					table_model.setupTable(piece, cantus, counterpoint);
					table_model.getValuesFromTable(piece, cantus, counterpoint);
				}
			}
		}
	}


	/***************************************************************************/


	class OpenAction extends AbstractAction
	// Bring up a JFileChooser dialog box to open A SpeciesChecker file
	// NOTE: will clear the screen if an invalid file is specified.
	{
		public OpenAction() {super("Open");}
		public void actionPerformed(ActionEvent event)
		{
			if (check_save() == true)
			{
				int opener_return; // what is returned by the JFileChooser object


				// Create a new JFileChooser if one doesn't exist already
				if (opener == null)
				{
					opener = new JFileChooser();
					opener.setCurrentDirectory(new File("."));
				}


				// Display the Open dialog box and load and display the file, clearing everything
				// if an invalid file is selected
				opener_return = opener.showOpenDialog(SplitPaneFrame.this);
				if (opener_return == JFileChooser.APPROVE_OPTION) // only do if OK chosen
				{
					clear_displays();
					filename = opener.getSelectedFile().getPath();
					FReader reader = new FReader();
					try
					{
						piece = reader.read_file(filename);
						cantus = piece.get_voice(0);
						if (piece.get_type_of_composition() != 0)
							counterpoint = piece.get_voice(1);
						else
							counterpoint = null;
						table_model.importIntoTable(piece, cantus, counterpoint);
						MusicTranslator translator = new MusicTranslator();
						top_area.setText(translator.produceCompositinHeader(piece, cantus, counterpoint));
					}
					catch (Exception e)
					{
						bottom_area.setText("You have selected an inappropriate file. Please try again.");
						piece = null;
						cantus = null;
						counterpoint = null;
					}
					update_menus();
				}
			}
		}
	}


	/***************************************************************************/


	class SaveAction extends AbstractAction
	// Save the current Composition under the current filename. Bring up a Save As dialog
	// box if there is no current filename.
	{
		public SaveAction() {super("Save");}
		public void actionPerformed(ActionEvent event)
		{
			save();
		}
	}


	/***************************************************************************/


	class SaveAsAction extends AbstractAction
	// Bring up a JFileChooser dialog box to save the current Composition as a new file
	{
		public SaveAsAction() {super("Save As");}
		public void actionPerformed(ActionEvent event)
		{
			save_as();
		}
	}


	/***************************************************************************/


	class CloseAction extends AbstractAction
	// Close the composition that is currently in memory after asking the user if he wants to save first
	{
		public CloseAction() {super("Close");}
		public void actionPerformed(ActionEvent event)
		{
			if (check_save() == true)
			{
				// Set the current analysis objects to null, update the menu items and clear the composition
				// information text area
				piece = null;
				cantus = null;
				counterpoint = null;
				clear_displays();
				filename = null;
				update_menus();
				top_area.setText("There is currently no loaded composition.");
			}
		}
	}


	/***************************************************************************/


	class ExitAction extends AbstractAction
	// Quit the program after asking the user if he wishes to save his work first
	{
		public ExitAction() {super("Exit");}
		public void actionPerformed(ActionEvent event)
		{
			if (check_save() == true)
				System.exit(0);
		}
	}


	/***************************************************************************/


	class InsertRowAction extends AbstractAction
	// Insert a row into the selected location. A dialog box is brought up allowing the user to choose
	// where to do this. It is not permitted to overwrite rows that already exist
	// This command is only appropriate for second or fifth species compositions.
	{
		public InsertRowAction() {super("Insert Row");}
		public void actionPerformed(ActionEvent event)
		{
			table_model.getValuesFromTable(piece, cantus, counterpoint);
			insert_row_dialog = new InsertRowDialog(SplitPaneFrame.this, piece.get_number_of_bars());
			insert_row_dialog.show();
			if (insert_row_dialog.was_updated() == true) // take action if the OK button was pressed
			{
				bottom_area.setText("");
				int result = insert_row_dialog.insert_row(table_model);
				if (result == -1)
					bottom_area.setText("SELECTED ROW ALREADY EXISTS");
			}
		}
	}


	/***************************************************************************/


	class DeleteRowAction extends AbstractAction
	// Delete the row that is selected by the user. Will not erase rows that contain the first beat
	// of a measure, as to do so would erase part of the cantus firmus
	// This command is only appropriate for second or fifth species compositions.
	{
		public DeleteRowAction() {super("Delete Row");}
		public void actionPerformed(ActionEvent event)
		{
			bottom_area.setText("");
			int selected_row = middle_area.getSelectedRow();
			if (selected_row == -1) bottom_area.setText("NO ROW SELECTED");
			else if (String.valueOf(middle_area.getValueAt(selected_row, 1)).equals("1"))
				bottom_area.setText("CANNOT ERASE A ROW THAT CONTAINS A NOTE OF THE CANTUS FIRMUS");
			else table_model.deleteRow(selected_row);
		}
	}


	/***************************************************************************/


	class RemoveCounterpointAction extends AbstractAction
	// Delete the counterpoint of a piece while retaining the cantus. First ask the user if he is sure
	// that he wants to do this.
	{
		public RemoveCounterpointAction() {super("Remove Counterpoint");}
		public void actionPerformed(ActionEvent event)
		{
			int response = JOptionPane.showConfirmDialog(SplitPaneFrame.this, "Are you sure you want to " +
				"delete your counterpoint?", "Warning", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
			if (response == JOptionPane.YES_OPTION)
			{
				// Create a new piece that contains the cantus but has no counterpoint
				table_model.getValuesFromTable(piece, cantus, counterpoint);
				cantus = piece.get_voice(0);
				counterpoint = null;
				int quality = piece.get_quality();
				int key = piece.get_key_signature();
				int meter = piece.get_time_signature();
				int bars = piece.get_number_of_bars();
				piece = new Composition(0, quality, key, meter, bars);
				piece.insert_ready_voice(0, cantus);


				// Update the displays
				clear_displays();
				table_model.importIntoTable(piece, cantus, counterpoint);
				MusicTranslator translator = new MusicTranslator();
				top_area.setText(translator.produceCompositinHeader(piece, cantus, counterpoint));
				update_menus();
			}
		}
	}


	/***************************************************************************/


	class AddCounterpointAction extends AbstractAction
	// Add a new counterpoint to a cantus firmus. A dialog box will be brought up asking the user what
	// type of counterpoint he would like to add. The composition will be populated with arbitrary
	// but appropriately placed notes.
	{
		public AddCounterpointAction() {super("Add Counterpoint");}
		public void actionPerformed(ActionEvent event)
		{
			if (add_counterpoint_dialog == null)
				add_counterpoint_dialog = new AddCounterpointDialog(SplitPaneFrame.this);
			add_counterpoint_dialog.show();
			if (add_counterpoint_dialog.was_updated() == true) // take action if the OK button was pressed
			{
				table_model.getValuesFromTable(piece, cantus, counterpoint);
				clear_displays();
				piece = add_counterpoint_dialog.get_counterpoint(piece);
				cantus = piece.get_voice(0);
				counterpoint = piece.get_voice(1);
				table_model.importIntoTable(piece, cantus, counterpoint);
				MusicTranslator translator = new MusicTranslator();
				top_area.setText(translator.produceCompositinHeader(piece, cantus, counterpoint));
				update_menus();
			}
		}
	}


	/***************************************************************************/


	class ChangePreferencesAction extends AbstractAction
	// Display a dialog box allowing the user to change which rules are applied when analyzing pieces.
	// The selections made are saved to disk so that they will persist when the program is reopened.
	{
		public ChangePreferencesAction() {super("Preferences");}
		public void actionPerformed(ActionEvent event)
		{
			preferences_dialog.show();
		}
	}


	/***************************************************************************/


	class AnalyzeCompositionAction extends AbstractAction
	// Analyze the entire composition for technical mistakes using the rules selected in
	// the preferences dialog box. Rules from the AnalyzerHorizontalCantus
	// and the appropriate Analyzer classes for the type of species are applied. Display
	// the results of the analysis in bottom_area.
	{
		public AnalyzeCompositionAction() {super("Analyze Composition");}
		public void actionPerformed(ActionEvent event)
		{
			// Updatae piece by by getting the composition data from the table 
			table_model.getValuesFromTable(piece, cantus, counterpoint);
			
			// Clear the error output area
			bottom_area.setText("");

			// Perform an analysis of the cantus using rules from the AnalyzerHorizontalCantus class
			perform_cantus_analysis();
			bottom_area.append("\n\n\n");

			// Perform an analysis of the counterpoint using rules from the appropriate Analyzer classes
			perform_counterpoint_analysis();
		}
	}


	/***************************************************************************/


	class AnalyzeCounterpointAction extends AbstractAction
	// Analyze the counterpoint portion of the composition for technical mistakes using
	// the rules selected in the preferences dialog box. Rules from 
	//  the appropriate Analyzer class for the type of species are applied. Display
	// the results of the analysis in bottom_area.
	{
		public AnalyzeCounterpointAction() {super("Analyze Counterpoint");}
		public void actionPerformed(ActionEvent event)
		{
			// Updatae piece by by getting the composition data from the table 
			table_model.getValuesFromTable(piece, cantus, counterpoint);

			// Clear the error output area
			bottom_area.setText("");

			// Perform an analysis of the counterpoint using rules from the appropriate Analyzer classes
			perform_counterpoint_analysis();
		}
	}


	/***************************************************************************/


	class AnalyzeCantusAction extends AbstractAction
	// Analyze the cantus portion of the composition for technical mistakes using the
	// rules selected in the preferences dialog box. Rules from the
	// AnalyzerHorizontalCantus class are applied. Display the results of the
	// analysis in bottom_area.
	{
		public AnalyzeCantusAction() {super("Analyze Cantus");}
		public void actionPerformed(ActionEvent event)
		{
			// Updatae piece by by getting the composition data from the table 
			table_model.getValuesFromTable(piece, cantus, counterpoint);
			
			// Clear the error output area
			bottom_area.setText("");

			// Perform an analysis of the cantus using rules from the AnalyzerHorizontalCantus class
			perform_cantus_analysis();
		}
	}


	/***************************************************************************/


	class GenerateCantusAction extends AbstractAction
	// Randomly generate a cantus using the rules selected in the preferences dialog box.
	// This may take a few moments to do.
	{
		public GenerateCantusAction() {super("Generate Cantus");}
		public void actionPerformed(ActionEvent event)
		{
			if (check_save() == true)
			{
				clear_displays();
				filename = null;
				bottom_area.setText("GENERATING A CANTUS FIRMUS...\n\n");
				GeneratorCantus cantus_generator = new GeneratorCantus();
				piece = cantus_generator.get_random_cantus(preferences_dialog.get_cantus_rules());
				cantus = piece.get_voice(0);
				counterpoint = null;
				table_model.importIntoTable(piece, cantus, counterpoint);
				MusicTranslator translator = new MusicTranslator();
				top_area.setText(translator.produceCompositinHeader(piece, cantus, counterpoint));
				update_menus();
				bottom_area.append("GENERATION COMPLETE.\n\n");
			}
		}
	}


	/***************************************************************************/


	class HelpAction extends AbstractAction
	// Display a dialog box containing the on-line help
	{
		public HelpAction() {super("Help Topics");}
		public void actionPerformed(ActionEvent event)
		{
			if (help_dialog == null)
				help_dialog = new HelpDialog();
			help_dialog.show();
		}
	}


	/***************************************************************************/


	class AboutAction extends AbstractAction
	// Display a dialog box with program and author information
	{
		public AboutAction() {super("About ScoreChecker");}
		public void actionPerformed(ActionEvent event)
		{
			if (about_dialog == null)
				about_dialog = new AboutDialog(SplitPaneFrame.this);
			about_dialog.show();
		}
	}
}